MODULE drawatoms

implicit none

CONTAINS 

   subroutine draw_atoms(atom,legm,icon)
!
!  Interfaccia per passare da JAV alla vecchia grafica e viceversa
!
   USE Atomi
   USE atom_type_util
   USE ilmviewmod
   USE unit_cell
   USE General, only: cell
   USE connect_mod
   type(atom_type), dimension(:), allocatable, intent(in)           :: atom
   type(bond_type), dimension(:), allocatable, intent(in), optional :: legm    
   integer, dimension(:), intent(in), optional                      :: icon
   type(bond_type), dimension(:), allocatable                       :: legmi
   type(cell_type)                                                  :: cellt
!
   if (present(icon)) then
       cellt = set_cell_type(cell)
       call iconn_to_legm(icon,legmi,atom,cellt%get_g())
       call ilmview_draw(atom,legmi)
   else
       call ilmview_draw(atom,legm)
   endif
!
   end subroutine draw_atoms

!-----------------------------------------------------------------------

   subroutine draw_xyzs(conn)
   USE Atomi
   USE ilmviewmod
   USE atom_type_util
   USE molpnew
   USE variables, only:lconn
   USE General, only: cell,spacegr,celem
   USE unit_cell
   type(atom_type), dimension(:), allocatable :: atom
   logical, optional, intent(in)              :: conn
!
   call xyz_to_atom(atom,xyzs(:3,:nats),bfac,occr=xyzs(5,:),occh=occpch,nz=nz,lab=aLab,elem=celem)
   if (present(conn)) then
       if (conn) then
           call create_connectivity(atom,lconn,set_cell_type(cell),spacegr)
       endif
   endif
   call ilmview_draw(atom,lconn)
!
   end subroutine draw_xyzs

!-------------------------------------------------------------------------------------------------
!corr
!corr   subroutine importa0_mpi(filename,ier,kadd)
!corr   USE mpi
!corr   character(len=*), intent(in) :: filename
!corr   integer, intent(out)         :: ier
!corr   logical, intent(in)          :: kadd
!corr   integer                      :: proc_id, mpi_err, np
!corr   integer                      :: file_free
!corr   integer                      :: status(MPI_STATUS_SIZE)
!corr
!corr   call MPI_Comm_size(MPI_COMM_WORLD, np, mpi_err)
!corr   call MPI_Comm_rank(MPI_COMM_WORLD, proc_id, mpi_err)
!corr
!corr   if (proc_id == 0) then
!corr       file_free = 1
!corr   else
!corr       call MPI_Recv(file_free,1,MPI_INTEGER, proc_id-1,1,MPI_COMM_WORLD,status,mpi_err)
!corr   endif
!corr   if (file_free == 1) call importa0(filename,ier,kadd)
!corr   if (proc_id /= np-1) call MPI_Send(file_free,1,MPI_INTEGER,proc_id+1,1,MPI_COMM_WORLD,mpi_err)
!corr
!corr   end subroutine importa0_mpi
!corr
!-------------------------------------------------------------------------------------------------

   subroutine importa0(strdir,atom,legm,ier,kadd)
!
!  Import model parsing string from command '%fragment'
!
   USE errormod
   USE general, only:StructureName,celem,cell,spacegr,neutro
   USE molcom,  only:jscreen
   USE importexport
   USE connect_mod
   USE ilmviewmod
   USE exportfiles
   USE variables
   USE atom_type_util
   USE gen_frm
   USE unit_cell
   USE elements
   character(len=*), intent(in)                              :: strdir
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm
   integer, intent(out)                                      :: ier
   logical, intent(in)                                       :: kadd
   type(error_type)                                          :: err
   type(element_type), dimension(:), allocatable             :: cform
   integer                                                   :: codef
   integer                                                   :: i,ierf
   real                                                      :: dist
   logical                                                   :: kadd1
   type(cell_type)                                           :: cellt
   character(len=len_trim(strdir))                           :: smiles_string
!
   ier = 0
   call parse_fragment_string(strdir,codef,cform,dist,smiles_string,err)
   if (.not.err%signal) then
       cellt = set_cell_type(cell)
       select case(codef)
         case (TETRA,OCTA,SQUARE,CUBE,TRIG,APRISM_TETRA,PRISM_TRIGONAL,ICOSAHEDRON)
           call make_fragment(atom,legm,cellt,spacegr,celem,codef,0,-cform(1)%z,-cform(2)%z,kadd,dist)
         case (ISOLATED)
           kadd1 = kadd
           do i=1,numelem(cform)
              if (i > 1) kadd1 = .true.  ! add next elements
              call make_fragment(atom,legm,cellt,spacegr,celem,ISOLATED,0,-cform(i)%z,nint(cform(i)%nw),kadd1,dist)
           enddo
         case (SMILES) 
           call get_smiles(smiles_string,atom,legm,celem,spacegr%nsymop,neutro,cellt,kadd,err)
           if (err%signal) call err%print()
         case (0) 
           call import_from_file(strdir,atom,legm,kadd,ierf)
           if (ierf /= 0) call err%set()
       end select
   else
       call err%print()
   endif
!
   if (jscreen > 0) call ilmview_open(StructureName) ! this open empty windows in case of error
   if (.not.err%signal) then
       if(jscreen>0) then
          call ilmview_draw(atom,legm)
       endif
       call update_model_and_export(atom,legm, 1)
   else
       if (jscreen == 0) ier = -1
   endif
!
   end subroutine importa0

END MODULE drawatoms
