module expo_main
   implicit none 

   type param_options_type
        integer            :: nogui
        integer            :: auto
        integer            :: indexing
        real               :: wavel
   end type param_options_type

contains                    

   subroutine mainloop(file1,ier,param_opt)
   USE ParametriMappa
   USE General
   USE Molcom
   USE PhaseModule
   USE fileutil
   USE filereading
   USE ilmviewmod
   USE importexport
   USE fileutil
   USE comandiexpo
   USE progproject
   USE variables, only:lconn,atm
   USE strutil
   USE indexing
   USE gen_frm
   character(len=*), intent(in)                   :: file1
   integer, intent(out)                           :: ier
   type(param_options_type), intent(in), optional :: param_opt
   integer                                        :: i, newf, iEnd
   integer                                        :: active_file
   integer, dimension(3)                          :: indx
   character(len=3)                               :: sval
   real                                           :: wavelength
!
   ier = 0
   active_file = POW_FILE
   if (ifFirstTime.eq.1.and.jscreen.ne.0) then
       ifFirstTime = 0
       do i=1,100
          ListaOK(i) = 0
       enddo
   endif
!
!  check if is a structure file
   if (code_fragfilen(file1) > 0) then
       StructureName = file_rem_ext(file_get_name(file1))
       if (jscreen > 0) then
           call wmenu_set_state('Info'//char(0),1)
           call ilmview_open(file1)
           call jav_import_structure(trim(file1)//char(0),0,1)
           call jav_set_name(file_get_name(file1), 1)
       else
           call import_structure(file1,0,atm,lconn,.false.,ier)
       endif
       return
   endif
!                                     
!  check if is a powder pattern file
   if (filetype_from_filename(file1) /= 0) then
!
!      Check for wavelength
       if (param_opt%wavel <= 0.0 .and. param_opt%indexing > 0) then
           wavelength = DEF_WAVE        
       else
           wavelength = param_opt%wavel
       endif
       
       call open_pattern(file1,wavelength,-1,ier)
!
!      check for indexing
       if (param_opt%indexing > 0) then
           indx(:) = 0
           write(sval,'(i3.3)')param_opt%indexing
           if (sval(1:1) == '1') indx(DICVOL06_PROG) = 1
           if (sval(2:2) == '1') indx(MCMAILLE_PROG) = 1
           if (sval(3:3) == '1') indx(NTREOR09_PROG) = 1
           call data_for_index()
           call finish_index(indx_set, indx)
       endif
       return
   endif
!
!  check if is a expo project file
   if (s_eqi(get_extension(file1),PRJEXT)) then
       ier = readproject(file1)
       return
   endif
!
   call spoolr(active_file,newf,file1,ier)
!
   ipdir = 0
   if (ListaOK(6).eq.1) then
       kuser3 = 3 * ( np(1) + 2 ) * ( np(3) + 2 )
   endif
!
   if (ier.eq.0) then
       call scr2()
       call gescom(active_file,iEnd,ier)
   endif
   if (ier /= 0) then
       iEnd = 1
   endif
   if (iEnd.eq.0) then
       if (ier.eq.99) then
           ier = 0
       endif
   endif
!
   return
   end subroutine mainloop

!-----------------------------------------------------------------------

   subroutine CheckPath(ier)
   USE General
   USE Molcom, only: kscreen
   USE fileutil
   USE spginfom
   USE strutil, only: lower
   USE iso_fortran_env, only: ERROR_UNIT
!
!-- add path to data files and check if they exist
!
   integer, intent(out)                    :: ier
   integer                                 :: i, nExitCode, Missing
   integer, parameter                      :: NCFGFILE = 4    ! Change this number if add/remove cfgfiles
   character(len=256), dimension(NCFGFILE) :: cfgfile
   character(len=len_trim(package_name))   :: progname
!
   ier = 0
!
!-- add extensions to the package name
!
   progname = lower(package_name)
   ffile  = progname //  trim(ffile)
   iasun  = progname // trim(iasun)
   extgroup = progname //  trim(extgroup)
!
!-- add path to data files
!-- exepath path to "sir2002.exe directory" //  "sirdir"
!                    directory where read-only files are stored
!-- cfgpath path to .cfg & .pal files
!                    directory where read/write files are stored
!
   if (len_trim(exepath) > 0) then
       ffile = trim(exepath) // trim(ffile)
       iasun  = trim(exepath) // trim(iasun)
       extgroup = trim(exepath) //  trim(extgroup)
       syminfo = trim(exepath) //  trim(syminfo)
   endif
!
   cfgfile = (/ffile,iasun,extgroup,syminfo/)
!
!  Check the existence of configuration files
   Missing = 0
   do i=1,NCFGFILE
      if (.not.file_exist(cfgfile(i))) then
          if (Missing == 0) then
             write(ERROR_UNIT,'(//," ***** ERROR *****",/," Missing files:")')
          endif
          Missing = Missing + 1
          write(ERROR_UNIT,'(a)') trim(cfgfile(i))
          if (kscreen == 1) then      
              call MsgWinErr('ERROR', ' File '//trim(cfgfile(i))//' does not exist.', ERR_WINDOW, nExitCode)
          else
              write(lo,'(//a)')'  *** WARNING ***  File '//trim(cfgfile(i))//' does not exist.'
          endif
      endif
   enddo
!
   if (Missing > 0) then
       ier = -2
   else
       call spg_set_symmetry_file(syminfo)  
   endif
!
   return
   end subroutine CheckPath

end module expo_main
! ----------------------------------------------------------------------
