MODULE proginterface

!
! Questo modulo contiene funzioni di interfaccia tra vecchia gestione e nuova gestione
!

!
!                 Routine di interfaccia per la gestione del contenuto
!S nk_to_elem(elem)                      Trasferisco in elem le informazioni sul contenuto
!S elem_to_nk(elem)                      Prendo le informazioni sul contenuto da elem
!S fillcounts()                          Prendo informazioni sui conteggi da expo
!
implicit none

CONTAINS

   subroutine nk_to_elem(elem)
!
!  Trasferisco in elem le informazioni sul contenuto
!
   USE elements
   USE general
   type(element_type), dimension(:), intent(out), allocatable :: elem
   integer                                       :: i
!
   call reallocate_elem(elem,nk)
   do i=1,nk
      elem(i)%z = no(i)
      elem(i)%c_radius = radii(i,1)
      elem(i)%w_radius = radii(i,2)
      select case (neutro)
         case (0)
          elem(i)%ax(:) = al(:,i)
          elem(i)%bx(:) = bs(:,i)
          elem(i)%cx = cl(i)
         case (1)
          elem(i)%fact = cl(i)
      end select
      elem(i)%zeff = zeff(i)
      elem(i)%nw = nw(i)
      elem(i)%radtype = neutro
      elem(i)%ptab = pxen_from_specie(elem(i)%lab)
      elem(i)%name = eleminfo(elem(i)%ptab)%name
   enddo
   call elem_set_scatt(elem)
!
   end subroutine nk_to_elem

!---------------------------------------------------------------------      

   subroutine elem_to_nk(elem,norm)
!
!  Prendo le informazioni sul contenuto da elem
!
   USE general
   USE elements
   USE strutil
   type(element_type), dimension(:), allocatable, intent(in) :: elem
   logical, intent(in), optional                :: norm  
   integer                                      :: i, nel
   character(len=4)                             :: spec
!corr   integer                                      :: lenspec   !!!!!, lennome
!
   nel = numelem(elem)
   if (nel == 0) return
   nk = min(nel,NMAXSPECIE)
!corr   lenspec = len(Species(1))  ! to remove warning with gfortran
   do i=1,nk
      no(i) = elem(i)%z
!corr      Species(i) = elem(i)%lab(1:lenspec)
              spec = lower(elem(i)%lab(1:4))
              read(spec,'(a4)')nalf(i)
      !read(Species(i),'(a4)') nalf(i)
      radii(i,1) = elem(i)%c_radius
      radii(i,2) = elem(i)%w_radius
      !neutro = elem(i)%radtype
      select case (neutro)
         case (0)   ! Raggi X
           al(:,i) = elem(i)%ax
           bs(:,i) = elem(i)%bx
           cl(i) = elem(i)%cx 
         case (1)   ! Neutroni
           al(:,i) = 0
           bs(:,i) = 0
           cl(i) = elem(i)%fact 
         case (2)   ! Elettroni  -- da controllare -> Gianluca
           al(:,i) = elem(i)%ae
           bs(:,i) = elem(i)%be
           cl(i) = elem(i)%ce 
      end select
      zeff(i) = elem(i)%zeff
      nw(i) = nint(elem(i)%nw)
   enddo
!
!  norm=.true. e' necessario se nalf viene passato alla yfrm03
   if (present(norm)) then
       if (norm) then
           do i=1,nk
              spec = lower(celem(i)%lab(1:4))
              read(spec,'(a4)')nalf(i)
           enddo
       endif
   endif
!
   end subroutine elem_to_nk

!-------------------------------------------------------------------

   subroutine fillcounts()
!
!  Prendo informazioni sui conteggi da expo
!
   USE Conteggi
   USE counts
   USE progtype, only:ALLOCA
   USE patternref, only:thmin,thmax
   integer :: i
!
   call alloca_counts(ALLOCA,npunti)
   do i=1,npunti
      ttetacount(i) = theta_int(i,1)
      ycountoss(i) = theta_int(i,2)
      ttetaczero(i) = ttetacount(i)
      backgr(i) = back_int(i,1)
      ycounttot(i) = theta_int(i,3)
   enddo
   thmin = ttetaczero(1)
   thmax = ttetaczero(npunti)
!
   end subroutine fillcounts

!-------------------------------------------------------------------

   subroutine fillref(refl)
!
!  Scarica refl nelle variabili expo
!
   USE General  , only : nrefl,dtor,alambda
   USE Riflessi, only : Mult,IndHKL,fxsav,WHM,SetFob,TThet,ipub,iphaz
   USE xdataMod, only : clp
   USE reflection_type_util
!
   implicit none
   type(reflection_type), dimension(:), allocatable, intent(inout) :: refl
   integer                                                         :: i
!
   call new_reflections(refl,nrefl)
   do i=1,nrefl
      refl(i)%hkl = IndHKL(i,1:3)                           ! indici h,k,l
      refl(i)%m   = Mult(i)                                 ! molteplicita'
      refl(i)%tthd = TThet(i)                               ! 2-theta in gradi
      refl(i)%slaq = (sin(0.5*TThet(i)*dtor)/alambda)**2    ! (sen(theta)/lambda)**2
      refl(i)%fo  = sqrt(SetFob(i,4))                       ! fattori di struttura osservati
      refl(i)%fwhm = WHM(i)                                 ! FWHM
      refl(i)%lp = clp(i)                                   ! LP
      refl(i)%ph = iphaz(i)                                 ! fasi calcolate
      refl(i)%fv   = fxsav(i)                               ! fattori di struttura veri
      refl(i)%phv = ipub(i)                                 ! fasi vere
   enddo
!
   end subroutine fillref

!---------------------------------------------------------------------------------------

   subroutine ref_to_expo(refl)
   USE General  , only : nrefl
   USE Riflessi, only : Mult,IndHKL,fxsav,WHM,SetFob,TThet,ipub,iphaz
   USE xdataMod, only : clp
   USE reflection_type_util
   type(reflection_type), dimension(:), allocatable, intent(in) :: refl
   integer                                                      :: i
   nrefl = numrefl(refl)
   call AllocaRiflessi(2)
   call AllocaRiflessi(1)
   do i=1,nrefl
      IndHKL(i,1:3) = refl(i)%hkl            ! indici h,k,l
      Mult(i)  = refl(i)%m                   ! molteplicit�
      TThet(i) = refl(i)%tthd                ! 2-theta in gradi
      SetFob(i,4) = refl(i)%fo*refl(i)%fo    ! fattori di struttura osservati
      WHM(i)  = refl(i)%fwhm                 ! FWHM
      clp(i) = refl(i)%lp                    ! LP
      iphaz(i) = refl(i)%ph                  ! fasi calcolate
      fxsav(i) = refl(i)%fv                  ! fattori di struttura veri
      ipub(i) = refl(i)%phv                  ! fasi vere
   enddo
   end subroutine ref_to_expo

!---------------------------------------------------------------------------------------

   subroutine examing(ref1,ref2)
!
!  Calcola errore di phase (erore) e shift di fase (shiftexam)
!
   USE reflection_type_util
   type(reflection_type), dimension(:), intent(in) :: ref1,ref2
   integer                                         :: nrefex
   integer                                         :: i,j
   integer, dimension(3,size(ref1))                :: khl
!
   nrefex = size(ref1)
   do i=1,nrefex
      khl(:,i) = ref1(i)%hkl
   enddo
   call examin(1,khl,ref1(:)%ph,ref2(:)%ph,nrefex,0,(/(1.0,j=1,nrefex)/),1)
!
   end subroutine examing

!-------------------------------------------------------------------

   subroutine write_phase_error(ref)
   USE reflection_type_util
   USE General, only: lo
   type(reflection_type), dimension(:), intent(in) :: ref
   integer                                         :: nrefex
   integer                                         :: i
   nrefex = size(ref)
   write(lo,'(" Phase error:")')
   call examin(1,(/(ref(i)%hkl,i=1,nrefex)/),ref%phv,ref%ph,nrefex,0,(/(1.0,i=1,nrefex)/),1)
   end subroutine write_phase_error

!-------------------------------------------------------------------

   subroutine generate_reflections(tthmin,tthmax,lambda)  ! ,err)
   USE patternref, only:thmin,thmax
   USE General, only:alambda
   USE Conteggi, only:npunti
   real, intent(in), optional :: tthmin,tthmax
   real, intent(in), optional :: lambda
   integer                    :: iflag
!
   if (present(tthmin) .and. present(tthmax)) then
       thmin = tthmin
       thmax = tthmax
   else
       if (npunti == 0) then
           thmin=1; thmax=80   !!!!40;
       endif
   endif
!
   if (present(lambda)) then
       alambda = lambda
   else
       if (alambda == 0.0) alambda = 1.54056
   endif
!
   call gener(iflag,0)
!
   end subroutine generate_reflections

END MODULE proginterface
