MODULE importexport

USE progtype

implicit none

integer, parameter :: TETRA=1,OCTA=2,SQUARE=3,CUBE=4,TRIG=5, APRISM_TETRA=6,&
                      PRISM_TRIGONAL=7,ICOSAHEDRON=8,ISOLATED=9,SMILES=10

CONTAINS

   subroutine import_structure(filename,ftype,atom,legm,kadd,ier,at_info,nsec,sec_struct)
   USE atom_basic
   USE pdb_frm
   USE general, only: celem, nrefl, alambda, neutro, spacegr, cell, ifAutomatic, structurename
   USE proginterface
   USE Conteggi, only:npunti
   USE Patternref, only:thmin,thmax
   USE enable_amb
   USE view
   USE spginfom
   USE sinterface
   USE atom_type_util
   USE variables, only: atm
   USE Molcom, only:jscreen
   USE gen_frm
   USE connect_mod
             USE unit_cell
                     USE errormod
                     USE constraints
                type(constraint_type), dimension(:), allocatable :: constrv
              !USE pdfcalc
   character(len=*), intent(in)                              :: filename
   integer, intent(in)                                       :: ftype ! if 0 the file type is set from filename
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm
   logical, intent(in)                                       :: kadd
   integer                                                   :: ier    !!!!,ierspace
   type(pdb_atom_info_type), dimension(:), allocatable, optional, intent(out) :: at_info   ! additional info for pdb file
   integer, intent(out), optional :: nsec
   type(secondary_struct_type), dimension(:), allocatable, optional, intent(out) :: sec_struct ! secondary structure from pdb
   integer :: filetype
   logical has_symmetry, gui
integer, dimension(:), allocatable :: vat,vpar
integer :: natom,kpar
logical :: etype
type(error_type) :: err
          !type(atom_type), dimension(:), allocatable :: atoms

   !spacegr%from_user = is_space_defined_list()
   if (kadd) then
       !has_symmetry = is_cell_defined()  ! in 'add' check only for cell
       has_symmetry = numatomtype(atm) > 0
   else
       has_symmetry = is_cell_defined_list() ! in 'import' check for cell in list
   endif
   gui = (jscreen > 0 .and. ifAutomatic == 0)
   call import_crystal(filename,ftype,atom,legm,celem,cell,spacegr,has_symmetry,neutro,gui,ier,at_info,nsec,sec_struct)
   if (ier == 0) then
       if (.not.has_symmetry) then
            call set_cell(cell(1:6))
            call set_spaceg_new(spacegr) 
       endif
       call elem_to_nk(celem)
       if (npunti > 0) call abilita_tasti('structure',1)
!
!      Generate reflections for cif and pdb files
       filetype = code_fragfilen(filename)
       if (npunti > 0 .and. nrefl == 0 .and. (filetype == CIF_FILE .or. filetype == PDB_FILE)) then
           call generate_reflections(thmin,thmax,alambda)
           call vedinew(1)
       endif
!
!      save distances for database in a file
       !call print_connect(atom%specie(),legm=legm,csv=.true.,csvstring=structurename,filename=trim(structurename)//'.csv')
       !call export_for_db(atom,legm,set_cell_type(cell),spacegr,celem,structurename)
   endif
 !!!!!!!!!!!!!test
       !corr   call equation_from_string('occ(c1)+occ(c2)+B(S1)=1',atom,celem,vat,vpar,etype,ier)
       !corr     write(0,*)'VAT=',atom(vat(:))%lab
       !corr     write(0,*)'VPAR=',vpar
       !corr     write(0,*)'TYPE=',etype
            !call make_test(atom,legm,celem)
!!!!!!!!!!!!!test
   end subroutine import_structure
 
!---------------------------------------------------------------------      

   subroutine import_from_smiles(smiles_string, atom, legm, elem, nsymtot, neutro, cell, err)
   USE atom_type_util
   USE OBmodule
   USE elements
   USE errormod
   USE proginterface
   USE unit_cell
   character(len=*), intent(in)                                 :: smiles_string
   type(atom_type), dimension(:), allocatable, intent(out)      :: atom
   type(bond_type), dimension(:), allocatable, intent(out)      :: legm
   type(element_type), dimension(:), allocatable, intent(inout) :: elem  
   integer, intent(in)                                          :: nsymtot, neutro
   type(cell_type), intent(in)                                  :: cell
   type(error_type), intent(out)                                :: err
!
   call OBConversionFromSmiles(smiles_string,atom,legm,err)
   if (err%signal) return
!
   call update_elem(elem,atom(:)%ptab,neutro,nsymtot)  ! update content
   call elem_to_nk(elem)
   call specie_from_ptab(atom,elem)
   call atom_string(atom,elem)                         ! label
   call set_biso(atom)                                 ! adp
   call set_intensity(atom)                            ! int.
   call cart_to_frac(atom,cell)
!
   end subroutine import_from_smiles

!---------------------------------------------------------------------      

   subroutine get_smiles(smiles_string,atom,legm,elem,nsymtot,neutro,cell,add,err)
   USE atom_type_util
   USE elements
   USE errormod
   USE unit_cell
   USE connect_mod
   character(len=*), intent(in)                                 :: smiles_string
   type(atom_type), dimension(:), allocatable, intent(inout)    :: atom
   type(bond_type), dimension(:), allocatable, intent(inout)    :: legm
   type(element_type), dimension(:), allocatable, intent(inout) :: elem  
   integer, intent(in)                                          :: nsymtot, neutro
   type(cell_type), intent(in)                                  :: cell
   logical, intent(in)                                          :: add
   type(error_type), intent(out)                                :: err
   type(atom_type), dimension(:), allocatable                   :: atom1
   type(bond_type), dimension(:), allocatable                   :: legm1
   integer                                                      :: natnew,natcurr
!
   call import_from_smiles(smiles_string,atom1,legm1,elem,nsymtot,neutro,cell,err)
   if (err%signal) return
   if (add) then  ! add smiles to current model
       natcurr = numatomtype(atom)
       call add_atoms_to_list(atom,atom1,natnew,legm1=legm,legm2=legm1)
       call remove_duplicate_labels(atom)
   else
       call copia_atoms(atom,atom1)
       call copia_legm(legm,legm1)
   endif
!
   end subroutine get_smiles

!---------------------------------------------------------------------      

   subroutine make_test(atom,legm,elem)
   USE atom_type_util
   USE connect_mod
   USE elements
!corr   USE contacts
!corr   USE molpnew
!corr   USE hydrogens
!corr   USE polyhedra
!corr   USE centroids
!corr   USE unit_cell
!corr   USE General, only:cell,spacegr,celem,structurename
!corr   USE spginfom
!corr   USE crystal_frm
   USE bond_valence
!
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm
   type(element_type), dimension(:), allocatable, intent(in) :: elem
!  test bond valence
   type(bond_valence_type), dimension(size(elem),size(elem)) :: bvtab
   integer                                                   :: ier
   type(atom_type), dimension(:), allocatable                :: atoms
   type(bond_type), dimension(:), allocatable                :: legms
   type(bond_type), dimension(:), allocatable                :: legmHview,legmshort
   logical :: hbond,shbond,lpacking,lsymm
   integer, dimension(2) :: kexpand
!corr   type(polyhedron_type), dimension(:), allocatable      :: poly
   integer :: npoly
!corr   type(centroid_type), dimension(:), allocatable :: centr
   integer :: ncentr
!   type(torsion_type), dimension(:), allocatable :: tors
!   logical :: chang
!   type(angle_type), dimension(:), allocatable :: angle
!   integer, dimension(size(atom)) :: veta
!   integer :: nd
!
   if (numatomtype(atom) == 0) return
!
!  test bond valence
   call bv_compute_table(elem,bvtab)
!
!  test for crystal file
   !call write_crystal_file(trim(structurename)//'.d12',set_cell_type(cell),spacegr,atom,structurename)
!
!  test grow frag
   !call apply_symmetry(atom,atoms,legm,legms,infos,level=(/1.0,1.0,1.0/)-0.5,fitopt=2)
   !call grow_fragments(atom,atoms,legm,legms,infos,set_cell_type(cell),lsym=.true.,kexp=(/EXPAND_ALL,0/))
   !call grow_fragments(atom,atoms,legm,legms,infos,set_cell_type(cell),spacegr,lsym=.false.,kexp=(/EXPAND_ALL,0/))
   !call copia_atoms(atom,atoms)
   !call copia_legm(legm,legms)
!
!  test bond perception
!  call bond_type_perception(atom,legm)
!
!  test hydrogens
   !call remove_atoms_from_list(atom,atom%ptab,H_at,legm=legm)  !remove H
   !call add_hydrogens(atom,legm,celem,set_cell_type(cell),spacegr,ier=ier)
!
!  test change bond distance
   !do i=1,numbonds(legm)
   !   call change_bond_distance(atom,legm,8,4.5, matrice_metrica(cell))
   !   call change_bond_distance(atom,legm,11,4.5, matrice_metrica(cell))
   !enddo
!
!  test for change angle
   !call legm_to_ang(legm,atom,angle)
   !call change_bond_angle(3,angle,atom,legm,170.,set_cell_type(cell))
!
!  test csys
   !write(0,'(1x,a,a,1x,a)')'CRY SYS: ',trim(cry_sys(spacegr%csys_code)), trim(cry_sys(csys_from_cell(cell)))
!
!  test for change torsion
   !call legm_to_tors(legm,atom,tors)
   !call change_torsion_angle(tors(10),atom,legm,150.,set_cell_type(cell),chang)
   !return
!
!  test centroids
   !call find_centroids(atom,legm,centr,ncentr)
!
!  test contacts
   !hbond = .false.; shbond = .true.; lpacking = .false.
   !kexpand = [0,0]
   !call find_contacts(atom,legm,legmHview,legmshort,atoms,legms,lpacking,kexpand,hbond,shbond,set_cell_type(cell),spacegr)
   !call copia_atoms(atom,atoms)
   !call copia_legm(legm,legms)
!
!  test polyhedra
   !lsymm = .false.
   !call find_polyhedra_sym(atom,legm,set_cell_type(cell),spacegr,atoms,legms,poly,npoly,lsym=lsymm)
!
!  test valence-bond
!
   end subroutine make_test

!----------------------------------------------------------------------------------------------------------------      

   subroutine import_from_file(filename,atom,legm,add,ier)
   USE connect_mod
   USE atom_type_util
   USE gen_frm
   character(len=*), intent(in)                              :: filename
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm
   logical, intent(in)                                       :: add
   integer, intent(out)                                      :: ier
   type(atom_type), dimension(:), allocatable                :: atom1
   type(bond_type), dimension(:), allocatable                :: legm1
   integer                                                   :: itype
   integer                                                   :: natnew
!
   itype = code_fragfilen(filename)
   call import_structure(filename,itype,atom1,legm1,add,ier)
   if (ier /= 0) return
!
   if (add) then
       call add_atoms_to_list(atom,atom1,natnew,legm1=legm,legm2=legm1)
       call remove_duplicate_labels(atom)
   else
       call copia_atoms(atom,atom1)
       call copia_legm(legm,legm1)
   endif
!
   end subroutine import_from_file

!---------------------------------------------------------------------      
 
   subroutine parse_fragment_string(str,code,cform,dist,strf,err) 
!
!  Parse string associated to command '%fragment'
!
   USE strutil
   USE elements
   USE errormod
   USE connect_mod
   character(len=*), intent(in)                               :: str
   integer, intent(out)                                       :: code
   real, intent(out)                                          :: dist
   character(len=*), intent(out)                              :: strf
   type(error_type)                                           :: err
   type(element_type), dimension(:), allocatable, intent(out) :: cform
   character(len=len_trim(str))                               :: str1,word
   integer                                                    :: ier,nlen2
   logical                                                    :: poly
   integer                                                    :: spec1,spec2
!
   code = 0
   spec1 = 0
   spec2 = 0
   dist = 0.0
!
   poly = .false.
   str1 = trim(str)
   call cutst(str1,line2=word,nlong2=nlen2)
   if (nlen2 == 0) return
   select case(lower(word))
      case ('tetra')
         code = TETRA
         poly = .true.
      case ('octa')
         code = OCTA
         poly = .true.
      case ('square')
         code = SQUARE
         poly = .true.
      case ('cube')
         code = CUBE
         poly = .true.
      case ('trigonal')
         code = TRIG
         poly = .true.
      case ('prism_tetra')
         code = APRISM_TETRA
         poly = .true.
      case ('prism_trig')
         code = PRISM_TRIGONAL
         poly = .true.
      case ('icosa')
         code = ICOSAHEDRON
         poly = .true.
      case ('atoms')
         code = ISOLATED
         call read_chemformula(str1,ier,cform)
         if (ier /= 0) goto 10
      case ('smiles')
         code = SMILES
         strf = trim(str1)
   end select
!
   if (poly) then
       call cutst(str1,line2=word)
       spec1 = z_from_specie(word)
       if (spec1 == 0) goto 10
       call add_element(cform,spec1)
       call cutst(str1,line2=word)
       spec2 = z_from_specie(word)
       if (spec2 == 0) goto 10
       call add_element(cform,spec2)
       if (len_trim(str1) > 0) then
           call s_to_r(str1,dist,ier)
           if (ier /= 0) go to 10
       endif
       if (dist == 0.0) dist = bond_distanceZ(spec1,spec2)
   endif

   return

10 call err%set("Error reading directive: '"//trim(str)//"'")
!
   end subroutine parse_fragment_string

!---------------------------------------------------------------------      

   subroutine make_fragment(atom,legm,cell,spg,elem,itipo,kpos,kspec1i,kspec2i,kadd,bd)
   USE atom_type_util
   USE unit_cell
   USE spginfom
   USE connect_mod
   USE elements
   USE proginterface
   USE unit_cell
!
   type(atom_type), dimension(:), allocatable, intent(inout)    :: atom   
   type(bond_type), dimension(:), allocatable, intent(inout)    :: legm
   type(cell_type), intent(in)                                  :: cell
   type(spaceg_type), intent(in)                                :: spg
   type(element_type), dimension(:), allocatable, intent(inout) :: elem
   integer, intent(in)                                          :: itipo         ! tipo di frammento
   integer, intent(in)                                          :: kpos          ! posizione del frammento
   integer, intent(in)                                          :: kspec1i,kspec2i ! specie nel frammento
   logical                                                      :: kadd          ! true se il frammento va aggiunto in atom
   real, intent(in)                                             :: bd            ! distanza 
   type(atom_type), allocatable, dimension(:)                   :: atom1
   integer                                                      :: i
   real, dimension(3)                                           :: xpos
   integer                                                      :: kkpos
   logical                                                      :: centr
   integer                                                      :: n1
   integer                                                      :: nat
   integer                                                      :: nadd,nlegadd
   real, dimension(3)                                           :: xrand
   logical                                                      :: addnew1,addnew2
   integer                                                      :: kspec1,kspec2
   integer                                                      :: newsize
!
   nat = numatomtype(atom)
!
!  If kspec is negative add new elements to the content (Z number = -kspec)
   if (kspec1i < 0 .or. kspec2i < 0) then
       addnew1 = .false.
       if (kspec1i < 0) then
!
!          set how many atoms to add
           select case (itipo)
               case (TETRA,SQUARE)     
                 newsize = 5
               case(OCTA,PRISM_TRIGONAL)
                 newsize = 7
               case(CUBE,APRISM_TETRA)
                 newsize = 9
               case(TRIG)
                 newsize = 4
               case(ICOSAHEDRON)
                 newsize = 13
               case(ISOLATED)
                 newsize = kspec2i
           end select
           if (kpos > 0 .and. itipo /= ISOLATED) newsize = newsize - 1
!
           call add_element(elem,abs(kspec1i),nw=newsize*spg%nsymop,add=addnew1)
           kspec1 = is_element(elem,abs(kspec1i)) 
       endif
       addnew2 = .false.
       if (kspec2i < 0) then
           call add_element(elem,abs(kspec2i),nw=spg%nsymop,add=addnew1)
           kspec2 = is_element(elem,abs(kspec2i))  
       endif
       if (addnew1 .or. addnew2) call elem_to_nk(elem)
   endif
   if (kspec1i > 0) kspec1 = kspec1i
   if (kspec2i > 0) kspec2 = kspec2i
!
!  Posizione del frammento
   if (kpos == 0) then
       call random_number(xpos)  ! in posizione random
       kkpos = -1
       centr = .true.
   else
       xpos = atom(kpos)%xc     ! nella posizione dell'atom kpos
       kkpos = kpos - nat      ! serve ad allineare la connettivita
       centr = .false.
   endif
!
!  crea il frammento
   select case(itipo)
       case(TETRA)              ! Tetrahedron
         call make_tetrahedron(atom1,bd,cell,xpos=xpos,kcentr=kkpos,centro=centr)

       case(OCTA)               ! Octahedron
         call make_octahedron(atom1,bd,cell,xpos=xpos,kcentr=kkpos,centro=centr)

       case(SQUARE)             ! Square Plane
         call make_square(atom1,bd,cell,xpos=xpos,kcentr=kkpos,centro=centr)

       case(ISOLATED)           ! Isolated Atoms
         if (kadd) then
             n1 = nat + 1
             nat = kspec2 + nat
         else
             n1 = 1
             nat = kspec2
             call clear_bonds(legm)
         endif
         call resize_atoms(atom,nat)
         do i=n1,nat
            call random_number(atom(i)%xc)
            call atom(i)%set_specie_from_el(kspec1,elem)
         enddo
 
       case (CUBE)                ! Cube
         call make_cube(atom1,bd,cell,xpos)

       case (TRIG)                ! Trigonale Plane
         call make_trigonal(atom1,bd,cell,xpos)

       case (APRISM_TETRA)        ! Antiprism Tetragonal
         call make_anti_prism_tetragonal(atom1,bd,cell,xpos)

       case (PRISM_TRIGONAL)      ! Prism Trigonal
         call make_prism_trigonal(atom1,bd,cell,xpos)

       case (ICOSAHEDRON)         ! Icosahedron
         call make_icosahedron(atom1,bd,cell,xpos)

   end select
!
!  assegna nz
   if (itipo /= ISOLATED) then
       if (kpos == 0) then
           !atom1(1)%nz = kspec1*nzConst
           call atom1(1)%set_specie_from_el(kspec1,elem)
           !atom1(2:)%nz = kspec2*nzConst
           call specie_from_el(atom1,kspec2,elem,2)
       else
           !atom1(1:)%nz = kspec2*nzConst
           call specie_from_el(atom1,kspec2,elem)
       endif
!
!      combina i 2 modelli aggiungendo il nuovo frammento in fondo alla lista
       if (kadd) then
           nadd = size(atom1)   ! numero atomi da aggiungere
!
!          Applica una rotazione random intorno al baricentro per kpos/=0 
!          In questo modo frammenti centrati sullo stesso atomo (kpos/=0) non vanno in sovrapposizione
           if (itipo /= 4 .and. kpos /=0) then
               call random_number(xrand)
               call rand_rotate_atoms(atom1,xrand,cell)
           endif
!
           call add_atoms_to_list(atom,atom1,nat)
           n1 = nat - nadd + 1  ! puntatore al primo atomo aggiunto
           if (kpos == 0) then
               nlegadd = nadd - 1   ! numero legami da aggiungere
               call add_bonds(legm,atom,(/(n1,i=1,nlegadd)/),(/(i+n1,i=1,nlegadd)/),cell)
           else
               nlegadd = nadd       ! numero legami da aggiungere
               call add_bonds(legm,atom,(/(kpos,i=1,nlegadd)/),(/(i+n1-1,i=1,nlegadd)/),cell)
           endif
       else
           call copia_atoms(atom,atom1)
           call clear_bonds(legm)
           nat = size(atom)
           n1 = 1
           call add_bonds(legm,atom,(/(n1,i=1,nat-1)/),(/(i+n1,i=1,nat-1)/),cell)
       endif
   endif
!
!  Calcola occupanza cristallog. senza modificare le coordinate     
   call calcola_occ(atom(n1:nat),spg,cell,modcoor=.false.)
   atom(n1:nat)%och = 1.0  ! inizializza occ. chimica
!
   call set_biso(atom(n1:nat))
!
!  Assegna intensita in base a Z
   call set_intensity(atom(n1:nat))
!          
   call atom_string(atom,elem,vet=(/(i,i=n1,nat)/))
!
   end subroutine make_fragment

!---------------------------------------------------------------------      
   
   subroutine export_structure(filename,atom,legm,ftype,comm,iscell,spg)
   USE General, only:cell,errcell,StructureName,nk,alambda,celem,package_alt_name,neutro,spacegr
   USE errormod
   USE connect_mod
   USE proginterface
   USE elements
   USE sinterface
   USE spginfom
   USE cif_frm
   USE pdb_frm
   USE shelx_frm
   USE OBmodule
   USE unit_cell
   USE atom_type_util
   USE nwchem_frm
   USE gamess_frm
   USE abinit_frm
   USE xyz_frm
   USE zmt_frm
   USE fra_frm
   USE mop_frm
   USE gen_frm
   USE crystal_frm
   USE povray_frm
   character(len=*), intent(in)                                    :: filename
   type(atom_type), dimension(:), intent(in), allocatable          :: atom
   type(bond_type), dimension(:), allocatable, intent(inout),optional :: legm
   integer, intent(in)                                             :: ftype
   character(len=*), intent(in), optional                          :: comm
   logical, intent(in), optional                                   :: iscell
   type(spaceg_type), optional                                     :: spg
   type(error_type)                                                :: err_exp
   real, dimension(nk,3)                                           :: cols
   real, dimension(3)                                              :: colcell
   real, dimension(2)                                              :: vfact
   real, dimension(3,3)                                            :: bmat
   type(spaceg_type)                                               :: spaceg
   integer                                                         :: filetype
   type(atom_type), dimension(:), allocatable                      :: atomc
   type(cell_type)                                                 :: cellt
!
   if (ftype == ALL_FILES) then
       filetype = code_fragfilen(filename)
   else
       filetype = ftype
   endif
!
   if (filetype == SHELX_FILE .or. filetype == CIF_FILE) then
       if (present(spg)) then
           spaceg = spg
       else
           spaceg = init_spaceg_type(spaceg_string())
       endif
   endif
   cellt = set_cell_type(cell)
!
   select case (filetype)
      case (SHELX_FILE)    ! .res file
        call create_shelxfile(filename,atom,legm,cell,errcell,spaceg,alambda,celem,StructureName,package_alt_name)

      case (PDB_FILE)     ! pdb file
        call create_pdbfile(filename,atomi=atom,cella=cell,legm=legm,spg=spaceg_string())

      case (FRA_FILE)     ! fra file
        call create_frafile(filename,atom=atom,cella=cell,comment='Structure '//trim(StructureName))

      case (XYZ_FILE)    ! cartesian xyz file
        call create_xyzfile(filename,atom=atom,cella=cell,comment='Structure '//trim(StructureName))

      case (POV_FILE)    ! POV file
        call jav_get_style(cols, colcell, vfact, bmat)
        call create_povrayfile(filename,atom=atom,legm=legm,cell=cellt,elem=celem,cols=cols,colc=colcell,iscell=iscell,rmat=bmat)
#if _WIN32
        call Csystem('C:\PROGRA~1\POV-RA~1.5\bin\pvengine.exe '//'+I'//trim(filename)//char(0))
#else
        call Csystem('povray +P +W800 +H600'//' +I'//trim(filename)//' &'//char(0))
#endif

      case (CIF_FILE)    ! CIF file
!
        if (present(comm)) then
            if (present(legm)) then
                call create_ciffile(filename,cell=cellt,atom=atom,progname=package_alt_name,    &
                spg=spaceg,elem=celem,commline=comm,legm=legm,wave=alambda,source_type=neutro)
            else
                call create_ciffile(filename,cell=cellt,atom=atom,progname=package_alt_name,    &
                spg=spaceg,elem=celem,commline=comm,wave=alambda,source_type=neutro)
            endif
        else
            if (present(legm)) then
                call create_ciffile(filename,cell=cellt,atom=atom,progname=package_alt_name,    &
                spg=spaceg,elem=celem,legm=legm,wave=alambda,source_type=neutro)
            else
                call create_ciffile(filename,cell=cellt,atom=atom,progname=package_alt_name,    &
                spg=spaceg,elem=celem,wave=alambda,source_type=neutro)
            endif
        endif

      case (ZMT_FILE)    ! zmt file (Fenske-Hall Z-Matrix)
        call create_zmtfile(filename,atom=atom,legm=legm,cell=cellt)

      case (MOL_FILE)    ! mol file
!corr        call create_molfile(filename,cella=cell,atom=atom,legm=legm,progname=package_alt_name,error=err_exp)
        call bond_type_perception(atom,legm,cellt)
        call frac_to_cart_copy(atom,atomc,orthomatrix_std(cell))
        call OBConversionWriteFile(filename,cell,atomc,legm,spacegr%symbol_xhm)

      case (NWCHEM_IN_DFT)
        call write_nwfile(filename,cellt,spacegr,atom,StructureName,nwtheory=1)

      case (NWCHEM_IN_DFTD)
        call write_nwfile(filename,cellt,spacegr,atom,StructureName,nwtheory=2)

      case (ABINIT_IN)
        call write_abfile(filename,cellt,spacegr,atom)

      case (CRYSTAL_IN)
        call write_crystal_file(filename,cellt,spacegr,atom,StructureName)

      case (GAMESS_IN)
        call write_gmfile(filename,cellt,atom,StructureName)

      case (GAUSSIAN_CART_IN, GAUSSIAN_ZMAT_IN)
        call frac_to_cart_copy(atom,atomc,orthomatrix_std(cell))
        call OBConversionWriteFile(filename,cell,atomc,legm,spacegr%symbol_xhm)

      case (MOP_FILE)    ! cartesian mop file 
        call create_mopcrtfile(filename,cella=cell,atom=atom,title=StructureName)

      case default 
        call err_exp%set('Unknown file format. File '//trim(filename))
   end select
!
   if (err_exp%signal) then
       call err_exp%print()
   endif
!
   end subroutine export_structure

!---------------------------------------------------------------------      

   subroutine export_info(filename,ftype,vet,atom,legm,legmc,adp_type)
   USE connect_mod
   USE fileutil
   USE atom_type_util
   USE general, only:spacegr,cell
   USE unit_cell
   character(len=*), intent(in)                           :: filename
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   type(bond_type), dimension(:), allocatable, intent(in) :: legm
   integer, intent(in)                                    :: ftype ! che tipo di informazioni
   integer, dimension(:), intent(in)                      :: vet   ! quali informazioni
   integer, intent(in), optional                          :: adp_type ! adp type: 1 for B, 2 for U
!corr   type(symminfo_type), dimension(:), allocatable, optional :: infos
   integer                                                :: j_in
   integer                                                :: ier
   type(angle_type), dimension(:), allocatable            :: angle
   type(torsion_type), dimension(:), allocatable          :: tors
   type(structure_info_type)                              :: info
   type(bond_type), dimension(:), allocatable, intent(in), optional :: legmc
   type(cell_type) :: cellt
!
!  Open file
   call get_unit(j_in)
   open(unit=j_in,file=filename,iostat=ier,status='unknown')
!
   if (ier == 0) then
       select case (ftype)
          case (1)    ! general info
            info = structure_info(atom,1)
            call print_structure_info(info,j_in)

          case (2)    ! atomi
            if (present(adp_type)) then
                call print_atoms_vet(atom,kpr=j_in,vet=vet,adp_type=adp_type,spg=spacegr)
            else
                call print_atoms_vet(atom,kpr=j_in,vet=vet,spg=spacegr)
            endif

          case (3)    ! legami
            call print_connect(atom%lab,legm=legm,kpri=j_in,vet=vet)

          case (4)    ! angoli
            cellt = set_cell_type(cell)
            call legm_to_ang(legm,atom,cellt%get_g(),angle)
            call print_connect(atom%lab,angle=angle,kpri=j_in,vet=vet)

          case (5)    ! torsioni
            call legm_to_tors(legm,atom,cellt,tors)
            call print_connect(atom%lab,tors=tors,kpri=j_in,vet=vet)
        
          case (7)
            call print_connect(atom%lab,legm=legmc,kpri=j_in,vet=vet)

       end select

       close (j_in)
   endif
!
   end subroutine export_info

!---------------------------------------------------------------------      

   function structure_info(atom,icalc) result(info)
   USE General, only:cell,nrefl,spacegr,celem
   USE unit_cell
   USE Symmetry,only:icent,nsymtot
   USE variables, only:ref
   USE reflection_type_util
   USE atom_type_util
   USE stat
   USE elements
   USE proginterface
   type(structure_info_type)                              :: info
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   integer, intent(in)                                    :: icalc
   integer                                                :: i
!
   info%nspec = numelem(celem)
   info%cell = cell(1:6)
   do i=1,info%nspec
      info%vspec(i) = celem(i)%nw   !nw(i)
      info%zspec(i) = celem(i)%z    !no(i)
   enddo
   info%spaceg = trim(spacegr%symbol_xhm)   !!!!trim(spaceg_string())
   info%icent = icent
   info%nsymmstr = spacegr%nsymop
!
!  Se icalc = 0 non vengono riempite quelle informazioni che richiedono un calcolo
   if (icalc == 1) then
       info%cell_volume = cell_volume(cell(1:6))
       if (numatomtype(atom) > 0) then
           info%volume_per_atom = volume_per_atom(atom, nsymtot, info%cell_volume)
       else
           info%volume_per_atom = volume_per_atom(celem,info%cell_volume)
       endif
       if (.not.allocated(ref)) call fillref(ref)
       if (numatomtype(atom) > 0 .and. nrefl > 0) then
           call fcalcang(ref,atom,spacegr,celem)
           info%r_factor = Rvalue(ref%fo,ref%fc)
       else
           info%r_factor = 0.0
       endif
   endif
!
   end function structure_info

!---------------------------------------------------------------------      

   subroutine print_structure_info(info,kpr)
   USE General, only:StructureName,cell,spacegr,celem
   type(structure_info_type), intent(in) :: info
   integer, intent(in)                   :: kpr
   integer                               :: i
!
   write(kpr,'("Name                : ",a)')trim(StructureName)
   write(kpr,'("Cell Lengths        : ","a ",f0.5," b ",f0.5," c ",f0.5)')cell(1:3)
   write(kpr,'("Cell Angles         : ", "alpha ",f0.5," beta ",f0.5," gamma ",f0.5)')cell(4:6)
   if (info%icent == 1) then
       write(kpr,'("Space Group         : ",a)')trim(info%spaceg)//' (centrosymmetric)'
   else
       write(kpr,'("Space Group         : ",a)')trim(info%spaceg)//' (noncentrosymmetric)'
   endif
   call spacegr%prn_symop(2,kpr)
!corr   write(kpr,'("Symmetry-operations   ")')
!corr   write(kpr,'(4x,i0,a,a)')(i,')',trim(SymmStrings(i)),i=1,nSymmStrings)
!corr   write(kpr,'(4x,i0,a,a)')(i,')',trim(spacegr%symopstr(i)),i=1,spacegr%nsymop)
   write(kpr,'("Cell Volume         : ",f0.3)')info%cell_volume
   write(kpr,'("Cell Content        : ",18(a,1x,i0,1x))')(trim(celem(i)%lab),info%vspec(i),i=1,info%nspec)
   write(kpr,'("Volume per atom     : ",f0.3)')info%volume_per_atom
   write(kpr,'("R-Factor            : ",f0.2)')info%r_factor
!
   end subroutine print_structure_info

!---------------------------------------------------------------------      

   subroutine import_for_compare(filename)
   USE General, only:knw,nrefl,celem,spacegr
   USE Molcom, only:ModelToCompare
   USE TestModel
   USE connect_mod
   USE Riflessi
   USE variables
   USE atom_type_util
   USE elements
   USE reflection_type_util
   USE gen_frm
   character(len=*), intent(in)                     :: filename
   type(bond_type), dimension(:), allocatable       :: legpub
   type(reflection_type), dimension(:), allocatable :: refp
   integer                                          :: itype
   integer                                          :: ier
   integer                                          :: i
!
!  Se e' gia' presente la direttiva knw non attivare la funzione
   if (knw == 0) then
!
!      salva pxen perche' import puo' cambiare il contenuto
       itype = code_fragfilen(filename)
       call import_structure(filename,itype,atmpub,legpub,.true.,ier)
!
!      calcola fasi dal modello importato
       if (ier == 0) then
           nCoordPub = numatomtype(atmpub)
           if (allocated(ref)) then
               allocate(refp(nrefl))
               refp(:) = ref(:)
               call sfcalc(refp,atmpub,spacegr,celem)
               ipub(:nrefl) = refp(:)%ph
               do i=1,nrefl
                  khl(:,i) = ref(i)%hkl
               enddo
           else
               ier = 1
           endif
       endif
       if (ier == 0) ModelToCompare = 1
!
!      ricalcola nz perche' puo essere cambiato il contenuto
!corr       if (is_model) atm(:)%nz = nz_from_pxen(pxenv,celem)
   endif
!
   end subroutine import_for_compare

!---------------------------------------------------------------------      
   subroutine test_bond_perception(atom,legmv)
   USE fileutil
   !USE connect_mod, only: numbonds, copia_legm
   USE atom_type_util
   USE atom_basic
   USE General, only: cell
   USE unit_cell
   USE connect_mod
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   type(bond_type), dimension(:), allocatable, intent(in) :: legmv
   type(bond_type), dimension(:), allocatable             :: legm
   type(file_handle)                                      :: fileh
   integer                                                :: ncorr
   integer                                                :: i
!
   call copia_legm(legm,legmv)
   call bond_type_perception(atom,legm,set_cell_type(cell))   ! create new bond order
!
!  Compare legm%ord with legmv%ord e save the result in a external file
   call fileh%fopen('result_bond.txt','w')
   if (fileh%good()) then
       ncorr = 0
       do i=1,numbonds(legmv)
          if (legmv(i)%ord /= legm(i)%ord) then
              ncorr = ncorr + 1
              write(fileh%handle(),'(a,a,i0,a,i0)')'legame sbagliato: ',    &
              trim(slabnum(atom(legm(i)%n1)%lab,legm(i)%n1))//'-'//trim(slabnum(atom(legm(i)%n2)%lab,legm(i)%n2))   &
              //' ord=', legm(i)%ord,' -> ordine corretto=',legmv(i)%ord
              write(0,'(a,a,i0,a,i0)')'legame sbagliato: ',    &
              trim(slabnum(atom(legm(i)%n1)%lab,legm(i)%n1))//'-'//trim(slabnum(atom(legm(i)%n2)%lab,legm(i)%n2))   &
              //' ord=', legm(i)%ord,' -> ordine corretto=',legmv(i)%ord
          endif
       enddo
       if (ncorr == 0) then
           write(fileh%handle(),'("No errors in the bond order!")')
           write(0,'("No errors in the bond order!")')
       endif
       call fileh%fclose()
   endif
   !stop   !uncomment for shell
   end subroutine test_bond_perception

!----------------------------------------------------------------------------------------------------

   subroutine set_reflection_table(table,vet,ftype,ier)
!
!  Usata per generare una tabella di informazioni sui riflessi
!
   USE General, only:cell,spacegr,nrefl,alambda,celem
   USE variables, only:atm,ref
   USE Atomi
   USE counts
   USE prog_constants
   USE reflection_type_util
   USE proginterface
   type(reflection_type), intent(inout), dimension(:), allocatable :: table
   integer, parameter                                              :: NCOL = 8
   integer, dimension(NCOL)                                        :: vet
   integer, intent(in)                                             :: ftype
   integer, intent(out)                                            :: ier
   integer :: nref
!
   vet(:) = 0
   ier = 0
!
!corr   if (nrefl == 0 .and. nats > 0 .and. npunti == 0)  then
   if (alambda == 0.0) alambda = CU_WAVE
   if (nrefl == 0 .and. nats > 0) then
       call create_reflections(1.0,100.,cell,spacegr,alambda,table,nref)
       ! do i=1,nref
       ! !   if (any(table(i)%hkl - ref(i)%hkl /= 0)) write(0,*)'WARNING'
       !    write(0,*)'hkl=',table(i)%hkl,table(i)%tthd
       !    !write(0,*)'hkl=',ref(i)%hkl,ref(i)%tthd
       ! enddo
   else
       if (allocated(ref)) then
           !call reallocate(table,size(ref),.false.)
           call new_reflections(table,size(ref))
           table(:) = ref(:)
       else
           call fillref(table)
       endif
       nref = nrefl
   endif
!
   if (nref > 0) then  ! ci sono i riflessi?
       if (nats > 0 .and. allocated(atm)) then
           vet(6:7) = 1
           call fcalcang(table,atm,spacegr,celem)  
           if (ftype == 2) then
              table(:)%fc = table(:)%fc**2      ! col. 6: Fc^2
           endif
       else
           table(:)%fc = 0
           table(:)%ph = 0
       endif
       vet(1:5) = 1     
       vet(8) = 1     ! multeplicity
       table(:)%slaq = dvalue(table(:)%tthd,alambda) ! col. 4: d
       if (ftype == 2) then
           table(:)%fo = table(:)%fo**2          ! col. 5: Fo^2
       endif
   else
       ier = 1
   endif
!
   end subroutine set_reflection_table

!----------------------------------------------------------------------------------------------------

   subroutine write_reflection_table(table,vcol,kpri,ftype)
   USE strutil
   USE reflection_type_util, only: reflection_type
   type(reflection_type), intent(in), dimension(:) :: table  ! tabella dei riflessi
   integer, dimension(:), intent(in)               :: vcol   ! quali colonne stampare
   integer, intent(in)                             :: kpri
   integer, intent(in)                             :: ftype  ! 1 -> F; 2 -> F^2
   integer, dimension(size(vcol))                  :: ktype
   integer, parameter                              :: NCOL = 8
   integer, dimension(NCOL)                        :: ival
   real, dimension(NCOL)                           :: rval
   character(len=20), dimension(7)                 :: sval
   integer                                         :: i
!
!  Titoli delle colonne
   ktype(:) = 3
   if (ftype == 1) then
       call set_row_list(vcol,ktype,sval=(/'n.     ','h k l  ','2-theta',    &
         'd      ','Fo     ','Fc     ','phase  ','m      '/),kpr=kpri)
   else
       call set_row_list(vcol,ktype,sval=(/'n.     ','h k l  ','2-theta',    &
         'd      ','Fo^2   ','Fc^2   ','phase  ','m      '/),kpr=kpri)
   endif
!
!  Scrivi la tabella dei riflessi
   ktype = (/1,3,23,23,23,23,1,1/)   ! formato delle colonne
   do i=1, size(table)
      ival(1) = i
      write(sval(2),'(3(i0,1x))')table(i)%hkl(:)
      rval(3) = table(i)%tthd
      rval(4) = table(i)%slaq
      rval(5) = table(i)%fo
      rval(6) = table(i)%fc
      ival(7) = table(i)%ph
      ival(8) = table(i)%m
      call set_row_list(vcol,ktype,ival,rval,sval,kpri)
   enddo
!
   end subroutine write_reflection_table

!-------------------------------------------------------------------------------------------------------  
#if 0
   subroutine export_profile(fname,x,yoss,ycalc,yback,string)
   USE fileutil
   character(len=*), intent(in), optional   :: fname
   real, dimension(:), intent(in)           :: x,yoss
   real, dimension(:), intent(in), optional :: ycalc,yback
   character(len=*), intent(in), optional   :: string
   integer                                  :: i
   integer                                  :: j_in
   type(file_handle)                        :: fpatt
!
   call fpatt%fopen(fname,'w')
!
   if (fpatt%good()) then
       j_in = fpatt%handle()
       if (present(string)) write(j_in,'(a)')trim(string)
       if (present(ycalc) .and. present(yback)) then
          write(j_in,'(f15.3,1x,f15.3,f15.6,1x,f15.6,1x,f15.6)')   &
               (x(i),yoss(i),ycalc(i),yback(i),yoss(i)-ycalc(i),i=1,size(x))
       else
          write(j_in,'(f15.3,1x,f15.3)')(x(i),yoss(i),i=1,size(x))
       endif
       call fpatt%fclose()
   endif
!
   end subroutine export_profile
#endif
END MODULE importexport

   real function GetBondDistance(ispec1, ispec2)
!
!  Restituisce la distanza di legame date due specie
!      
   USE bondtmod
   USE connect_mod
   USE General, only: celem
   implicit none
   integer :: ispec1, ispec2
   integer :: z1,z2
!
   if (ispec1 < 0) then
       z1 = abs(ispec1)
   else
       z1 = celem(ispec1)%z
   endif
   if (ispec2 < 0) then
       z2 = abs(ispec2)
   else
       z2 = celem(ispec2)%z
   endif
   GetBondDistance = bond_distanceZ(z1,z2)
!
   end function GetBondDistance


MODULE exportfiles

implicit none

integer                       :: nexportf    ! number of files to export
character(250), dimension(10) :: export_name ! name of file
logical, dimension(10)        :: is_export   ! se vero il file e stato gia esportato 

CONTAINS
   
   subroutine export_file(filename,ier)
   USE Atomi
   USE General, only:nrefl,spacegr,celem
   USE reflection_type_util
   USE fileutil
   USE variables, only:atm, lconn
   USE importexport
   USE atom_type_util
   USE strutil
   USE proginterface
   character(len=*), intent(in)                     :: filename
   integer, intent(out)                             :: ier
   character(len=len_trim(filename))                :: ext
   type(reflection_type), dimension(:), allocatable :: reflb
!
   ext = get_extension(filename)
   if (len_trim(ext) > 0) then
       select case (upper(ext(1:4)))

          case ('HKL')      ! h k l   Fo^2 sigma in formato shelX
              if (nrefl > 0) then
                  allocate(reflb(nrefl))
                  call fillref(reflb)
                  call write_reflections(fname=filename,refl=reflb,code=2)
              else
                  ier = 2
              endif

          case ('FC')       ! h k l  Fo   Fc  phase
              if (nrefl > 0) then
                  allocate(reflb(nrefl))
                  call fillref(reflb)
                  if (nats > 0) then
                      call fcalcang(reflb,atm,spacegr,celem)
                  else
                      reflb(:)%fc = 0
                  endif
                  call write_reflections(fname=filename,refl=reflb,code=4)
              else
                  ier = 2
              endif

          case ('INT')
              call intensity_file_for_database(filename,ier)

          case default
              if (nats > 0) then
                  call export_structure(filename,atm,lconn,ftype=0)
              endif
              
       end select
   endif
!
   end subroutine export_file

!-------------------------------------------------------------------------------------------------------  

   subroutine intensity_file_for_database(filename,ier)
   USE General, only:nrefl,lo,no,nw,nk,cell,spacegr,celem,neutro,nsyn
   USE atom_type_util
   USE reflection_type_util
   USE variables, only:atm
   USE proginterface
   USE fileutil
   USE elements
   USE spginfom
   USE strutil
   USE ccryst, only: aniso_adp_is_ok
   USE unit_cell
   character(len=*), intent(in)                     :: filename
   integer, intent(out)                             :: ier
   type(reflection_type), dimension(:), allocatable :: reflb
   integer                                          :: nat,nghost
   type(file_handle)                                :: fileh
   character(len=*), dimension(0:2), parameter      :: subf = ['inorganic    ','organic      ','metallorganic']
   character(len=200)                               :: sform
   integer, parameter                               :: NMAXREFLD = 500
   real                                             :: mu, dens, maxf, vol
   integer                                          :: zval,i,ng
   integer, dimension(6)                            :: icell
   real                                             :: rapp_I,rapp_rho_v,rir
   type(cell_type)                                  :: cellt
!!
!!   real, parameter                                  :: maxf_Cu_corundum = 275670.5000  ! Calc. Intens. Corundum (from COD cif n. 1000017) b-iso=3.
!!   real, parameter                                  :: rho_vsq_Cu_corundum = 259076.6250 ! Calc. rho*vol*vol  Corundum  (from COD cif n. 1000017)
!!
   real, parameter                                  :: maxf_Cu_corundum = 346889.6562  ! Calc. Intens. Corundum (from COD cif n. 1000017) b-iso=1.
   real, parameter                                  :: rho_vsq_Cu_corundum = 259076.6250 ! Calc. rho*vol*vol  Corundum  (from COD cif n. 1000017)
!!
!! real, parameter                                  :: maxf_Cu_corundum = 392989.4062  ! Calc. Intens. Corundum (from COD cif n. 1000032) scardi-lutterotti 
!! real, parameter                                  :: rho_vsq_Cu_corundum = 259097.6250 ! Calc. rho*vol*vol  Corundum  (from COD cif n. 1000032 scardi-lutterotti)
!!
!! real, parameter                                  :: maxf_Cu_corundum = 347374.3750  ! Calc. Intens. Corundum (from COD cif n. 1000032) b-iso=1.
!! real, parameter                                  :: rho_vsq_Cu_corundum = 259097.6250 ! Calc. rho*vol*vol  Corundum  (from COD cif n. 1000032)
!!
   integer, dimension(6)                            :: code
   type(element_type), dimension(:), allocatable    :: chform
   character(len=3)                                 :: adv
   integer                                          :: ierc
   integer, parameter                               :: MAXBISO = 20
!corr   type(gref_type), dimension(:), allocatable :: gref
!   
   icell(:) = 1
   ier = 0
   if (nrefl == 0) call generate_reflections(1.0,100.0,1.54056)
   if (nrefl > 0) then
       nghost = numatomspec(atm,0)  ! this is the number of ghost atoms
       nat = numatomtype(atm) - nghost
       if (nghost > 0) then
           adv = 'no'
           write(lo,'(a)',advance=adv)'BIG TROUBLE: some atoms with Z=0: '
           ng = 0
           do i=1,numatomtype(atm)
              if (atm(i)%get_nz() == 0) then
                  ng = ng + 1
                  if (ng == nghost) then
                      adv = 'yes'
                      write(lo,'(a)',advance=adv)trim(atm(i)%lab)
                  else
                      write(lo,'(a)',advance=adv)trim(atm(i)%lab)//', '
                  endif
              endif
           enddo
           if (real(nghost) / nat > 0.7.and.nat> 10) nat = 0
       endif
       allocate(reflb(nrefl))
       call fillref(reflb)
       if (nat > 0) then
           cellt = set_cell_type(cell)
           call calcola_occ_new(atm,spacegr,cellt%get_g(),0.3)
!          check for adp
           do i=1,nat
              if (atm(i)%ptab == H_at) cycle
                !write(0,*) 'ADP=',i,aniso_adp_is_ok(atm(i)%bij)
              !if (any(abs(atm(i)%bij(:)) > 10)) then
              !    write(0,*)'BISO=',i
              !    atm(i)%bij(1) = 0.0
              !endif
              if (atm(i)%biso <= 0 .or. atm(i)%biso > MAXBISO) then
                  atm(i)%bij = 0
                  call set_biso(atm(i))
              endif
           enddo
           call fcalcang(reflb,atm,spacegr,celem,anis=.true.)
       else
           !ier = 1
           !return
           reflb%fc = 0; reflb%ph = 0;
       endif
       reflb(:)%lp = LPcorrection(reflb%tthd,neutro,nsyn)
       call fileh%fopen(filename,ios='w')
       if (fileh%good()) then
           vol = cell_volume(cell)
!corr           spg = init_spaceg_type(spaceg_string())
           if (.not. spg_check_cell(spacegr,cell)) then
               write(lo,'(a)')'Cell dimensions and spacegroup are non consistent'
           endif
           if (nat > 0) then
               dens = density_value(molecular_weight(atm%ptab,atm%och*atm%ocry),vol,spacegr%nsymop)
               mu = linear_abs_coeff(atm,dens)
!
               maxf = maxval(reflb(:)%m*reflb(:)%lp*reflb(:)%fc**2)
               rapp_rho_v = (rho_vsq_Cu_corundum )/(vol*vol*dens)
               rapp_I = maxf/maxf_Cu_corundum
               write(6,'(a,f0.8)')'Rho_v2=     ',vol*vol*dens
               rir = rapp_rho_v * rapp_I
               !write(0,'(a,f0.3)')'RIR=   ',rir
!
               call formula_from_atom(atm,chform)
               zval = nint(molecular_weight(atm%ptab,spacegr%nsymop*atm%och*atm%ocry)/molecular_weight(chform%ptab,chform%nw))
               call chemical_formula(chform, sform, fform=2, hide1=.true.)
!               
               write(fileh%handle(),'(1x,a,a)')'FORMULA: ',trim(sform)
               write(fileh%handle(),'(1x,a,a)')'SUBFILE: ',subf(is_organic(no(:nk),nw(:nk)))
           else
               rir = 0
               dens = 0
               zval = 0
               mu = 0
               write(fileh%handle(),*) 'SUBFILE: undefined'
           endif
           if (spacegr%undef()) then
               spacegr%csys_code = csys_from_cell(cell)
           endif
           call spg_get_cell_code(spacegr,cell,code,ierc)
           do i=1,6
              if(code(i).ne.0) icell(i) = 0
           enddo
           write(fileh%handle(),'(1x,a,3(1x,f0.4),3(1x,f0.3))')    'CELL:    ',cell(1:6)
           write(fileh%handle(),'(1x,a,6i6)')    'WRITE CELL: ',icell(1:6)
           write(fileh%handle(),'(1x,a,a)')    'SPG:     ',trim(spacegr%symbol_xhm)
           !if (spacegr%undef()) then
           !    write(fileh%handle(),'(1x,a,a)')    'CRY SYS: ',trim(cry_sys(csys_from_cell(cell)))
           !else
           if (spacegr%csys_code == CS_Trigonal) then
               if (uc_is_rhombohedral(cell,0.01)) then
                   write(fileh%handle(),'(1x,a,a)')    'CRY SYS: ',trim(cry_sys(spacegr%csys_code))//' (rhombohedral axes)'
               elseif (uc_is_hexagonal(cell,0.01)) then
                   write(fileh%handle(),'(1x,a,a)')    'CRY SYS: ',trim(cry_sys(spacegr%csys_code))//' (hexagonal axes)'
               endif
           else
               write(fileh%handle(),'(1x,a,a)')    'CRY SYS: ',trim(cry_sys(spacegr%csys_code))
           endif
           !endif
           write(fileh%handle(),'(1x,a,f0.3)') 'VOLUME : ',vol
           write(fileh%handle(),'(1x,a,f0.3,a)') 'Density: ',dens,' g cm-3'
           write(fileh%handle(),'(1x,a,i5)') 'Z : ',zval
           write(fileh%handle(),'(1x,a,f0.3,a)') 'mu(CuKa):',mu,' cm-1'
           write(fileh%handle(),'(1x,a,f0.3)') 'RIR     :',rir
           write(fileh%handle(),'(1x,a)') '----------------------------------------------------------------------------- '
           write(fileh%handle(),'(1x,a)') '  Remarks: Diffraction pattern calculated by EXPO from COD database cif file'
           write(fileh%handle(),'(1x,a)') '  Remarks: RIR calculated by EXPO     '
           write(fileh%handle(),'(1x,a)') '----------------------------------------------------------------------------- '
!!!!!!
!FIXME - test this to remove reflections in overlap and compute group intensity
!corr           reflb(:)%fwhm = 1.0
!corr           call make_ref_groups(reflb,gref,0.00005)
!corr           call print_refg(gref,0)
!corr           do i=1,size(gref)
!corr              if (gref(i)%code > 1) then
!corr                  reflb(gref(i)%vref(1))%fc = sum(reflb(gref(i)%vref(:))%fc)
!corr                  reflb(gref(i)%vref(2:))%fc = 0
!corr              endif
!corr           enddo
!!!!!!
           call write_reflections(jfile=fileh%handle(),refl=reflb,code=5,nref=min(NMAXREFLD,nrefl))
           call fileh%fclose()
       endif
   else
       ier = 2
   endif

   end subroutine intensity_file_for_database


!-------------------------------------------------------------------------------------------------------  

   subroutine export_files()
   integer :: ierrex
   integer :: i
!
!  esporta i file non esportati in precedenza
   do i=1,nexportf
      !if (.not.is_export(i)) then
          call export_file(export_name(i),ierrex)
      !    if (ierrex == 0) is_export(i) = .true.
      !endif
   enddo
!
   end subroutine export_files

!-------------------------------------------------------------------------------------------------------  

   subroutine update_model_and_export(atom,legm,kmodel)
   USE connect_mod, only: bond_type
   USE atom_basic, only: atom_type
   USE sinterface
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   type(bond_type), dimension(:), allocatable, intent(in) :: legm
   integer, intent(in)                                    :: kmodel
!
   if (kmodel == 1) then
       call update_model(atom,legm)
       call export_files()
   endif
!
   end subroutine update_model_and_export

END MODULE exportfiles

