module fra_frm

implicit none

private :: read_fra_file_jin, read_fra_file_name
interface read_fra_file
  module procedure read_fra_file_jin, read_fra_file_name
end interface

contains

   subroutine read_fra_file_jin(j_in,filename,atom,is_bfac,is_occ,error)
   USE strutil
   USE errormod
   USE atom_type_util
   USE elements
   integer, intent(in)                                       :: j_in    ! unita' di lettura del file
   character(len=*), intent(in)                              :: filename
   type(atom_type), allocatable, dimension(:), intent(inout) :: atom    ! atomi in uscita
   logical, intent(out)                                      :: is_bfac ! esiste il fattore termico?
   logical, intent(out)                                      :: is_occ  ! esiste l'occupanza?
   type(error_type), intent(out)                             :: error   ! errore in lettura
   integer                                                   :: natom
   integer                                                   :: ier
   integer                                                   :: nline
   character(len=500)                                        :: line,line2
   integer                                                   :: nlong1,nlong2
   real, dimension(50)                                       :: vet
   integer, dimension(50)                                    :: ivet
   integer                                                   :: iv
   integer                                                   :: i
   character(len=10)                                         :: strn
!
!  Conta gli atomi per allocare atom
   natom = 0
   do 
      read(j_in,'(a)',iostat=ier)line
      if (ier < 0) exit
      line = adjustl(line)
      if (len_trim(line) == 0) cycle
      if (line(1:1) /= '>' .and. line(1:1) /= '!') natom = natom + 1
   enddo
   if (natom == 0) then
       call error%set('Cannot read file '//trim(filename))
       go to 20
   else
       call new_atoms(atom,natom)
   endif
   rewind (j_in)
!
!  Leggi gli atomi
   nline = 0
   natom = 0
   is_bfac = .true.
   is_occ = .true.
   do
      nline = nline + 1
      read(j_in,'(a)',iostat=ier)line
      if (ier < 0) exit
      call s_filter(line)
      if (len_trim(line) == 0) cycle
      if (line(1:1) /= '>' .and. line(1:1) /= '!') then
!
!         Leggi simbolo chimico 
          call cutst(line,nlong1,line2,nlong2)
          natom = natom + 1
          atom(natom)%lab = trim(line2)
!
!         Leggi coordinate x,y,z,b,occ
          call Getnum(line,vet,ivet,iv)
          if (iv < 3) go to 10
          select case (iv)
             case (3)
              atom(natom)%xc = vet(1:3)
              is_bfac = .false.
              atom(natom)%och = 1.0
              is_occ = .false.
             case (4)
              atom(natom)%xc = vet(1:3)
              atom(natom)%biso = vet(4)
              atom(natom)%och = 1.0
              is_occ = .false.
             case (5:)
              atom(natom)%xc = vet(1:3)
              atom(natom)%biso = vet(4)
              atom(natom)%och = vet(5)
          end select
      endif
   enddo
!
!  Metti in nz il puntatore al file xen
   do i=1,natom
      atom(i)%ptab = pxen_from_specie(atom(i)%lab)
   enddo
!
   return
10 write(strn,'(i0)')nline
   call error%set('Error on reading line '//trim(strn)//' in file '//trim(filename))   
20 continue
!
   end subroutine read_fra_file_jin

!-------------------------------------------------------------------------------------------------

   subroutine read_fra_file_name(filename,atom,is_bfac,is_occ,error)
   USE errormod
   USE atom_basic
   USE fileutil
   character(len=*), intent(in)                              :: filename ! unita' di lettura del file
   type(atom_type), allocatable, dimension(:), intent(inout) :: atom     ! atomi in uscita
   logical, intent(out)                                      :: is_bfac  ! esiste il fattore termico?
   logical, intent(out)                                      :: is_occ   ! esiste l'occupanza?
   type(error_type), intent(out)                             :: error    ! errore in lettura
   type(file_handle) :: filef
!
   call filef%fopen(filename)
   if (filef%good()) then
       call read_fra_file_jin(filef%handle(),filename,atom,is_bfac,is_occ,error)
       call filef%fclose()
   else
       call error%set('Cannot open: '//trim(filename)//char(10)//'Message: '//trim(filef%err_msg()))
   endif
!
   end subroutine read_fra_file_name

!-------------------------------------------------------------------------------------------------

   subroutine read_fractional(j_in,cellc,is_cell,atom,error)
   USE strutil
   USE errormod
   USE unit_cell
   USE atom_type_util
   USE elements
   integer, intent(in)                                       :: j_in    ! unita' di lettura del file
   real, dimension(6), intent(inout)                         :: cellc   ! cella corrente
   logical, intent(in)                                       :: is_cell ! esiste gia' una cella?
   type(atom_type), allocatable, dimension(:), intent(inout) :: atom    ! atomi in uscita
   type(error_type), intent(out)                             :: error   ! errore in lettura
   integer                                                   :: nline
   character(len=500)                                        :: line,line2
   integer                                                   :: nlong1,nlong2
   real, dimension(50)                                       :: vet
   integer, dimension(50)                                    :: ivet
   integer                                                   :: iv
   integer                                                   :: natom
   real, dimension(6)                                        :: cellfile
   integer                                                   :: i
   character(len=10)                                         :: strn
   integer                                                   :: ier
!
!  Conta gli atomi per allocare atom
   nline = 1
   read(j_in,*,err=10,end=10)
   nline = nline + 1
   read(j_in,*,err=10,end=10)
   natom = 0
   do 
      read(j_in,'(a)',iostat=ier)line
      if (ier < 0) exit
      call s_filter(line)
      if (len_trim(line) == 0) cycle   ! righe vuote
      natom = natom + 1
   enddo
   if (natom == 0) then
       call error%set('Cannot read this file')
       go to 20
   else
       allocate(atom(natom))
   endif
   rewind (j_in)
!
   nline = 1
   read(j_in,*,err=10)
!
!  Leggi la cella
   nline = nline + 1
   read(j_in,'(a)',err=10)line
   call s_filter(line)
   call Getnum(line,vet,ivet,iv)
   if (iv /= 6) go to 10
   cellfile = vet(1:6)
!
   natom = 0
   do 
      nline = nline + 1
      read(j_in,'(a)',iostat=ier)line
      if (ier < 0) exit
      call s_filter(line)
      if (len_trim(line) == 0) cycle   ! gestione righe vuote
!
      call s_detag(line) ! elimina caratteri tra parentesi
!
!     Leggi simbolo chimico 
      call cutst(line,nlong1,line2,nlong2)
      if (nlong2 > 2 .or. .not.s_is_alpha(line2(:nlong2))) go to 10   
      natom = natom + 1
      atom(natom)%lab = line2(1:2)
!
!     Leggi coordinate frazionarie
      call Getnum(line,vet,ivet,iv)
      if (iv /= 3) go to 10
      atom(natom)%xc = vet(1:3)
   enddo
!
   if (is_cell) then                                 ! metti nella cella corrente se esiste
       call frac_to_cart(atom,orthomatrix_std(cellfile))
       call cart_to_frac(atom,orthomatrixi_std(cellc)) 
   else
       cellc(:) = cellfile(:)                        ! altrimenti la cella corrente diventa quella letta dal file
   endif
!
!  Metti in nz il puntatore al file xen
   do i=1,natom
      atom(i)%ptab = pxen_from_specie(atom(i)%lab)
   enddo
!
   return
10 write(strn,'(i0)')nline
   call error%set('Error on reading line '//trim(strn))   
20 continue
!
   end subroutine read_fractional

!------------------------------------------------------------------------------------------

   subroutine create_frafile(filename,jfile,atom,cella,comment)
!
!  Crea un Free Form Fractional file 
!   
!corr   USE trig_constants, only:pi
   USE fileutil
   USE atom_basic
!corr   USE elements, only:NLEN_LAB
   character(len=*), intent(in), optional    :: filename  ! se presente apre il file
   integer, intent(in), optional             :: jfile      ! unita' di lettura del file
   type(atom_type), dimension(:), intent(in) :: atom
   real, dimension(6), intent(in)            :: cella
   character(len=*), intent(in)              :: comment
!corr   character(len=NLEN_LAB)                   :: labat
   integer                                   :: i
   integer                                   :: j_in
   integer                                   :: ier
!
!  Apertura del file
   if (present(filename)) then
       call get_unit(j_in)
       open(unit=j_in,file=filename,iostat=ier,status='unknown')
   else
       j_in = jfile   ! in questo caso il file e' stato gia' aperto
   endif
!
!  Scrivi un commento sulla prima riga
   write(j_in,'(a)')trim(comment)
!
!  Scrivi la cella sulla seconda riga
   write(j_in,'(6(1x,f10.5))')cella(1:6)
!
!  Scrivi simbolo chimico e coordinate frazionarie sulle altre righe
   do i=1,size(atom)
!corr      labat = atom(i)%specie()
      write(j_in,'(a2,3(1x,f10.5))')atom(i)%specie(),atom(i)%xc
   enddo          
!
   if (present(filename)) close(j_in)
!
   end subroutine create_frafile

end module fra_frm
