module gen_frm

USE prog_constants, only: CU_WAVE

implicit none

enum, bind(c)
  enumerator :: ALL_FILES, CIF_FILE, PDB_FILE, SHELX_FILE, FRA_FILE,   &
                XYZ_FILE, ZMT_FILE, MOL_FILE, MOL2_FILE, MOP_FILE,     &
                POV_FILE, GAMESS_OUT, NWCHEM_OUT, ABINIT_OUT, GAMESS_IN, NWCHEM_IN_DFT, &
                GAUSSIAN_CART_IN, GAUSSIAN_ZMAT_IN, NWCHEM_IN_DFTD, ABINIT_IN, CRYSTAL_IN, &
                CRYSTAL_OUT,QE_OUT
endenum

integer, private, dimension(:), allocatable :: vmodel_cif
integer, private                            :: selected_cif

type xpd_option_type
  real :: wave = CU_WAVE
  real :: ttmin = 5
  real :: ttmax = 70
  real :: ttstep = 0.02
  real :: fwhm = 0.1
  real :: scal = 1000.
end type xpd_option_type

private :: make_powder_pattern_var, make_powder_pattern_xpd
interface make_powder_pattern
  module procedure make_powder_pattern_var, make_powder_pattern_xpd
end interface

contains
   subroutine import_crystal(filename,ftype,atom,legm,elem,cell,spgin,has_symmetry,neutro,gui_enabled,ier,at_info,nsec,sec_struct)
!
!  If has_symmetry is true structure will be imported in symmetry (cell and spgin) defined in input
!  If has_symmetry is false cell and spgin will contain the symmetry of the file or default if symmetry is absent in file
!
   USE fileutil
   USE errormod
   USE connect_mod
   USE molpnew
   USE atom_type_util
   USE model_util
   USE strutil
   USE elements
   USE spginfom
   USE unit_cell
   USE cif_frm
   USE pdb_frm
   USE fra_frm
   USE xyz_frm
   USE zmt_frm
   USE shelx_frm
   USE mol2_frm
   USE mop_frm
   USE qe_frm
!corr   USE mol_frm
   USE wlist
   USE OBmodule
   USE crystal_phase
   USE crystal_frm
   character(len=*), intent(in)                              :: filename
   integer, intent(in)                                       :: ftype ! if 0 the file type is set from filename
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm
   type(element_type), dimension(:), allocatable, intent(inout) :: elem
   real, dimension(6), intent(inout)                            :: cell
   logical, intent(in)                                          :: has_symmetry  ! if true structure will be imported in cell and spgin
   type(spaceg_type), intent(inout)                             :: spgin
   integer, intent(in)                                          :: neutro
   logical, intent(in)                                          :: gui_enabled    ! if true GUI is used to chose between more models
   type(bond_type), dimension(:), allocatable                :: legmc
   type(atom_type), dimension(:), allocatable                :: atomc
   integer                                                   :: j_in
   integer                                                   :: ier    !!!!,ierspace
   type(pdb_atom_info_type), dimension(:), allocatable, optional, intent(out) :: at_info   ! additional info for pdb file
   integer, intent(out), optional :: nsec
   type(secondary_struct_type), dimension(:), allocatable, optional, intent(out) :: sec_struct ! secondary structure from pdb
   type(spaceg_type)                            :: spgfile
   logical                                                   :: is_bfac,is_occ
   logical                                                   :: make_conn
   logical                                                   :: trasl
   type(error_type)                                          :: err_read
   type(error_type), dimension(:), allocatable               :: errcif
   integer                                                   :: i
   logical                                                   :: make_label
   logical                                                   :: make_occ
   logical                                                   :: is_conect
   character(len=16)                                         :: spacepdb
   integer                                                   :: nmodel
   logical, dimension(2)                                     :: optconn
   real, dimension(6)                                        :: cellfile
   integer                                                   :: filetype
   integer, parameter                                        :: NMAXATOM = 500
   integer                                                   :: nblockcif
   type(crystal_phase_type), dimension(:), allocatable       :: phase_file
   integer, parameter                                        :: CODE_CIFLIST = 101
   type(file_handle)                                         :: fileh
   type(cell_type) :: cellt
   logical :: usecov = .false.
!
   ier = 0
   call fileh%fopen(filename)
!
   if (fileh%good()) then
       j_in = fileh%handle()
       make_conn = .false.     
       is_bfac = .false.
       trasl = .false.
       make_label = .false.
       make_occ = .false.
       optconn = (/.true.,.true./)
       if (ftype == ALL_FILES) then
           filetype = code_fragfilen(filename)
       else
           filetype = ftype
       endif

       select case (filetype)

         case (FRA_FILE)    ! fra or frac file
           !has_symmetry = is_cell_defined()           
           spgfile = init_spaceg_type()
           call read_fra_file(j_in,filename,atomc,is_bfac,is_occ,err_read)
!      
!          se si genera errore leggi il file come standard fractional file
           if (err_read%signal) then
               call clear_atoms(atomc)
               rewind (j_in)
               call read_fractional(j_in,cell(1:6),has_symmetry,atomc,err_read)
               is_bfac = .false.
               is_occ = .false.
           else
               if (.not.has_symmetry) cell =  [10., 10., 10., 90., 90., 90.]
           endif
           if (.not.err_read%signal) then
               !if (.not.is_space_defined()) call set_spaceg_new(init_spaceg_type())   ! force 'P 1'
               make_conn = .true.
               make_occ = .not.is_occ
               make_label = .true.
           endif

         case (XYZ_FILE)    ! cartesian file
           if (.not.has_symmetry) cell =  [10., 10., 10., 90., 90., 90.]
           spgfile = init_spaceg_type()
           call read_xyzfile(j_in,cell(1:6),atomc,err_read)
           if (.not.err_read%signal) then
               make_conn = .true.
               make_occ = .true.
               trasl = .true.
               make_label = .true.
           endif

         case (SHELX_FILE)    ! ShelX file (.ins,.res)
           !has_symmetry = is_cell_defined_list()           
           call read_shelxfile(j_in,atomc,cell,has_symmetry,spgfile,err_read)
           if (.not.err_read%signal) then
               !if (.not.has_symmetry) call set_cell(cell(1:6))
               !if (.not.is_space_defined_list()) call set_spaceg_new(spgfile)
               make_conn = .true.
               is_bfac = .true.
           endif

         case (PDB_FILE)    ! pdb file
           !has_symmetry = is_cell_defined_list()
           if (present(at_info) .and. present(nsec) .and. present(sec_struct)) then
               call read_PDBfile(j_in,cell,has_symmetry,atomc,legmc,is_conect,spacepdb,err_read,   &
                                 at_info,nsec=nsec,sec_struct=sec_struct)
           else
               call read_PDBfile(j_in,cell,has_symmetry,atomc,legmc,is_conect,spacepdb,err_read)
           endif
           if (.not.err_read%signal) then
               spgfile = init_spaceg_type(spacepdb)
               !if (.not.has_symmetry) call set_cell(cell(1:6))
               if (is_conect)call copia_legm(legm,legmc)
               !if (.not.is_space_defined_list()) then
               !    !spgfile = init_spaceg_type(spacepdb)
               !    call set_spaceg_new(spgfile)
               !endif
               make_conn = .not.is_conect
               is_bfac = .true.
               trasl = .true.
               make_occ = .true.
           endif

         case (CIF_FILE)    ! cif file
          ! if (kadd) then
          !     has_symmetry = is_cell_defined()  ! in 'add' check only for cell
          ! else
          !     has_symmetry = is_cell_defined_list() ! in 'import' check for cell in list
          ! endif
           call read_CIFfile(j_in,phase_file,nblockcif,has_symmetry,cell,errcif)
!           call read_CIFfile(j_in,phase_file,nblockcif,has_symmetry,cell,errcif,etype='S',dummy=.true.)  ! for INT file generation
           if (any(errcif(:nblockcif)%signal .eqv. .false.)) then
               nmodel = 0
               allocate(vmodel_cif(nblockcif))
               vmodel_cif(:) = 0
               do i=1,nblockcif
                  if (numatomtype(phase_file(i)%at) > 0) then
                      nmodel = nmodel + 1
                      vmodel_cif(nmodel) = i
                  endif
               enddo
               if (nmodel > 0) then
                   selected_cif = vmodel_cif(1)
                   !if (nmodel > 1 .and. jscreen > 0 .and. ifAutomatic == 0) then
                   if (nmodel > 1 .and. gui_enabled) then
                       call wlist_create(CODE_CIFLIST,1,nmodel,['Data block name'],'CIF file',   &
                       trim(i_to_s(nmodel))//' crystal structures. Select one of them.',2,1,cif_func,     &
                       [set_column((/(trim(phase_file(vmodel_cif(i))%phase_name)//' ('//    &
                       trim(phase_file(vmodel_cif(i))%spg%symbol_xhm)//')',i=1,nmodel)/))])
                   endif
               else
                   selected_cif = 1
               endif
               deallocate(vmodel_cif)
               err_read = errcif(selected_cif)
!
!              controlla la cella 
               if (.not.has_symmetry) then
                   !call set_cell(phase_file(selected_cif)%cell%get_par())
                   cell = phase_file(selected_cif)%cell%get_par()
               endif
               if (phase_file(selected_cif)%exist_spg()) then
                   spgfile = phase_file(selected_cif)%spg
               else
                   spgfile = init_spaceg_type()   ! force 'P 1'
               endif
               if (numatomtype(phase_file(selected_cif)%at) > 0) then
                   call copia_atoms(atomc,phase_file(selected_cif)%at)
                   make_conn = .true.
                   is_bfac = .true.
                   trasl = .false.
                   make_occ = .false.
               endif
           else
               err_read = errcif(1)
           endif

         case (ZMT_FILE)    ! z-matrix
           spgfile = init_spaceg_type() 
           if (.not.has_symmetry) cell =  [10., 10., 10., 90., 90., 90.]
           !if (.not.is_space_defined()) call set_spaceg_new(init_spaceg_type())   ! force 'P 1'
           call read_zmtfile(j_in,cell,atomc,err_read)
           if (.not.err_read%signal) then
               make_conn = .true.
               make_label = .true.
               make_occ = .true.
           endif

         case (MOP_FILE)    ! mop file
           spgfile = init_spaceg_type() 
           if (.not.has_symmetry) cell =  [10., 10., 10., 90., 90., 90.]
           !if (.not.is_space_defined()) call set_spaceg_new(init_spaceg_type())   ! force 'P 1'
!
!          Prova a leggere il file come mop in formato coordinate interne
           call read_mopintfile(j_in,cell,atomc,err_read)
!      
!          se si genera errore leggi il file come mop a coordinate cartesiane
           if (err_read%signal) then
               call clear_atoms(atomc)
               rewind (j_in)
               call read_mopcartfile(j_in,cell,atomc,err_read)
           endif
           if (.not.err_read%signal) then
               make_conn = .true.
               make_occ = .true.
               make_label = .true.
               trasl = .true.
           endif
#if 0
         case (MOL_FILE)    ! mol file
           spgfile = init_spaceg_type() 
           if (.not.has_symmetry) cell =  [10., 10., 10., 90., 90., 90.]
           call read_molfile(j_in,cell,atomc,legmc,err_read)
           if (.not.err_read%signal) then
               call copia_legm(legm,legmc)
               trasl = .true.
               make_occ = .true.
               make_label = .true.
           endif
#endif
         case (MOL_FILE,GAMESS_OUT,NWCHEM_OUT,ABINIT_OUT,CRYSTAL_OUT)    ! mol file, gamess output file
           if (is_espresso_file(filename)) then
               call read_qe_file(filename,atomc,cellfile,spgfile,err_read)
           elseif (is_crystal_file(filename) > 0) then
               call read_crystal_file(filename,atomc,cellfile,spgfile,err_read)
               if (.not.has_symmetry) cell = cellfile
               if (.not.err_read%signal) then
                   make_conn = .true.
                   make_label = .true.
                   make_occ = .true.
               endif
           else
               call OBConversionReadFile(filename,atomc,legmc,err_read)
               if (.not.has_symmetry) then
                   if (has_cell) then
                       cell = MyCell
                   else
                       cell =  [10., 10., 10., 90., 90., 90.]
                   endif
               endif
               if(has_spg) then
                  spgfile = init_spaceg_type(spgname)
               else
                  spgfile = init_spaceg_type() 
               endif

               if (.not.err_read%signal) then
                   call copia_legm(legm,legmc)  
                   call copia_legm(legm,legmc)
                   call cart_to_frac(atomc,orthomatrixi_std(cell))
                   trasl = .true.
                   make_occ = .true.
                   make_label = .true.
               endif
           endif

         case (MOL2_FILE)    ! mol2 file
           call read_mol2file(j_in,atomc,legmc,spgfile,cellfile,err_read)
           if (.not.err_read%signal) then
               !if (.not.is_cell_defined()) call set_cell(cellfile)
               call cart_to_frac(atomc,orthomatrixi_std(cell))
               !if (.not.is_space_defined()) call set_spaceg_new(spgfile)
               call copia_legm(legm,legmc)
               trasl = .true.
               make_occ = .true.
           endif

         case default 
           call err_read%set('Unknown file format. File '//trim(filename))

       end select

       call fileh%fclose()

   else
       call err_read%set('Cannot open: '//trim(filename)//char(10)//' Message: '//trim(fileh%err_msg()))
   endif
!
   if (err_read%signal) then
       call err_read%print()
       ier = 1
   else
       if (numatomtype(atomc) > 0) then  ! cif files can't contain atom coordinates
           call copia_atoms(atom,atomc)
!
!          Trasla il frammento in cella: azione eseguita solo se il file contiene coord. cart.
           if (trasl) call translate_in_cell(atom)
!
!          Update space group in input if was not set by user
           !if (.not.spgin%from_user) spgin = spgfile
           if (.not.has_symmetry) spgin = spgfile
!
!          Calcola occupanza cristallog. senza modificare le coordinate     
           cellt = set_cell_type(cell)
           call calcola_occ(atom,spgin,cellt,modcoor=.false.)
           if(make_occ) atom%och = 1.0  ! inizializza occ. chimica
!
!          Update content
           call update_elem(elem,atom(:)%ptab,neutro,spgin%nsymop,atom%ocry)
           call specie_from_ptab(atom,elem)
!
!          Creare le label
           if (make_label) call atom_string(atom,elem)  
!
!          Genera connettivita' se non esiste nel file importato
           if (make_conn) then
!              disable because coordinates could be wrong or for too atoms
               !if (err_read%warning .or. size(atom) > NMAXATOM) optconn(:) = .false.   !!FIXME - potresti usare un code error
               if (size(atom) > NMAXATOM) optconn(:) = .false. 
               if (filetype == PDB_FILE) then  ! special connectivity for duplicate residues
                   optconn(:) = .false.
                   call create_connectivity(atom,legm,cellt,spgin,check=optconn(1),move=optconn(2), &
                        vet=atom(:)%rcod(1),code=2,usecov=usecov)
                   atom%rcod(1) = 0
               else
                   call create_connectivity(atom,legm,cellt,spgin,check=optconn(1),move=optconn(2),usecov=usecov)
               endif
           endif
!
!          Genera fattori termici se non esistono nel file
           if(.not.is_bfac)call set_biso(atom)
!!!
!!!          Calcola occupanza cristallog. senza modificare le coordinate     
!!           call calcola_occ(atom,spgin,cellt,modcoor=.false.)
!!           if(make_occ) atom%och = 1.0  ! inizializza occ. chimica
!
           call set_intensity(atom)
!
       else
           call clear_atoms(atom)
           call clear_bonds(legm)
       endif
!
       if (err_read%warning) call err_read%print()
   endif
!
   end subroutine import_crystal

!---------------------------------------------------------------------      

   subroutine cif_func(code)
   integer, intent(in) :: code
   select case (code)
      case (:-1)
        selected_cif = vmodel_cif(-code)
      case (0)  ! open list
      case (1)  ! Ok on list
      case (2)  ! Cancel on list
   end select
   end subroutine cif_func

!---------------------------------------------------------------------      

   integer function code_fragfilen(filedirname)   result(code)
!
!  Utilizza l'estensione del nome del file per individuare il tipo
!
   USE strutil
   USE fileutil
   character(len=*), intent(in)         :: filedirname
!
   select case (remc0(upper(get_extension(filedirname))))
     case ('FRA','FRAC')
       code = FRA_FILE

     case ('XYZ')
       code = XYZ_FILE

     case ('RES','INS')
       code = SHELX_FILE

     case ('PDB','ENT')
       code = PDB_FILE

     case ('CIF')
       code = CIF_FILE

     case ('ZMT','FH','FHZ')
       code = ZMT_FILE

     case ('MOP')
       code = MOP_FILE

     case ('MOL','MDL','SDF','SD')
       code = MOL_FILE

     case ('MOL2','ML2')
       code = MOL2_FILE

     case ('NW')
       code = NWCHEM_IN_DFT

     case ('POV')
       code = POV_FILE

     case ('OUT','LOG')
       code = GAMESS_OUT

     case ('INP','GAMIN')
       code = GAMESS_IN

     case ('COM','GAU')
       code = GAUSSIAN_CART_IN

     case ('GZMAT')
       code = GAUSSIAN_ZMAT_IN

     case ('IN')
       code = ABINIT_IN

     case ('D12')
       code = CRYSTAL_IN

     case default 
       code = 0

   end select
!
   end function code_fragfilen

!---------------------------------------------------------------------      

   subroutine import_crystal_type(crystal,neutro,filename,ftype)
   USE crystal_phase
   USE spginfom
   USE unit_cell
   type(crystal_phase_type), intent(inout) :: crystal
   integer, intent(in)                     :: neutro
   character(len=*), intent(in)            :: filename
   integer, intent(in), optional           :: ftype
   integer                                 :: ftypef
   real, dimension(6)                      :: cell
   type(spaceg_type)                       :: spg
   integer                                 :: ier
!
   if (present(ftype)) then
       ftypef = ALL_FILES
   else
       ftypef = ftype
   endif
   call import_crystal(filename,ftype,crystal%at,crystal%bonds,crystal%elem,cell,spg,.false.,neutro,.false.,ier)
   if (ier == 0) then
       call crystal%set_symmetry(spg,set_cell_type(cell))
   endif
!
   end subroutine import_crystal_type

!-------------------------------------------------------------------------------------------------------  

   subroutine make_powder_pattern_var(xc,yc,ref,atom,spg,cell,elem,thmin,thmax,thstep,wave,fwhm,scal)
   USE atom_basic
   USE unit_cell
   USE crystal_phase
   USE calculate_pdp
   USE reflection_type_util
   USE pearsonf
   USE psvoigtf
   USE tchzf
   USE profile_function
   USE arrayutil
   USE spginfom
   USE progtype, only: refine_condition_type
   real, dimension(:), allocatable, intent(inout)                  :: xc,yc
   type(reflection_type), dimension(:), allocatable, intent(inout) :: ref
   type(atom_type), dimension(:), allocatable, intent(in)          :: atom
   type(spaceg_type), intent(in)                                   :: spg
   type(cell_type), intent(in)                                     :: cell
   type(element_type), dimension(:), allocatable, intent(in)       :: elem
   real, intent(in)                                                :: thmin,thmax,thstep
   real, intent(in)                                                :: wave,fwhm,scal
   type(crystal_phase_type)                                        :: crystal
   integer                                                         :: nc,i
   real                                                            :: tmax
   type(refine_condition_type)                                     :: rfc
   type(pearson_function_type)                                     :: pear
   type(pvoigt_function_type)                                      :: pvoi
   type(tch_function_type)                                         :: tch
   integer                                                         :: ier
   type(spaceg_type)                                               :: p1
!
   if (spg%undef()) then
       call p1%set_p1()  ! force undefined space group to P1
       call crystal%set(atom,spg=p1,cell=cell,elem=elem)
   else
       call crystal%set(atom,spg=spg,cell=cell,elem=elem)
   endif
!
!  Make series tmin,tstep 
!!!!!FIXME - use a make_series from arrayutil or from NR
   nc = floor((thmax - thmin) / thstep + 1)   ! number of counts
   call new_array(xc,nc)
   xc(:) = thmin + [(i*thstep,i=0,nc-1)]     
   tmax = xc(nc)
   call new_array(yc,nc)
   !write(0,*)'NC=',nc,' thmin=',thmin,' thmax=',thmax,' step=',thstep
!
   call crystal%make_reflections(thmin,tmax,wave)
   call crystal%sfactor()
   call crystal%LPcorr(0,0)
   !call crystal%print(0,'SCAR')
!
!!!!!FIXME - use a calculate_powder_pattern_crystal
   rfc%lim%ncu = nc;  rfc%lim%nrefu=numrefl(crystal%ref); rfc%typefun = PEARSON7
   call AllocaRietnew(1,rfc%lim%nrefu)
   rapIcalc(:) = 1.0
   call init_function_param(pear,pvoi,tch,fwhm)
   call calculate_powder_pattern(rfc,xc,yc,crystal%ref,crystal%scal,1,0.0,pear,pvoi,tch,ier)
   yc = scal * yc / maxval(yc)
   call AllocaRietnew(0,rfc%lim%nrefu)
   call crystal%get_reflections(ref)
   !call write_reflections(0,refl=ref,code=5)
   !do i=1,nc
   !   write(71,*)xc(i),yc(i)
   !enddo
!
   end subroutine make_powder_pattern_var

!--------------------------------------------------------------------------------------

   subroutine make_powder_pattern_xpd(xc,yc,ref,atom,spg,cell,elem,xpd)
   USE unit_cell
   USE spginfom
   USE elements
   USE atom_basic
   USE reflection_type_util
   real, dimension(:), allocatable, intent(inout)                  :: xc,yc
   type(reflection_type), dimension(:), allocatable, intent(inout) :: ref
   type(atom_type), dimension(:), allocatable, intent(in)          :: atom
   type(spaceg_type), intent(in)                                   :: spg
   type(cell_type), intent(in)                                     :: cell
   type(element_type), dimension(:), allocatable, intent(in)       :: elem
   type(xpd_option_type), intent(in)                               :: xpd

   call make_powder_pattern_var(xc,yc,ref,atom,spg,cell,elem,xpd%ttmin,xpd%ttmax,xpd%ttstep,xpd%wave,xpd%fwhm,xpd%scal)

   end subroutine make_powder_pattern_xpd

!--------------------------------------------------------------------------------------

   subroutine make_powder_pattern_cmd(cmd,atom,spg,cell,elem)
   USE commandsmod
   USE unit_cell
   USE spginfom
   USE atom_basic
   USE elements
   USE reflection_type_util
   USE prog_constants
   type(command_type), intent(in)                            :: cmd
   type(atom_type), dimension(:), allocatable, intent(in)    :: atom
   type(spaceg_type), intent(in)                             :: spg
   type(cell_type), intent(in)                               :: cell
   type(element_type), dimension(:), allocatable, intent(in) :: elem
   real, dimension(:), allocatable                           :: xpd,ypd
   type(reflection_type), dimension(:), allocatable          :: refpd
   !real :: ttmin=5., ttmax=70., ttstep=0.02, wave=CU_WAVE, fwhm=0.1
   type(xpd_option_type) :: xpdopt
!
   call cmd%prn(0)

   if (cmd%ndir == 0) return

   if (cmd%ndir > 0) then
       !corrcall make_powder_pattern(xpd,ypd,refpd,atom,spg,   &
       !corr                     cell,elem,ttmin,ttmax,ttstep,wave,fwhm)
       call make_powder_pattern(xpd,ypd,refpd,atom,spg,cell,elem,xpdopt)
!!!FIXME - use testc.exp
!!!!!FIXME - use a make_series from arrayutil in make_powder_pattern
!!!!!FIXME - use a calculate_powder_pattern_crystal in make_powder_pattern
!!!FIXME - meglio non sovrascivere file esistenti
       call export_profile(cmd%strdir(1),xpd,ypd,string='#      2theta    yoss')  
!!!!!FIXME - con piu' cif crea prima il metodi from_file per la classe cristal e poi overloda make_powder_pattern
!!!!!FIXME - con crystal. la vecchia make_powder_patter chiamera la prima
   endif
!
   end subroutine make_powder_pattern_cmd

!-------------------------------------------------------------------------------------------------------  

   subroutine export_profile(fname,x,yoss,ycalc,yback,string)
   USE fileutil
   character(len=*), intent(in), optional   :: fname
   real, dimension(:), intent(in)           :: x,yoss
   real, dimension(:), intent(in), optional :: ycalc,yback
   character(len=*), intent(in), optional   :: string
   integer                                  :: i
   integer                                  :: j_in
   type(file_handle)                        :: fpatt
!
   call fpatt%fopen(fname,'w')
!
   if (fpatt%good()) then
       j_in = fpatt%handle()
       if (present(string)) write(j_in,'(a)')trim(string)
       if (present(ycalc) .and. present(yback)) then
          write(j_in,'(f15.3,1x,f15.3,f15.6,1x,f15.6,1x,f15.6)')   &
               (x(i),yoss(i),ycalc(i),yback(i),yoss(i)-ycalc(i),i=1,size(x))
       else
          write(j_in,'(f15.3,1x,f15.3)')(x(i),yoss(i),i=1,size(x))
       endif
       call fpatt%fclose()
   endif
!
   end subroutine export_profile

!-------------------------------------------------------------------------------------------------------  

   subroutine export_for_db(atom,legm,cell,spg,elem,sname)
!
!  Generate file csv for statistical analysis of distances
!
   USE elements
   USE atom_type_util
   USE unit_cell
   USE spginfom
   USE connect_mod
   USE strutil
   USE fileutil
   type(atom_type), dimension(:), allocatable, intent(in)    :: atom
   type(bond_type), dimension(:), allocatable                :: legm
   type(cell_type), intent(in)                               :: cell
   type(spaceg_type), intent(in)                             :: spg
   type(element_type), dimension(:), allocatable, intent(in) :: elem
   character(len=*), intent(in)                              :: sname
   integer                                                   :: i,charge,a1,a2,a3
   type(file_handle)                                         :: fdb
   type(angle_type), dimension(:), allocatable               :: ang
!
!  Distances
   call fdb%fopen(trim(sname)//'d.csv','w')
   do i=1,numbonds(legm)
      a1 = legm(i)%n1
      a2 = legm(i)%n2
      if (atom(a1)%z() > atom(a2)%z() .and. atom(a2)%z() /= H_at) then  !Sort Z: es. C-N but not N-C
          a2 = legm(i)%n1
          a1 = legm(i)%n2
      endif
      write(fdb%handle(),'(a)')';'//trim(sname)//';'    &
                       //atom(a1)%spec()//';'//i_to_s(atom(a1)%charge())//';'                &
                       //atom(a2)%spec()//';'//i_to_s(atom(a2)%charge())//';'//r_to_s(legm(i)%dist)
   enddo
   call fdb%fclose()
!
!  Angles
   call fdb%fopen(trim(sname)//'a.csv','w')
   call legm_to_ang(legm,atom,cell%get_g(),ang)
   if (numangles(ang) > 0) then
       do i=1,numangles(ang)
          a1 = ang(i)%n1
          a2 = ang(i)%n2
          a3 = ang(i)%n3
          if (atom(a1)%z() > atom(a2)%z() .and. atom(a2)%z() /= H_at) then  !Sort Z: es. C-N-H but not H-N-C
              a3 = ang(i)%n1
              a1 = ang(i)%n3
          endif
          write(fdb%handle(),'(a)')';'//trim(sname)//';'    &
                           //atom(a1)%spec()//';'//i_to_s(atom(a1)%charge())//';'                &
                           //atom(a2)%spec()//';'//i_to_s(atom(a2)%charge())//';'                &
                           //atom(a3)%spec()//';'//i_to_s(atom(a3)%charge())//';'//r_to_s(ang(i)%val)
       enddo
   endif
   call fdb%fclose()
!
   end subroutine export_for_db

end module gen_frm
