MODULE sinterface

implicit none

CONTAINS 

   subroutine atom_to_xyzs(atom,legm)
!
!  Copia atom e legm in variabili xyzs e iconn
!
   USE Atomi
   USE connect_mod
   type(atom_type), dimension(:), intent(in)                        :: atom
   type(bond_type), dimension(:), intent(in), allocatable, optional :: legm
   integer                                                          :: i,j
!
   nats = size(atom)
   if (nats > MaxAtoms) then
!corr       write(0,*)'Too many atoms!'
       return
   endif
   do i=1,nats
      do j=1,3
         xyzs(j,i) = atom(i)%xc(j)
      enddo
      nz(i) = atom(i)%get_nz()
      xyzs(4,i) = atom(i)%inte
      xyzs(5,i) = atom(i)%ocry
      occpch(i) = atom(i)%och
      bfac(i) = atom(i)%biso                                                                    
      aLab(i) = atom(i)%lab
   enddo
!
   if (present(legm)) call bond_to_iconn(legm,iconn)
!
   end subroutine atom_to_xyzs

!--------------------------------------------------------------------------------------
   
   subroutine xyzs_to_atom(atom)
   USE Atomi
   USE atom_type_util
   USE General, only: celem
   type(atom_type), intent(inout), dimension(:), allocatable :: atom
   call xyz_to_atom(atom,xyzs(:3,:nats),bfac,occr=xyzs(5,:),occh=occpch,nz=nz,lab=aLab,inte=xyzs(4,:),elem=celem)
   end subroutine xyzs_to_atom

!--------------------------------------------------------------------------------------

   subroutine update_model(atom,legm) 
   USE variables
   USE atom_type_util
   USE connect_mod
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   type(bond_type), dimension(:), allocatable, intent(in) :: legm
!
!  aggiorna xyzs e iconn
   if (numatomtype(atom) > 0) then
       call atom_to_xyzs(atom,legm)
!
!      aggiorna atm
       call copia_atoms(atm,atom)
!
!      aggiorna lconn
       call copia_legm(lconn,legm)
   endif
!
   end subroutine update_model

!--------------------------------------------------------------------------------------

   subroutine update_atm()
   USE Atomi
   USE variables
   USE atom_type_util
   USE connect_mod
   USE General, only: cell,celem
   USE unit_cell
   type(cell_type) :: cellt
!
   if (nats > 0) then
!
!      aggiorna atm
       call xyz_to_atom(atm,xyzs(:3,:nats),bfac,occr=xyzs(5,:),occh=occpch,nz=nz,lab=aLab,inte=xyzs(4,:),elem=celem)
!
!      aggiorna lconn
       cellt = set_cell_type(cell)
       call iconn_to_legm(iconn,lconn,atm,cellt%get_g())
   endif
!
   end subroutine update_atm

!--------------------------------------------------------------------------------------

   function spaceg_string() result(space)
   USE Symmetry
   USE strutil
   character(len=16) :: space
   write(space,'(16a1)')nSpaceG
   space(1:1) = upper(space(1:1))
   end function spaceg_string

!--------------------------------------------------------------------------------------

   logical function is_cell_defined_list()
   USE General, only:cell, InfoOK
!
!  InfoOK(1) == 1 significa che e' stata creata la lista 1
   is_cell_defined_list = all(cell(1:6) > 0.0) .and. InfoOK(1) == 1
!
   end function is_cell_defined_list

!--------------------------------------------------------------------------------------

   logical function is_space_defined_list()
   USE strutil
   USE General, only:InfoOK
   character(len=16) :: spg
   spg = spaceg_string()
   call s_filter(spg)
!
!  InfoOK(1) == 1 significa che e' stata creata la lista 2
   is_space_defined_list = len_trim(spg) /= 0 .and. InfoOK(2) == 1
   end function is_space_defined_list

!--------------------------------------------------------------------------------------

   subroutine set_cell_as_defined()
   USE General, only:cell, InfoOK
   if (all(cell(1:6) > 0.0)) InfoOK(1) = 1
   end subroutine set_cell_as_defined

!--------------------------------------------------------------------------------------

   subroutine set_space_as_defined()
   USE General, only:InfoOK
   if (len_trim(spaceg_string()) /= 0) InfoOK(2) = 1
   end subroutine set_space_as_defined

!--------------------------------------------------------------------------------------

   logical function is_cell_defined()
   USE General, only:cell
!
   is_cell_defined = all(cell(1:6) > 0.0)
!
   end function is_cell_defined

!--------------------------------------------------------------------------------------

   logical function is_space_defined()
   USE strutil
   character(len=16) :: spg
   spg = spaceg_string()
   call s_filter(spg)
   is_space_defined = len_trim(spg) /= 0 
   end function is_space_defined

!--------------------------------------------------------------------------------------

   subroutine create_cell()
   real, dimension(6) :: cella
   cella(1:3) = 10.0
   cella(4:6) = 90
   call set_cell(cella)
   end subroutine create_cell

!--------------------------------------------------------------------------------------

   subroutine set_cell(cella)
   USE General, only: cell,pp
   USE unit_cell
   real, dimension(:), intent(in) :: cella
   real, dimension(3,3)           :: gr
!
   cell(1:6) = cella(1:6)
!
!  serve nella generazione dei riflessi (call gener)
   gr = matrice_mreciproca(cell(1:6))
   pp(1:6) = 0.25*(/gr(1,1),gr(2,2),gr(3,3),2*gr(1,2),2*gr(1,3),2*gr(2,3)/) 
!
   end subroutine set_cell

!--------------------------------------------------------------------------------------

   subroutine save_cell_bin(unitbin,cell)
!
!  Save cell on binary format
!
   integer, intent(in)            :: unitbin
   real, dimension(:), intent(in) :: cell
!
   write(unitbin)cell(1:6)
!
   end subroutine save_cell_bin

!--------------------------------------------------------------------------------------

   subroutine read_cell_bin(unitbin,cell,err)
!
!  Read cell from binary file
!
   USE errormod
   integer, intent(in)             :: unitbin
   real, dimension(6), intent(out) :: cell
   integer                         :: ier
   type(error_type), intent(out)   :: err
!
   read(unitbin,iostat=ier) cell(1:6)
   if (ier == 0) then
       call set_cell(cell)
   else
       call err%set('Error on reading cell parameters')
   endif
!
   end subroutine read_cell_bin

!--------------------------------------------------------------------------------------

   subroutine save_space_bin(unitbin,space)
   USE spginfom
   integer, intent(in)           :: unitbin
   type(spaceg_type), intent(in) :: space
!   
   write(unitbin)space
!
   end subroutine save_space_bin

!--------------------------------------------------------------------------------------

   subroutine read_space_bin(unitbin,err)
   USE spginfom
   USE errormod
   integer, intent(in)           :: unitbin
   type(error_type), intent(out) :: err
   type(spaceg_type)             :: space
   integer                       :: ier
!
   read(unitbin,iostat=ier)space
   if (ier == 0) then
       call set_spaceg_new(space)
   else
       call err%set('Error on reading space group')
   endif
!
   end subroutine read_space_bin

!--------------------------------------------------------------------------------------

   subroutine set_spaceg(space,ierspace)
   USE Symmetry
   USE General, only: spacegr
   USE spginfom
   character(len=*), intent(in)   :: space
   integer, intent(out), optional :: ierspace
   integer                        :: ishif
   integer, dimension(3)          :: kshif
   character(len=16)              :: spaceg
   integer                        :: ier
!
   ier = 0
   if (len_trim(space) > 0) then
       spaceg(:) = space
       call StandardSpg(spaceg,ier)
       if (ier == 0) then
           ishif = 0
           kshif(:) = 0
           call insym(ishif,kshif,ier)
!corr           totop = pts * float( ( icent + 1 ) * nsym )     ! era nella yfrm02
           spacegr = init_spaceg_type(space) ! set space group in structure spacegr
           call xspinb()  
       endif
   endif
       !write(0,*)'NSYM=',nsym,nsymtot,latt,jsys
   if (present(ierspace)) ierspace = ier
!
   end subroutine set_spaceg

!--------------------------------------------------------------------------------------

   subroutine set_spaceg_new(spg)
   USE spginfom
   USE Symmetry
   USE General, only:cell, spacegr
   USE reflection_type_util
   type(spaceg_type), intent(in) :: spg
   common /ori/ ntra,taboTC(8,3)
   integer :: ntra
   real :: taboTC
!corr   integer                       :: i,j
!
   spacegr = spg   ! save spacegroup in a global variable
!
!  Symbol
   if (len_trim(spg%symbol_xhm) > 0) then
       read(spg%symbol_xhm,'(16a1)') nSpaceG
   elseif (len_trim(spg%symbol_alt(1)) > 0) then
       read(spg%symbol_alt(1),'(16a1)') nSpaceG
   endif
!
!  Numero di operatori e centratura
   nsymTot = spg%nsymop
!corr   totop = spg%nsymop
   pts = spg%ncoper
!corr   ncentr = spg%ncoper
!
   call get_symm_from_spg(spg,cell,kmat,tmat,jsys,latt,nsym,icent)
!
!corr   nSymmStrings = spg%nsymop
!   if (icent == 1) call CheckCentro(nsym)    ! serve????? contiene la xspinb che serve
   if (nsym <= 24) then  ! se nsym > 24 (es. F -4 2 3) supero la dim di ismat/tsmat
       call musym()    ! ismat, tsmat 
       call Preoper()  ! serve
   else  ! dare un warning
   endif
!
!  Direzione polare
!corr   kpol = 0
!corr   vpol(:) = 0
!corr   if (spg%polarx) vpol(1) = 1
!corr   if (spg%polary) vpol(2) = 1
!corr   if (spg%polarz) vpol(3) = 1
!corr   kpol=100*vpol(1)+10*vpol(2)+vpol(3)
!!!
!FIXME: set seminvariants: jsvet, isvet
!!!
!
!  Origini permesse
   call get_origini_spg(spg)  !!!FIXME jsvet
   call CheckRhombAx(nSpaceG, cell, ifRhombAx)
   jspini = -1
   !corrcall xspinb()
   call get_origini_spg(spacegr)
   taboTC(:,:) = tabo  ! copy tabo in taboTC per vecchia to_coord
      !write(0,*)'lattice=',trim(spg%lattyp),latt
!!!???   if3den = ?????
!!!???   call calmod(ngc,nbr,ncs,sem,nse)
!
   end subroutine set_spaceg_new

!--------------------------------------------------------------------------------------

   integer function get_kpol(spg)
   USE spginfom
   type(spaceg_type), intent(in) :: spg
   get_kpol = 0
   if (spg%polarx) get_kpol = 100
   if (spg%polary) get_kpol = get_kpol + 10
   if (spg%polarz) get_kpol = get_kpol + 1
   end function get_kpol

!--------------------------------------------------------------------------------------

   subroutine get_origini_spg(spg)
   USE spginfom
   USE Symmetry
   implicit none
   type(spaceg_type), intent(in) :: spg
   integer                       :: i,j
   integer                       :: kpol
!
!  Origini permesse
!corr   OkOrig = 1
   nOrigRoxi = spg%norig_ind
          !write(0,*)'norig=',nOrigRoxi,spg%norig
   do i=1,nOrigRoxi
      tabo(i,:) = spg%orig_ind(:,i)
      if(spg%polarx .and. spg%polary .and. spg%polarz) then
          tabo(i,:3) = 128
      else if( spg%polarx .and. spg%polary)    then
          tabo(i,1) = 128
          tabo(i,2) = 128
      else if( spg%polarx .and. spg%polarz)    then
          tabo(i,1) = 128
          tabo(i,3) = 128
      else if( spg%polary .and. spg%polarz)    then
          tabo(i,2) = 128
          tabo(i,3) = 128
      else if( spg%polarx)    then
          tabo(i,1) = 256
      else if( spg%polary)    then
          tabo(i,2) = 256
      else if( spg%polarz)    then
          if (jsys == 5 .and. latt == 1) then
              tabo(i,3) = 128
          else
              tabo(i,3) = 256 ! Primitive (latt=1) .and. Hexagonal (jsys=5)
          endif
      end if
   enddo
!
!  per xyzmax
   xyzmax(:) = -9999
   do i=1,nOrigRoxi
      do j=1,3
         if (tabo(i,j) > xyzmax(j)) then
             xyzmax(j) = tabo(i,j)
         endif
      end do
   end do
   do j=1,3
      if (xyzmax(j) < 0.6) then
          xyzmax(j) = 2.0
      elseif (xyzmax(j) < 0.7) then
          xyzmax(j) = 32.0   !  caso 1/3, 2/3 
      elseif (xyzmax(j) < 0.9) then
          xyzmax(j) = 32.0   !  caso 1/6, 5/6 
      elseif (xyzmax(j) < 1.0) then
          xyzmax(j) = 4.0
      endif
   end do
!
!  if3den (code from OriginiRoxi)
   if3den = 0
   kpol = get_kpol(spg)
   if (icent == 1) then
       if (latt == 7 .and. jsys == 5) then  ! Rhombohedrical (latt=7) and Hexagonal (jsys=5)
           if3den = 1
       endif
   else
       if (latt == 1 .and. jsys == 5) then  ! Primitive (latt=1) .and. Hexagonal (jsys=5)
           if (latt == 1) then
               if (kpol == 1) then        ! Floating origin along z
                   if3den = 1 
               else
                   if (jsvet(1) == 128) then
                       if3den = 1 
                   endif
               endif
           endif
       endif
   endif
   end subroutine get_origini_spg

!--------------------------------------------------------------------------------------
 
   subroutine set_cell_and_spg(modif)
   USE General, only:cell, alambda
   USE PatternRef, only:thmin,thmax
   USE proginterface
   USE strutil
   USE spginfom
   USE variables, only:atm
   USE atom_type_util
   USE view
   logical, intent(out) :: modif
   character(18)      :: spacecharnew, spacechar
   real, dimension(6) :: cellnew
   integer            :: kreturn   !!!!, ier
!
   modif = .false.
!
!  Create cell id doesn't exist (possible in case of File > Import Pattern > SA)
   if(.not.is_cell_defined()) call create_cell()
   cellnew(:) = cell(1:6)
!
!  Create s.g. if doesn't exist  
   !corrif (.not.is_space_defined()) call set_spaceg('P 1')
   if (.not.is_space_defined()) call set_spaceg_new(init_spaceg_type())  ! Set as P 1
!
   spacechar = trim(spaceg_string())//char(0)
   spacecharnew = spacechar
   call cell_and_spg_dlg(cellnew, spacecharnew, kreturn)                             
   if (kreturn == 1) then   ! OK pressed
       if (any(abs(cell(1:6) - cellnew(1:6)) > epsilon(1.0))) then
           modif = .true.
           if (allocated(atm)) then
               call coord_in_newcell(atm,cell,cellnew)
           endif
           cell(1:6) = cellnew(1:6)
           call set_cell(cell)
           call generate_reflections(thmin,thmax,alambda)
           call vedinew()
           !write(0,*) 'cell modificata'
       endif
!
       if (upper(spacechar) /= upper(spacecharnew)) then
           modif = .true.
           call set_spaceg_new(init_spaceg_type(spacecharnew))
           !write(0,*) 'g.s. modificato'
       endif
!
!      Set cell and space group as defined by user.
       call set_cell_as_defined()   
       call set_space_as_defined()
   endif
!
   end subroutine set_cell_and_spg

END MODULE sinterface
