module xyz_frm

implicit none

contains

   subroutine read_xyzfile(j_in,cellc,atom,error)
   USE strutil
   USE atom_type_util
   USE errormod
   USE unit_cell
   USE elements
   integer, intent(in)                                       :: j_in    ! unita' di lettura del file
   real, dimension(6), intent(in)                            :: cellc   ! cella corrente
   type(atom_type), allocatable, dimension(:), intent(inout) :: atom    ! atomi in uscita
   type(error_type), intent(out)                             :: error   ! errore in lettura
   integer                                                   :: nline
   character(len=500)                                        :: line,line2
   integer                                                   :: nlong1,nlong2
   real, dimension(50)                                       :: vet
   integer, dimension(50)                                    :: ivet
   integer                                                   :: iv
   integer                                                   :: natom
   integer                                                   :: i
   character(len=10)                                         :: strn
   integer                                                   :: ier
   integer                                                   :: n,nread
!
!  Leggi il numero di atomi dalla prima riga
   nline = 1
   read(j_in,*,err=10,end=10)natom
   if (natom == 0) then
       call error%set('Cannot read this file')
       go to 20
   else
       allocate(atom(natom))
   endif
!
!  Salta riga con commento  
   nline = nline + 1
   read(j_in,*,err=10)
!
!  Leggi natom atomi
   nread = 0   ! conta il numero di atomi letti
   do n=1,natom
      nline = nline + 1
      read(j_in,'(a)',iostat=ier)line
      if (ier < 0) exit
!
!     Leggi simbolo chimico 
      call cutst(line,nlong1,line2,nlong2)
      if (nlong2 > 2 .or. .not.s_is_alpha(line2(:nlong2))) go to 10   
      nread = nread + 1
      atom(nread)%lab = line2(1:2)
!
!     Leggi coordinate frazionarie
      call Getnum(line,vet,ivet,iv)
      if (iv /= 3) go to 10
      atom(nread)%xc = vet(1:3)
   enddo
!  considera che per errore
!  nread potrebbe essere inferiore ad natom
   call resize_atoms(atom,nread)
!
!  Non modificare la cella corrente 
   call cart_to_frac(atom,orthomatrixi_std(cellc))
!
!  Metti in nz il puntatore al file xen
   do i=1,natom
      atom(i)%ptab = pxen_from_specie(atom(i)%lab)
   enddo
!
   return
10 write(strn,'(i0)')nline
   call error%set('Error on reading line '//trim(strn))   

20 continue
!
   end subroutine read_xyzfile

!-------------------------------------------------------------------------------------------------

   subroutine create_xyzfile(filename,jfile,atom,cella,comment)
!
!  Crea un XYZ chemical file format
!   
   USE unit_cell
   USE atom_type_util
   USE fileutil
   character(len=*), intent(in), optional    :: filename  ! se presente apre il file
   integer, intent(in), optional             :: jfile     ! unita' di lettura del file
   type(atom_type), dimension(:), intent(in) :: atom
   real, dimension(6), intent(in)            :: cella
   character(len=*), intent(in)              :: comment
   type(atom_type), dimension(size(atom))    :: atomc
   integer                                   :: num_atoms
   integer                                   :: i
   integer                                   :: ier
   integer                                   :: j_in
!
!  Apertura del file
   if (present(filename)) then
       call get_unit(j_in)
       open(unit=j_in,file=filename,iostat=ier,status='unknown')
   else
       j_in = jfile   ! in questo caso il file e' stato gia' aperto
   endif
!
!  Scrvi il numero di atomi sulla prima riga
   num_atoms = size(atom)
   write(j_in,'(i0)')num_atoms
!
!  Scrivi un commento sulla seconda riga
   write(j_in,'(a)')trim(comment)
!
!  Scrivi simbolo chimico e coordinate cartesiane sulle altre righe
   atomc = atom
   call frac_to_cart(atomc,orthomatrix_std(cella))             !converti in coord. cartesiasane
   do i=1,num_atoms
      write(j_in,'(a2,3(1x,f10.5))')atomc(i)%specie(),atomc(i)%xc
   enddo          
!
   if (present(filename)) close(j_in)
!
   end subroutine create_xyzfile

end module xyz_frm
