 MODULE RIETV

 USE rrestr, only: restraint_type
 USE progtype
 USE atom_basic, only: atom_type
 USE unit_cell, only: cell_type

 implicit none

 type(restraint_type), dimension(:), allocatable :: resvr     ! restraints del rietveld
 type(atom_type), dimension(:), allocatable      :: atm_sav   ! salva atomi durante affinamento
 type(cell_type) :: cellph

 integer, parameter, private :: RIET_REFINE=0, LEBAIL_DECOMP=1, PAWLEY_DECOMP=2

 integer, parameter :: BB_GEOMETRY=1, DS_GEOMETRY=2
 integer, parameter :: diffr_geom = BB_GEOMETRY

 CONTAINS

   subroutine inizializza_rietveld
!
!  Variabili da inizializzare ogni volta che si parte con una nuova struttura
!
   USE RefinecomRef
   USE RRESTR
   USE constraints
   USE calculate_pdp
   character(len=3) :: strb
   integer          :: i
!
!corr   epsmch = epsilon(epsmch)
!corr   epsmchd = epsilon(epsmchd)
!
   call init_refine_cond(rfcond)
!
!  Restraints e constraints
   !call reset_restraints(resvr)
   call clear_constraints(constrv)
!
!  Inizializza parametri di profilo
   call init_function_param(pear,pvoi,tch)
   zero = rparam_type(str='zero')
   scalr = rparam_type(val=1.0,str='scale')
   sdisp%val = 0.0
   stran%val = 0.0
   do i=1,NMAXCOEFB
      write(strb,'(a,i0)')'b',i
      coefbkg(i) = rparam_type(str=strb)
   enddo
!
!  Cella
!corr   cellr(1) = rparam_type(val=1.0,str='a')
!corr   cellr(2) = rparam_type(val=1.0,str='b')
!corr   cellr(3) = rparam_type(val=1.0,str='c')
!corr   cellr(4) = rparam_type(val=1.0,str='alpha')
!corr   cellr(5) = rparam_type(val=1.0,str='beta')
!corr   cellr(6) = rparam_type(val=1.0,str='gamma')
!
   thminbkg = 0
   kabsor=0
   kopcorr = 0
   hklpo = (/0,0,1/)
   gpo = rparam_type(val=1.0,str='G')
!
   end subroutine inizializza_rietveld

!--------------------------------------------------------------------

   subroutine init_refine_cond(cond)
!
!  Default per le condizioni di affinamento
!
   USE RefinecomRef
   USE progtype, only: refine_condition_type
   USE genlsq, only: GAUNEW,LMDER
   USE General, only:ifAutomatic,nrefl,neutro
   USE profile_function
   USE patternref, only:thmax,thmin
   USE Conteggi, only:npunti
   !corr USE Profile
   type(refine_condition_type), intent(out) :: cond
!
   call init_limit_ref(cond%lim,thmin,thmax,nrefl,npunti)   ! limiti 
   cond%algo = GAUNEW              ! tipo di algoritmo
   !cond%algo = LMDER              ! tipo di algoritmo
!corr   cond%typeback = 2               ! tipo di background
   cond%typefun = PEARSON7         ! tipo di funzione di profilo
   !cond%typefun = TCHZ 
!corr   cond%nback = 6                  ! numero di coefficienti di background
!corr   cond%auto_coefb = .false.       ! determina in automatico il numero di coeff. di back.?
   cond%auto_prof = .false.        ! affinamento automatico profilo
   cond%auto_stru = .false.        ! affinamento automatico struttura
   cond%ncauto = 3                 ! num. di cicli di affinamento automatico
   if (ifAutomatic == 1) then
       cond%winmess = .false.      ! messagistica attiva?
   else
       cond%winmess = .true.
   endif
   cond%typewei = 2                ! tipo di peso
   cond%eps = 0.0001               ! parametro di convergenza
   cond%maxcy = 30                 ! numero massimo di cicli
   cond%raction = 0                 ! 0=Rietveld 1=Le Bail 2=Pawley
   !cond%sprint = 'SVW'             ! Opzioni di stampa
   cond%sprint = 'SW'              ! Opzioni di stampa
   cond%riding_model = (neutro == 0)
       !cond%riding_model = .false.
!   
   end subroutine init_refine_cond

!--------------------------------------------------------------------

   subroutine set_rcode_function(pears,psvoig,tchh,sref,code)
!  
!  Definisce codici di affinamento per parametri di profilo.
!  sref deve contenere l'elenco dei paramatri da affinare separati con una virgola. 
!  Se sref e' ALL tutti i parametri della funzione vengono affinati
!  code e' il valore da trasferire in funzione%parametro%rcod
!  
   !corr USE progtype
   USE pearsonf
   USE psvoigtf
   USE tchzf
   type(pearson_function_type), intent(inout), optional :: pears
   type(pvoigt_function_type), intent(inout), optional  :: psvoig
   type(tch_function_type), intent(inout), optional     :: tchh
   character(len=*), intent(in)                       :: sref
   integer, intent(in)                                :: code
   character(len=len_trim(sref)+2)                    :: str
   integer                                            :: kall
!  
   if (index(sref,'all') > 0) then
       str = ' '
       kall = 1
   else
       str = ','//trim(sref)//','
       kall = 0
   endif
!  
   if (present(pears)) then
       if (index(str,',u,') > 0 .or. kall==1) pears%u%rcod = code
       if (index(str,',v,') > 0 .or. kall==1) pears%v%rcod = code
       if (index(str,',w,') > 0 .or. kall==1) pears%w%rcod = code
       if (index(str,',beta0,') > 0 .or. kall==1) pears%b0%rcod = code
       if (index(str,',beta1,') > 0 .or. kall==1) pears%b1%rcod = code
       if (index(str,',beta2,') > 0 .or. kall==1) pears%b2%rcod = code                              
       if (index(str,',asym,') > 0 .or. kall==1) pears%asym%rcod = code
   endif
!  
   if (present(psvoig)) then
       if (index(str,',u,') > 0 .or. kall==1) psvoig%u%rcod = code
       if (index(str,',v,') > 0 .or. kall==1) psvoig%v%rcod = code
       if (index(str,',w,') > 0 .or. kall==1) psvoig%w%rcod = code
       if (index(str,',eta0,') > 0 .or. kall==1) psvoig%eta0%rcod = code
       if (index(str,',eta1,') > 0 .or. kall==1) psvoig%eta1%rcod = code
       if (index(str,',eta2,') > 0 .or. kall==1) psvoig%eta2%rcod = code
       if (index(str,',asym,') > 0 .or. kall==1) psvoig%asym%rcod = code
   endif
!  
   if (present(tchh)) then
       if (index(str,',u,') > 0 .or. kall==1) tchh%u%rcod = code
       if (index(str,',v,') > 0 .or. kall==1) tchh%v%rcod = code
       if (index(str,',w,') > 0 .or. kall==1) tchh%w%rcod = code
       if (index(str,',z,') > 0 .or. kall==1) tchh%z%rcod = code
       if (index(str,',x,') > 0 .or. kall==1) tchh%x%rcod = code
       if (index(str,',y,') > 0 .or. kall==1) tchh%y%rcod = code
       if (index(str,',asym,') > 0 .or. kall==1) tchh%asym%rcod = code
   endif
!  
   end subroutine set_rcode_function

!--------------------------------------------------------------------------------------
    
   subroutine init_limit_ref(limit,tmin,tmax,nref,np)
   USE progtype, only:ref_limit_type
!corr   USE general, only:nrefl
!corr   USE patternref, only:thmax,thmin
!corr   USE Conteggi, only:npunti
   type(ref_limit_type), intent(out) :: limit
   real, intent(in)                  :: tmin,tmax
   integer, intent(in)               :: np,nref
!
!  Defaults
   !corr limit = ref_limit_type(thmin,thmax,nrefl,nrefl,1,nrefl,npunti,npunti,1,npunti)
   limit = ref_limit_type(tmin,tmax,nref,nref,1,nref,np,np,1,np)
!   
   end subroutine init_limit_ref

!--------------------------------------------------------------------------------------

   subroutine set_limit_ref(limit,refold,tthetamax)
   USE progtype, only:ref_limit_type
   USE variables, only:ref
   USE counts, only:ttetaczero   !!!,ttetacount
   USE reflection_type_util
   type(ref_limit_type), intent(inout)             :: limit
   type(reflection_type), dimension(:), intent(in) :: refold
   real, intent(in)                                :: tthetamax
   integer                                         :: oldsize
!
   oldsize = size(ref)
   limit%ttmax = tthetamax
   limit%nrefu = count(refold%tthd <= limit%ttmax .and. refold%tthd >= limit%ttmin)   
   if (limit%nctot > 0) limit%ncu = count(ttetaczero <= limit%ttmax)       
   !limit%ncu = count(ttetacount <= limit%ttmax)       
   !call reallocate(ref,limit%nrefu)
   call resize_reflections(ref,limit%nrefu)
!
!  Controlla che la dimensione di ref non sia stata allargata   
   if (limit%nrefu > oldsize) then
       ref(oldsize+1:limit%nrefu) = refold(oldsize+1:limit%nrefu)
   endif
!   
   end subroutine set_limit_ref
   
!--------------------------------------------------------------------------------------

   subroutine calcola_profilo_leb (Nactot)
   USE RefinecomRef
   USE GENLSQ
   USE variables, only:ref
   USE General, only:knw
   USE pearsonf
   USE psvoigtf
   USE counts
   USE profile_function
!
   integer, intent(in)           :: Nactot
   real, dimension(size(ref)) :: Iobs
   real, dimension(size(ref)) :: fold
   real, dimension(size(ref)) :: Iobsold
   real                          :: scaleRF
   integer                       :: n,ncl
   real                          :: norms
   real                          :: sumfc
   real                          :: tol
   integer                       :: ierp
!
   if (rfcond%raction == 1) then
!
      tol = lebtol
      ncl = 10
      !ncl = 20
      sumfc = sum(ref%fo)
      Iobs = ref%fo**2 
      do n=1,ncl

         Iobsold = Iobs
         fold = ref%fo
         chisquared_old = chisquarednow

         select case (rfcond%typefun)

           case (PEARSON7)
             call foleb_pears(ref,1,rfcond%lim%nrefu,ref%fo**2,Iobs,pear,scalr,ycountoss,ycounttot,ttetaczero,backgr)

           case (PVOIG)
             call foleb_psvoi(ref,1,rfcond%lim%nrefu,ref%fo**2,Iobs,pvoi,scalr,ycountoss,ycounttot,ttetaczero,backgr)

           case (TCHZ)
             call foleb_tchz(ref,1,rfcond%lim%nrefu,ref%fo**2,Iobs,tch,scalr,ycountoss,ycounttot,ttetaczero,backgr)

         end select

         ref%fo = sqrt(Iobs)
         scaleRF = sum(ref%fo) / sumfc
         ref%fo = ref%fo / scaleRF

         call setRInt(ref)
!
         call calcola_profilo(Nactot,ierp)
!
         !if (chisquarednow > 1.05*chisquared_old) then
         if (chisquarednow > tol*chisquared_old) then
             ref%fo = fold  
             call setRInt(ref)
             call calcola_profilo(Nactot,ierp)
             !write(6,*)'dopo ripristino chi=',chisquarednow
             !write(6,*)'prima ripristino chi=',chisquared_old
             exit
         endif
         norms = maxval(abs(Iobsold-Iobs)/max(Iobs,1.0))
         !write(6,*)'Ciclo n.',n,'norm=',norms
         !prova call erreF(ref%fo,Iobs,ref%fc**2)
         !if (knw == 1) rinfo%rfvo = Rvalue(ref%fv,ref%fo)
         if (norms < 0.01) exit
!
      enddo
!
   endif
!
   end subroutine calcola_profilo_leb
   
!--------------------------------------------------------------------------------------
  
   subroutine get_rbragg(reff,rbragg,rf)
!
!  Calcola R-Bragg e R-F
!
   USE RefinecomRef
   USE stat, only:Rvalue
   USE pearsonf
   USE variables, only:ref !!!FIXME
   USE counts
   USE profile_function
   USE reflection_type_util, only: reflection_type
!
   type(reflection_type), dimension(:) :: reff
   real, intent(out)                 :: rbragg,rf
   real, dimension(rfcond%lim%nrefu) :: Fobs2,Iobs
   real, dimension(rfcond%lim%nrefu) :: Fcal2,Ical
   integer                           :: nreff
   integer                           :: i
!
   nreff = size(reff) !rfcond%lim%nrefu
   Fcal2 = reff%fc**2 
   select case (rfcond%typefun)

     case (PEARSON7)
       call foleb_pears(ref,1,nreff,Fcal2,Fobs2,pear,scalr,ycountoss,ycounttot,ttetaczero,backgr)

     case (PVOIG)
       call foleb_psvoi(ref,1,nreff,Fcal2,Fobs2,pvoi,scalr,ycountoss,ycounttot,ttetaczero,backgr)

     case (TCHZ)
       call foleb_tchz(ref,1,nreff,Fcal2,Fobs2,tch,scalr,ycountoss,ycounttot,ttetaczero,backgr)

   end select
   !call erreF(sqrt(Fobs2),Fobs2,Fcal2)
   do i=1,nreff
      Iobs(i) =  reff(i)%m*Fobs2(i)*reff(i)%po*reff(i)%lp*reff(i)%as
      Ical(i) =  reff(i)%m*Fcal2(i)*reff(i)%po*reff(i)%lp*reff(i)%as
   enddo
   rf = Rvalue(sqrt(Fobs2),reff%fc)
   rbragg = Rvalue(Iobs,Ical)
!
   end subroutine get_rbragg
   
  !------------------------------------------------------------------------------------------
#if 0
   subroutine calculate_powder_pattern(rfc,xczero,yc,ref,scal,nwave,ratioint,pear,pvoi,tch,ier)
   USE progtype
   USE pearsonf
   USE psvoigtf
   USE tchzf
   USE profile_function
   USE reflection_type_util
!
   implicit none
   type(refine_condition_type), intent(in)                         :: rfc    ! settings for powder pattern
   real, dimension(:), intent(in)                                  :: xczero    ! 2-theta with zero correction
   real, dimension(:), intent(out)                                 :: yc        ! calculated pattern
   type(reflection_type), dimension(:), allocatable, intent(inout) :: ref       ! reflections
   real, intent(in)                                                :: scal      ! scale
   integer, intent(in)                                             :: nwave     ! 1 or 2 (for second wavelength)
   real, intent(in)                                                :: ratioint  ! ratio of intensity between 2 wavel.
   type(pearson_function_type), intent(in)                         :: pear
   type(pvoigt_function_type), intent(in)                          :: pvoi
   type(tch_function_type), intent(in)                             :: tch
   real, dimension(rfc%lim%nrefu)                                  :: fcstr
   integer                                                         :: posfc,posfr
   integer, intent(out)                                            :: ier
!
   posfc = rfc%lim%ncu
   posfr = rfc%lim%nrefu
   if (rfc%raction > 0) then
       fcstr(:) = ref(:posfr)%fo
   else
       fcstr(:) = ref(:posfr)%fc
   endif
!
   select case (rfc%typefun)
     case (PEARSON7)
       call pearson(pear,ref(:posfr),xczero(:posfc),fcstr,scal,yc,nwave,ratioint,ier)
     case (PVOIG)
       call psvoigt(pvoi,ref(:posfr),xczero(:posfc),fcstr,scal,yc,nwave,ratioint,ier)
     case (TCHZ)
       call psvoigt_tchz(tch,ref(:posfr),xczero(:posfc),fcstr,scal,yc,nwave,ratioint,ier)
     case default
       ier = 1
   end select
!
   end subroutine calculate_powder_pattern
#endif   
  !------------------------------------------------------------------------------------------   
  
   subroutine genera_profilo2(ncfin,nrfin,ycount,sumyco)
   USE refinecomref
   !corr USE profile
   USE variables, only:ref
   USE pearsonf
!
   implicit none
   integer, intent(in), optional :: ncfin,nrfin
   real, dimension(:), intent(out) :: ycount
   real, intent(out)             :: sumyco
   integer                       :: posfc,posfr
!
   posfc = ncfin   
   posfr = nrfin 
!
   call pearson_calc(ref(:posfr),ycount(:posfc),sumyco,gpo)
!
   end subroutine genera_profilo2
   
!-------------------------------------------------------------------

   subroutine calcola_correzioneLP_da_rimuovere(refl)
!
!  Correzione Lorentz-polarizzazione
!      Formula usata :
!                     Lp = (1+kCos(2theta)**2) / (2*Sen(theta)**2*Cos(theta))
!                        = 0.5*(1+kCos(2theta)**2) * Csc(theta)**2*Sec(theta)
   USE RefinecomRef
   USE reflection_type_util, only:reflection_type
   USE General, only:neutro,nsyn,dtor
   type(reflection_type), dimension(:), intent(inout) :: refl
   integer                                            :: numb
   integer                                            :: i
   integer                                            :: kal
   real                                               :: tth,th
   real                                               :: Csc_th, Sec_th, Tan_th, Sin_th
   real                                               :: kpol
!
   numb = size(refl)   
   if (neutro == 1 .or. nsyn == 1) then
       kpol = 0.0
   else
       kpol = 0.8
   endif
   LOOP_KALPHA : do kal=1,kalphar
      do i=1,numb
         tth = refl(i)%tthd*dtor  
         th = 0.5 * tth
         Sin_th = Sin(th); Csc_th = 1.0/Sin_th ; Sec_th = 1.0/cos(th); Tan_th = Sin_th / Cos(th)
         refl(i)%lp = 0.5*(1 + kpol*Cos(tth)**2)*Csc_th**2*Sec_th
      enddo
   enddo LOOP_KALPHA

   end subroutine calcola_correzioneLP_da_rimuovere

!-----------------------------------------------------------------------

   subroutine CalcI(refl)
   USE RefinecomRef
   USE variables, only:atm    
   USE reflection_type_util
   USE atom_type_util
   USE General, only: spacegr,celem,neutro,nsyn
   type(reflection_type), dimension(:), intent(inout) :: refl
!
   refl(:)%lp = LPcorrection(refl%tthd,neutro,nsyn)
   if(numatomtype(atm) > 0) call sfcalc(refl,atm,spacegr,celem)
   refl%po = 1.0
   refl%as = 1.0
   call setRInt(refl)
!
   end subroutine CalcI
   
!-----------------------------------------------------------------------

   subroutine setRInt(refl)
   USE RefinecomRef
   USE PatternRef, only : iriet
   USE reflection_type_util, only : reflection_type
   USE profile_function
   implicit none
   type(reflection_type), dimension(:), intent(in) :: refl
   real, dimension(size(refl))                     :: Icalc
   real                                            :: maxffx
   integer                                         :: i
   real, dimension(size(refl))                     :: fstr
   integer                                         :: num_ref

     rapIcalc(:) = 1.0
       !return
!
   if (iriet == 2) return           !!!!!!!!prova temporanea
   if (rfcond%raction > 0) then
       fstr(:) = refl(:)%fo   
   else
       fstr(:) = refl(:)%fc   
   endif
   num_ref = size(refl)
   do i=1,num_ref
      Icalc(i)=refl(i)%m*(fstr(i)**2)*refl(i)%po*refl(i)%lp*refl(i)%as
   enddo
   maxffx = MAXVAL(Icalc(:num_ref))
   rapIcalc(:num_ref)=1.0+Icalc(:num_ref)/maxffx
!
   end subroutine setRInt

! ----------------------------------------------------------------------

   subroutine define_strategy_struct(codes,strategy,atom,radtype)
   USE setref
   integer, intent(in)                                    :: codes
   type(refinement_strategy_type), intent(inout)          :: strategy
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   integer, intent(in)                                    :: radtype
!   
   select case (codes)
      case (1)
        call strategy%add_stage('XYZ@SEQ',atom,radtype)
        call strategy%add_stage('XYZ',atom,radtype)
        call strategy%add_stage('B@SEQ',atom,radtype)
        call strategy%add_stage('B',atom,radtype)
        call strategy%add_stage('XYZB@SEQ',atom,radtype)
        call strategy%add_stage('XYZB',atom,radtype)
        !call strategy%prn(0)

      case (2)
        call strategy%add_stage('XYZ',atom,radtype)
        call strategy%add_stage('XYZB',atom,radtype)

      case (3)
        call strategy%add_stage('XYZB',atom,radtype)
   end select
!
   end subroutine define_strategy_struct

!--------------------------------------------------------------------

   subroutine define_strategy_prof(strategy,raction)
   USE setref
   USE profile_function
   USE RefinecomRef
   type(refinement_strategy_type), intent(inout) :: strategy
   integer, intent(in)                           :: raction

   select case (raction)
!corr     case (RIET_REFINE, LEBAIL_DECOMP)
!corr       call strategy%add_stage('zero')
!corr       call strategy%add_stage('zero_scale')
!corr       call strategy%add_stage('zero_scale_back')
!corr       call strategy%add_stage('scale_cell')
!corr       call strategy%add_stage('zero_scale_back_cell')
!corr       select case(rfcond%typefun)
!corr         case(PEARSON7)
!corr           call strategy%add_stage('scale_w_beta0_asym')
!corr           call strategy%add_stage('zero_back_scale_cell_w_beta0_asym')
!corr           call strategy%add_stage('scale_u_v_w')
!corr           call strategy%add_stage('zero_back_scale_u_v_w_beta0_asym')
!corr           call strategy%add_stage('back_scale_asym_beta0_beta1_beta2')

     case (RIET_REFINE, LEBAIL_DECOMP)
       call strategy%add_stage('zero')
       call strategy%add_stage('zero_scale')
       call strategy%add_stage('zero_scale_back')
       call strategy%add_stage('scale_cell')
       call strategy%add_stage('zero_scale_back_cell')
       call strategy%add_stage('sd')
       call strategy%add_stage('zero_scale_back_cell_sd')
       call strategy%add_stage('st')
       call strategy%add_stage('scale_st')
       !call strategy%add_stage('zero_scale_back_cell_st')

       select case(rfcond%typefun)
         case(PEARSON7)
           call strategy%add_stage('scale_w_beta0_asym')
           call strategy%add_stage('zero_back_scale_cell_w_beta0_asym')
           call strategy%add_stage('scale_u_v_w')
           call strategy%add_stage('zero_back_scale_u_v_w_beta0_asym')
           call strategy%add_stage('back_scale_asym_beta0_beta1_beta2')

         case(PVOIG)
           call strategy%add_stage('w')
           call strategy%add_stage('w_scale_back_cell_zero')
           call strategy%add_stage('scale_w_eta0_asym')
           call strategy%add_stage('zero_back_scale_cell_w_eta0_asym')
           call strategy%add_stage('scale_u_v_w')
           call strategy%add_stage('zero_back_scale_u_v_w_eta0_asym')
           call strategy%add_stage('back_scale_asym_eta0_eta1_eta2')
         case(TCHZ)
           call strategy%add_stage('w')
           call strategy%add_stage('w_scale_back_cell_zero')
           call strategy%add_stage('scale_w_xp_yp_asym')
           call strategy%add_stage('zero_back_scale_cell_w_xp_yp_zp_asym')
           call strategy%add_stage('scale_u_v_w')
           call strategy%add_stage('zero_back_scale_u_v_w_xp_yp_asym')
           call strategy%add_stage('back_scale_asym_xp_yp_zp')
       end select

     case (PAWLEY_DECOMP)
       call strategy%add_stage('zero_scale')
       call strategy%add_stage('fo4_scale')
       call strategy%add_stage('fo5_scale')
       call strategy%add_stage('zero_scale')
       call strategy%add_stage('fo4_scale')
       call strategy%add_stage('fo5_scale')
       call strategy%add_stage('zero_scale_back')
       call strategy%add_stage('fo4_scale')
       call strategy%add_stage('fo5_scale')
       call strategy%add_stage('scale_cell')
       call strategy%add_stage('fo4_scale')
       call strategy%add_stage('fo5_scale')
       call strategy%add_stage('zero_back_scale_cell')
       call strategy%add_stage('scale_fo4')
       call strategy%add_stage('scale_fo5')

       select case(rfcond%typefun)
         case(PEARSON7)
           call strategy%add_stage('scale_w_beta0_asym')
           call strategy%add_stage('scale_fo4')
           call strategy%add_stage('scale_fo5')
           call strategy%add_stage('scale_zero_back_w_beta0_asym_cell')
           call strategy%add_stage('scale_fo4')
           call strategy%add_stage('scale_fo5')
           call strategy%add_stage('scale_u_v_w')
           call strategy%add_stage('scale_fo4')
           call strategy%add_stage('scale_fo5')
           call strategy%add_stage('scale_zero_back_beta0_asym_u_v_w_fo2')
           call strategy%add_stage('scale_fo4')
           call strategy%add_stage('scale_fo5')
           call strategy%add_stage('scale_back_beta0_beta1_beta2_asym_fo2')
         case(PVOIG)
           call strategy%add_stage('scale_w_eta0_asym')
           call strategy%add_stage('scale_fo4')
           call strategy%add_stage('scale_fo5')
           call strategy%add_stage('scale_zero_back_w_eta0_asym_cell')
           call strategy%add_stage('scale_fo4')
           call strategy%add_stage('scale_fo5')
           call strategy%add_stage('scale_u_v_w')
           call strategy%add_stage('scale_fo4')
           call strategy%add_stage('scale_fo5')
           call strategy%add_stage('scale_zero_back_eta0_asym_u_v_w_fo2')
           call strategy%add_stage('scale_fo4')
           call strategy%add_stage('scale_fo5')
           call strategy%add_stage('scale_back_eta0_eta1_eta2_asym_fo2')
         case(TCHZ)
           call strategy%add_stage('scale_w_xp_yp_asym')
           call strategy%add_stage('scale_fo4')
           call strategy%add_stage('scale_fo5')
           call strategy%add_stage('scale_zero_back_w_xp_yp_asym_cell')
           call strategy%add_stage('scale_fo4')
           call strategy%add_stage('scale_fo5')
           call strategy%add_stage('scale_u_v_w')
           call strategy%add_stage('scale_fo4')
           call strategy%add_stage('scale_fo5')
           call strategy%add_stage('scale_zero_back_xp_yp_asym_u_v_w_fo2')
           call strategy%add_stage('scale_fo4')
           call strategy%add_stage('scale_fo5')
           call strategy%add_stage('scale_back_xp_yp_zp_asym_fo2')
       end select
       call strategy%add_stage('scale_fo4')
       call strategy%add_stage('scale_fo5')
       call strategy%add_stage('scale_fo1')
       call strategy%add_stage('scale_zero_back_cell_all_fo2')
       call strategy%add_stage('scale_fo4')
       call strategy%add_stage('scale_fo6')
       call strategy%add_stage('scale_fo7')

   end select
   call strategy%add_stage('zero_back_scale_cell_all')

   end subroutine define_strategy_prof

!-------------------------------------------------------------------

   subroutine check_bt2(x,atom,constr,btmin,btmax,kaction,sprint,ierror)
!
!  Check thermal factor. If kaction = 1 parameter is blocked
!
   USE constraints
   USE type_constants, only: DP
   USE atom_type_util
!
   implicit none
   real(DP), dimension(:), intent(in)                        :: x            ! parametri affinati
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom
   type(constraint_type), dimension(:), allocatable          :: constr
   real, intent(in)                                          :: btmin,btmax  ! max e min per parametri termici
   integer, intent(in)                                       :: kaction      ! codice per gestione del parametro
   character(len=*), intent(in)                              :: sprint       ! codice di stampa
   integer, intent(out)                                      :: ierror       ! codice di errore
   integer                                                   :: i
   integer                                                   :: nco
   integer                                                   :: codej
!
   ierror = 0
   do i=1,numatomtype(atom)
      nco = par_in_constraint(constr,i,4) 
      if (nco > 0) then
          codej = constr(nco)%ref
      else
          codej = atom(i)%rcod(4)
      endif
      if (codej > 0) then
          if (x(codej) < btmin .or. x(codej) > btmax) then
              if (index(sprint,'V') > 0) then
                  write(6,'(a,i0,a,f0.2,a,a,f0.2,a,f0.2,a)')' WARNING: for atom ',i,' thermal factor ',  &
                        x(codej),' is out of range ','(',btmin,'<-->',btmax,')'
              endif
              if (kaction == 1) then
                  if (nco > 0) then
                      constr(nco)%ref = 0
                  else
                      atom(i)%rcod(4) = 0
                  endif
              endif
              ierror = 1
          endif
      endif
   enddo
!
   end subroutine check_bt2

!-------------------------------------------------------------------

   subroutine check_occupancy(x,atom,constr,kaction,sprint,ierror)
!
!  Check thermal factor. If kaction = 1 parameter is blocked
!
   USE constraints
   USE type_constants, only: DP
   USE atom_type_util
!
   implicit none
   real(DP), dimension(:), intent(in)                        :: x            ! parametri affinati
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom
   type(constraint_type), dimension(:), allocatable          :: constr
   integer, intent(in)                                       :: kaction      ! codice per gestione del parametro
   character(len=*), intent(in)                              :: sprint       ! codice di stampa
   integer, intent(out)                                      :: ierror       ! codice di errore
   integer                                                   :: i
   integer                                                   :: nco
   integer                                                   :: codej
   real, parameter                                           :: ocmin=0.0,ocmax=1.0  ! min and max for occupancy
!
   ierror = 0
   do i=1,numatomtype(atom)
      nco = par_in_constraint(constr,i,5) 
      if (nco > 0) then
          codej = constr(nco)%ref
      else
          codej = atom(i)%rcod(5)
      endif
      if (codej > 0) then
!!!!TODO: check every occ in constraint
          if (x(codej) < ocmin .or. x(codej) > ocmax) then
              if (index(sprint,'V') > 0) then
                  write(6,'(a,i0,a,f0.2,a,a,f0.2,a,f0.2,a)')' WARNING: for atom ',i,' occupancy factor ',  &
                        x(codej),' is out of range ','(',ocmin,'<-->',ocmax,')'
              endif
              if (kaction == 1) then
                  if (nco > 0) then
                      constr(nco)%ref = 0
                  else
                      atom(i)%rcod(5) = 0
                  endif
              endif
              ierror = 1
          endif
      endif
   enddo
!
   end subroutine check_occupancy

!--------------------------------------------------------------------
   subroutine check_fo(x,kaction,ierror)
!
!  Controlla i parametri termici, se kaction = 1 il parametro viene bloccato
!
   USE type_constants, only:DP
   USE refinecomref
   implicit none
   real(DP), dimension(:), intent(in) :: x    
   integer                               :: kaction
   integer, intent(out)                  :: ierror
   integer                               :: nneg
   integer                               :: i
   integer                               :: nkref
!
              return
   nneg = 0
   nkref = 0
   do i=1,rfcond%lim%nrefu
      if (fopar(i)%rcod > 0) then
          nkref = nkref + 1
          if (x(nkref) < 0) then
              if (kaction == 1) fopar(i)%rcod = 0
              nneg = nneg + 1
          endif
      endif
   enddo
   if (nneg > 0) then
       ierror = 2
       write (0,*)'fo negativi:',nneg
   endif
!
   end subroutine check_fo
!--------------------------------------------------------------------
   subroutine calcola_correzioneOP(refl,gmax,jplane,cell)
   USE reflection_type_util, only: reflection_type
   USE Symmetry, only : nsym,kmat
   USE CGEOM
   USE unit_cell
!
   implicit none
   type(reflection_type), dimension(:), intent(inout) :: refl
   real, intent(in)                                   :: gmax
   integer, dimension(3), intent(in)                  :: jplane
   type(cell_type), intent(in)                        :: cell
   integer                :: i,l
   integer, dimension(3)  :: jh1,jh2
   integer                :: numb
   integer                :: mmk
!corr   real                   :: gmax
   real                   :: angolo,sinq,cosq
   real                   :: pref
   real                   :: sumden,denom
   real, dimension(3,3)   :: gr
!
   numb = size(refl)   !!!!!rfcond%lim%nrefu
   gr = matrice_mreciproca(cell%get_par())
!
!corr   gmax=gpo%val
!corr   jplane(:)=hklpo(:)
   LOOP_RIFLESSI : do i = 1 , numb
      jh1(:)=refl(i)%hkl 
      mmk = 0
      sumden = 0
      do l=1,nsym  ! loop per riflessi equivalenti
         jh2(:) = matmul(jh1,kmat(l,:,:))
         angolo = angle_vectors(jh2,jplane,gr) * dtor
         cosq =  cos ( angolo )
         sinq =  sin ( angolo )
         pref=(gmax**2)*(cosq**2)+(sinq**2)/gmax
         if (pref < 0.0) cycle     ! puo verificarsi che pref sia < 0 e pref**(-1.5) = INDEFINITO
         sumden = sumden + pref**(-1.5)
         mmk = mmk + 1
      enddo
      if (mmk > 0)then
          denom = sumden/mmk
      else
          denom=1.0
      endif
      refl(i)%po = denom
   enddo LOOP_RIFLESSI
!
   end subroutine calcola_correzioneOP

!--------------------------------------------------------------------
   subroutine calcola_derivataOP(refl,gmax,jplane,cell,derOP)
   USE reflection_type_util, only: reflection_type
   USE Symmetry, only : nsym,kmat
   USE CGEOM
   USE unit_cell
   USE trig_constants
!
   implicit none
   type(reflection_type), dimension(:), intent(inout) :: refl
   real, intent(in)                                   :: gmax
   integer, dimension(3), intent(in)                  :: jplane
   type(cell_type), intent(in)                        :: cell
   real, dimension(:), intent(out)            :: derOP
   integer                :: i,l
   integer, dimension(3)  :: jh1,jh2   !!!!,jplane
   integer                :: mmk
!   real                   :: gmax
   real                   :: angolo,sinq,cosq
   real                   :: pref,preff
   real                   :: sumden,denom
   real, dimension(3,3) :: gr
!
!corr   if (kopcorr == 0) return
!
   derOP(:)=0.0
!corr   gmax=gpo%val
!corr   jplane(:)=hklpo(:)
   gr = matrice_mreciproca(cell%get_par())
   LOOP_RIFLESSI : do i=1,size(derOP) 
      jh1(:)=refl(i)%hkl 
      mmk = 0
      sumden = 0
      do l=1,nsym  ! loop per riflessi equivalenti
         jh2(:) = matmul(jh1,kmat(l,:,:))
         angolo = angle_vectors(jh2,jplane,gr) * dtor
         cosq =  cos ( angolo )
         sinq =  sin ( angolo )
         pref=(gmax**2)*(cosq**2)+(sinq**2)/gmax
         if (pref < 0.0) cycle     ! puo verificarsi che pref sia < 0 e pref**(-1.5) = INDEFINITO
         preff=2.0*gmax*(cosq**2)-(sinq**2)/(gmax**2)
         sumden = sumden + pref**(-1.5)
!
         derOP(i) = derOP(i) + preff*(pref**(-2.5))
!
         mmk = mmk + 1
      enddo
      denom=1.0
      if (mmk > 0)then
          denom = sumden/mmk
          derOP(i)=(-1.5/mmk)*derOP(i)
      endif
      refl(i)%po = denom
   enddo LOOP_RIFLESSI
!
   end subroutine calcola_derivataOP
! ----------------------------------------------------------------------
   subroutine set_sdval(sdval)
   USE RefinecomRef
   USE variables, only:atm
   USE RRESTR
   USE constraints
   USE type_constants, only: DP
   USE atom_type_util
   USE profile_function
   USE General, only: spacegr
   USE spginfom
   implicit none
   real(DP), dimension(:), intent(in) :: sdval
   integer                               :: j
   integer                               :: i
   integer                               :: nkref
   integer                               :: kk
   integer                               :: katom,kparam
   integer                               :: kp
   real                                  :: coefc
   integer                               :: kcon
   integer                               :: code
   integer                               :: npar
   real                                  :: sumsd
   real(DP), dimension(6)             :: sdcell
!
   nkref = 0
!
!  Fo se pawley
   if (rfcond%raction == 2) then
       do i=1,rfcond%lim%nrefu
          if (fopar(i)%rcod > 0) then
              nkref = nkref + 1
          endif
       enddo
   endif
!
!  Zero
   if (zero%rcod > 0) then
       nkref = nkref + 1
       zero%sd = sdval(nkref)
   endif
!
!  sample displacement
   if (sdisp%rcod > 0) then
       nkref = nkref + 1
       sdisp%sd = sdval(nkref)
   endif
!
!  sample displacement
   if (stran%rcod > 0) then
       nkref = nkref + 1
       stran%sd = sdval(nkref)
   endif
!
!  Background
   if (coefbkg(1)%rcod > 0) then
!corr       do i=1,rfcond%nback
       do i=1,backriet%ncoef
          nkref = nkref + 1
          coefbkg(i)%sd = sdval(nkref)
       enddo
   endif
!
!  Cella
   if(rinfo%refine_cell) then
      do j=1,6
         if (cellph%get_cod(j) > 0) then
             nkref = nkref + 1
             !cellph%sd(j) = sdval(nkref)
             sdcell(j) = sdval(nkref)
         endif
      enddo
      call cellph%set_sdref(real(sdcell),spacegr)
!corr      select case (spacegr%csys_code)
!corr         case (CS_Tetragonal, CS_Hexagonal)  ! a=b/=c
!corr          cellph%sd(2) = cellph%sd(1)
!corr
!corr         case (CS_Trigonal)         
!corr          if (index(spacegr%symbol_xhm,':R') > 0) then   ! Rhombohedral setting: a=b=c,al=bet=gam/=90
!corr              cellph%sd(2:3) = cellph%sd(1)
!corr              cellph%sd(5:6) = cellph%sd(4)
!corr          endif
!corr
!corr         case (CS_Cubic)                     ! a=b=c
!corr          cellph(2:3)%sd = cellph(1)%sd
!corr
!corr      end select

   endif
!
!  Profilo
   select case(rfcond%typefun)
     case (PEARSON7)
       if (pear%u%rcod > 0) then
           nkref = nkref + 1
           pear%u%sd = sdval(nkref)
       endif
       if (pear%v%rcod > 0) then
           nkref = nkref + 1
           pear%v%sd = sdval(nkref)
       endif
       if (pear%w%rcod > 0) then
           nkref = nkref + 1
           pear%w%sd = sdval(nkref)
       endif
       if (pear%b0%rcod > 0) then
           nkref = nkref + 1
           pear%b0%sd = sdval(nkref)
       endif
       if (pear%b1%rcod > 0) then
           nkref = nkref + 1
           pear%b1%sd = sdval(nkref)
       endif
       if (pear%b2%rcod > 0) then
           nkref = nkref + 1
           pear%b2%sd = sdval(nkref)
       endif
       if (pear%asym%rcod > 0) then
           nkref = nkref + 1
           pear%asym%sd =  sdval(nkref)
       endif

     case (PVOIG)
       if (pvoi%u%rcod > 0) then
           nkref = nkref + 1
           pvoi%u%sd = sdval(nkref)
       endif
       if (pvoi%v%rcod > 0) then
           nkref = nkref + 1
           pvoi%v%sd = sdval(nkref)
       endif
       if (pvoi%w%rcod > 0) then
           nkref = nkref + 1
           pvoi%w%sd = sdval(nkref)
       endif
       if (pvoi%eta0%rcod > 0) then
           nkref = nkref + 1
           pvoi%eta0%sd = sdval(nkref)
       endif
       if (pvoi%eta1%rcod > 0) then
           nkref = nkref + 1
           pvoi%eta1%sd = sdval(nkref)
       endif
       if (pvoi%eta2%rcod > 0) then
           nkref = nkref + 1
           pvoi%eta2%sd = sdval(nkref)
       endif
       if (pvoi%asym%rcod > 0) then
           nkref = nkref + 1
           pvoi%asym%sd =  sdval(nkref)
       endif

     case (TCHZ)
       if (tch%u%rcod > 0) then
           nkref = nkref + 1
           tch%u%sd = sdval(nkref)
       endif
       if (tch%v%rcod > 0) then
           nkref = nkref + 1
           tch%v%sd = sdval(nkref)
       endif
       if (tch%w%rcod > 0) then
           nkref = nkref + 1
           tch%w%sd = sdval(nkref)
       endif
       if (tch%z%rcod > 0) then
           nkref = nkref + 1
           tch%z%sd = sdval(nkref)
       endif
       if (tch%x%rcod > 0) then
           nkref = nkref + 1
           tch%x%sd = sdval(nkref)
       endif
       if (tch%y%rcod > 0) then
           nkref = nkref + 1
           tch%y%sd = sdval(nkref)
       endif
       if (tch%asym%rcod > 0) then
           nkref = nkref + 1
           tch%asym%sd =  sdval(nkref)
       endif
   end select

!
!  Scala
   if (scalr%rcod > 0) then
       nkref = nkref + 1
       scalr%sd= sdval(nkref)
   endif
!
!  Orientazione preferenziale
   if (gpo%rcod > 0) then
       nkref = nkref + 1
       gpo%sd = sdval(nkref)
   endif
!
!  Struttura
   if (rinfo%refine_structure) then
       do j=1,numatomtype(atm)
          do kk=1,3 
             if (atm(j)%rcod(kk) > 0) then
                nkref = nkref + 1
                atm(j)%xsd(kk) = sdval(nkref)
             endif
          enddo
          if (atm(j)%rcod(4) > 0) then
              nkref = nkref + 1
              atm(j)%bsd = sdval(nkref)
          endif
          if (atm(j)%rcod(5) > 0) then
              nkref = nkref + 1
              atm(j)%osd = sdval(nkref)
          endif
       enddo
!
       do kcon=1,nconstraints(constrv)
          if (constrv(kcon)%ref > 0) then
              code = constrv(kcon)%code
              select case(code)
                 case (CONSTR_B)   ! constraints on b
                   nkref = nkref + 1
                   do kp = 1,constrv(kcon)%npar
                      katom = constrv(kcon)%vat(kp)
                      atm(katom)%bsd =  sdval(nkref)
                   enddo

                  case (CONSTR_CO)  ! constraints on coord.
                   nkref = nkref + 1
                   do kp = 1,constrv(kcon)%npar
                      katom = constrv(kcon)%vat(kp)
                      kparam = constrv(kcon)%vpar(kp)
                      coefc = constrv(kcon)%coef(kp)
                      atm(katom)%xsd(kparam) = coefc*sdval(nkref)
                   enddo

                  case (CONSTR_SOF)  ! constraints on sof
                   npar = constrv(kcon)%npar
                   sumsd = 0.0
                   do i=1,npar-1  ! loop sui par. indipendenti
                      nkref = nkref + 1
                      katom = constrv(kcon)%vat(i)
                      atm(katom)%osd = sdval(nkref)
                      sumsd = sumsd + sdval(nkref)
                   enddo
!corr                   coefc = 1.0  ! va definito dall'utente in una futura implementazione
                   katom = constrv(kcon)%vat(i)   ! parametro dipendente
                   atm(katom)%och = sumsd
                                     
              end select
          endif
       enddo
   endif
!
   end subroutine set_sdval
!-------------------------------------------------------------------------------
   subroutine printfin(kpr,title)
   USE RefinecomRef
   USE background
   USE RRESTR
   USE variables, only:atm
   USE General, only:cell,lo
   USE atom_type_util
   USE STRUTIL
   USE compare_struct
   USE unit_cell
   USE constraints
   USE profile_function
!
   implicit none
   integer, intent(in)                    :: kpr
   character(len=*), optional, intent(in) :: title
   integer                                :: i
!
   write(kpr,'(/1x,80("="))')
   if (present(title)) then
       write(kpr,'(a)') centra_str(title,80)
   endif
!
   call print_rvalues(kpr,title=.true.)
   select case (rfcond%typewei)
     case (1)
       write(kpr,'(/,1x,a)') 'Weighting scheme: w=1.0'
     case (2)
       write(kpr,'(/,1x,a)') 'Weighting scheme: w=1.0/ycount'
     case (3)
       write(kpr,'(/,1x,a)') 'Weighting scheme: w=1.0/ycount^2'
   end select
   write(kpr,'(1x,a,f0.6,/)') 'Convergence criterium: eps=',rfcond%eps
!
!  Stampa informazioni sul background
!corr   select case (rfcond%typeback)
   select case (backriet%btype)
      case (POLY)
!corr        write(kpr,'(/1x,a,i0)')'Background modelled by a polynomial of order ',rfcond%nback-1
        write(kpr,'(/1x,a,i0)')'Background modelled by a polynomial of order ',backriet%ncoef-1

      case (CHEBY)
!corr        write(kpr,'(/1x,a,i0)')'Background modelled by a Chebyshev polynomial of degree ',rfcond%nback
        write(kpr,'(/1x,a,i0)')'Background modelled by a Chebyshev polynomial of degree ',backriet%ncoef

      case (FOUR_SERIES)
!corr        write(kpr,'(/1x,a,i0)')'Background modelled by a cosine Fourier series of degree ',rfcond%nback
        write(kpr,'(/1x,a,i0)')'Background modelled by a cosine Fourier series of degree ',backriet%ncoef

   end select
!corr   if (rfcond%typeback==POLY) write(kpr,'(9x,a,t20,a)')'Origin',trim(string_sigf(thminbkg,7))
   if (backriet%btype==POLY) write(kpr,'(9x,a,t20,a)')'Origin',trim(string_sigf(thminbkg,7))
!corr   do i=1,rfcond%nback
   do i=1,backriet%ncoef
      write(kpr,'(9x,a,t20,a)')trim(coefbkg(i)%str),trim(string_sigf(coefbkg(i)%val,7))
   enddo
!
!  Zero
   write(kpr,'(/9x,a,t20,a)')trim(zero%str),string_sigf(zero%val,7)
!
!  Sample displacement and transparency
   write(kpr,'(/9x,a,t20,a)')trim(sdisp%str),string_sigf(sdisp%val,7)
   write(kpr,'(/9x,a,t20,a)')trim(stran%str),string_sigf(stran%val,7)
!
!  Scale
   write(kpr,'(/9x,a,t20,a)')trim(scalr%str),string_sigf(scalr%val,7)
!
!  Cella
   write(kpr,'(/)')
   call print_cell(cell,kpr=kpr)
!
!  Volume della cella
!corr   write(kpr,'(9x,a,t20,f0.3)')'volume',cell_volume(cell(1:6))
!
!  Informazioni sul profilo
   select case (rfcond%typefun)

     case (PEARSON7)
       write(kpr,'(/1x,a)') 'Profile shape function: Pearson VII'
       write(kpr,30)pear%u%str,string_sigf(pear%u%val,7)
       write(kpr,30)pear%v%str,string_sigf(pear%v%val,7)
       write(kpr,30)pear%w%str,string_sigf(pear%w%val,7)
       write(kpr,30)pear%b0%str,string_sigf(pear%b0%val,7)
       write(kpr,30)pear%b1%str,string_sigf(pear%b1%val,7)
       write(kpr,30)pear%b2%str,string_sigf(pear%b2%val,7)
       write(kpr,30)pear%asym%str,string_sigf(pear%asym%val,7)

     case (PVOIG)
       write(kpr,'(/1x,a)') 'Profile shape function: Pseudo-Voigt '
       write(kpr,30)pvoi%u%str,string_sigf(pvoi%u%val,7)
       write(kpr,30)pvoi%v%str,string_sigf(pvoi%v%val,7)
       write(kpr,30)pvoi%w%str,string_sigf(pvoi%w%val,7)
       write(kpr,30)pvoi%eta0%str,string_sigf(pvoi%eta0%val,7)
       write(kpr,30)pvoi%eta1%str,string_sigf(pvoi%eta1%val,7)
       write(kpr,30)pvoi%eta2%str,string_sigf(pvoi%eta2%val,7)
       write(kpr,30)pvoi%asym%str,string_sigf(pvoi%asym%val,7)

     case (TCHZ)
       write(kpr,'(/1x,a)') 'Profile shape function: TCH Pseudo-Voigt '
       write(kpr,30)tch%u%str,string_sigf(tch%u%val,7)
       write(kpr,30)tch%v%str,string_sigf(tch%v%val,7)
       write(kpr,30)tch%w%str,string_sigf(tch%w%val,7)
       write(kpr,30)tch%z%str,string_sigf(tch%z%val,7)
       write(kpr,30)tch%x%str,string_sigf(tch%x%val,7)
       write(kpr,30)tch%y%str,string_sigf(tch%y%val,7)
       write(kpr,30)tch%asym%str,string_sigf(tch%asym%val,7)

   end select

30 format (9x,a,t20,a)
!
!  Orientazione preferenziale
   if (kopcorr > 0) then
       write(kpr,'(/,a,3i3/)') '  Preferred orientation on plane: ',hklpo(:)
       write(kpr,'(9x,a,t20,a)')trim(gpo%str),trim(string_sigf(gpo%val,7))
   endif
!
   if (numatomtype(atm) > 0) then
       call print_atoms(atm(:),title='Structure')
       call print_constraints(constrv,atm,lo,.true.)
       call stampa_restraints(kpr,resvr,atm,set_cell_type(cell))
       call to_coord_ref(1)
   endif
!
   write(kpr,'(1x,80("="))')
!
   end subroutine printfin
!----------------------------------------------------------------------------------
   subroutine print_rvalues(kpr,title)
   USE RefinecomRef
   USE General, only:knw
   USE variables, only:ref
   USE stat, only:Rvalue
   integer, intent(in) :: kpr
   logical, intent(in) :: title
   real                :: rbragg,rf,rfvc,rfvo
!
   if (title) then
       write(kpr,'(/1x,a)') 'Agreement factors'
   endif
   write(kpr,"(2x,5h Rp =,f8.3,8h   Rwp =,f8.3,7h   Re =,f8.3,8h   Chi2=,f8.3)") rinfo%rp,rinfo%rwp,errexp,chi2
   write(kpr,"(2x,5h Rp'=,f8.3,8h   Rwp'=,f8.3,7h   Re'=,f8.3,8h   DW  =,f8.3)") errep1,errepw1,errexp1,rinfo%dw
   if (rfcond%raction == 0) then 
!
!      Calcola e stampa RF e R-bragg
       call get_rbragg(ref,rbragg,rf)
       write(kpr,30)rf,rbragg
30     format(3x,'R-structure factor =',f8.3,'   R-Bragg factor     =',f8.3)
!
!      Calcola e stampa R tra veri e calcolati
       if (knw == 1) then
           rfvc = Rvalue(ref%fv,ref%fc)
           write(kpr,"(3x,'RF(true-cal)       =',f8.3)")rfvc
       endif
   else
!
!      Calcola e stampa R tra veri e osservati
       if (knw == 1) then
           rfvo = Rvalue(ref%fv,ref%fo)
           write(kpr,"(3x,'RF(true-obs)       =',f8.3)")rfvo
       endif
   endif
!
   end subroutine print_rvalues
! ----------------------------------------------------------------------
   subroutine print_parameter(param,sigsh,kpr,sd,separ)
   USE progtype
   USE strutil
   USE genlsq, only:shiftx
   type(rparam_type), intent(in)            :: param
   integer, intent(in)                      :: sigsh
   integer, intent(in)                      :: kpr
   logical, intent(in), optional            :: sd
   logical, intent(in), optional            :: separ
   logical                                  :: sep
   real                                     :: shiftp   !!!!!corr,stdval
   logical                                  :: is_std
!
   if (present(separ)) then
       sep = separ
   else
       sep = .false.
   endif
   if (present(sd)) then
       is_std = .true.
   else
       is_std = .false.
   endif
   if (is_std) then
       if (abs(param%sd) >= epsilon(0.0)) then
!       if (param%rcod > 0) then
           write(kpr,'(9x,a,t20,a)')trim(param%str),trim(string_esd(param%val,param%sd))
       else
           write(kpr,'(9x,a,t20,a)')trim(param%str),trim(string_sigf(param%val,7))
       endif
   else
       if (param%rcod > 0) then
           shiftp = shiftx(param%rcod)  
           if (sep) then
               write(kpr,'(9x,a,t20,a,t40,a/)')param%str,string_sigf(param%val,7),trim(string_sigf(shiftp,sigsh))
           else
               write(kpr,'(9x,a,t20,a,t40,a)')param%str,string_sigf(param%val,7),trim(string_sigf(shiftp,sigsh))
           endif
       endif
   endif
!
   end subroutine print_parameter

!--------------------------------------------------------------------

   subroutine azzeraref(atom,cell,spg,constr,zerop)
!
!  Se zerop = .false. l'OP non e azzerata
!
   USE RefinecomRef
   USE constraints
   USE unit_cell
   USE spginfom
   USE atom_type_util
   USE unit_cell
   USE setref
   implicit none
   type(atom_type), dimension(:), allocatable                      :: atom
   type(cell_type), intent(inout)                                  :: cell
   type(spaceg_type), intent(in)                                   :: spg
   type(constraint_type), dimension(:), allocatable, intent(inout) :: constr
   logical, optional, intent(in) :: zerop
   logical                       :: zerogpo
!   
   zero%rcod = 0
   sdisp%rcod = 0
   stran%rcod = 0
   coefbkg%rcod = 0
   scalr%rcod = 0
   if (allocated(fopar)) fopar(:)%rcod = 0
   call cell%set_refine(spg,0)
   call set_rcode_function(pear,pvoi,tch,sref='all',code=0)
   if (present(zerop)) then
       zerogpo = zerop
   else
       zerogpo = .true.
   endif
          ! zerogpo = .true.
   if (zerogpo) gpo%rcod = 0
   !if (numatomtype(atom) > 0 .and. rinfo%refine_structure) then
   rinfo%refine_cell = .false.
   rinfo%refine_structure = .false.
   rinfo%refine_profile = .false.
!
   if (numatomtype(atom) > 0) then
       !call set_rcode_atoms(atom,cell,spg,constr,'XYZBO',0)
       call set_rcode_atoms(atom,cell,spg,constr,'XYZBO',0)
       call refine_constraint(constr,atom,0)
   endif
!
   end subroutine azzeraref

!----------------------------------------------------------------------

   subroutine pawley(code)
   USE progtype
   USE General, only:nrefl
   USE RefinecomRef
   USE genlsq
   integer             :: i
   integer, intent(in) :: code
!
   if (code == 1) then
       allocate(fopar(rfcond%lim%nrefu))
!
!      Associa ad ogni Fo un parametro
       do i=1,rfcond%lim%nrefu
          fopar(i)%rcod = 0
          fopar%str = 'F'
       enddo
!
       rfcond%raction = 2
       rfcond%algo = GAUNEW              ! tipo di algoritmo
!
   else
       deallocate(fopar)
       rfcond%raction = 0
       rfcond%algo = GAUNEW              ! tipo di algoritmo
   endif
!
   end subroutine pawley
!-------------------------------------------------------------------------------

   subroutine printfin_esd(kpr,Nact,refcond,title,sdpar)
   USE RefinecomRef
   USE GENLSQ
   USE type_constants, only: DP
   USE RRESTR
   USE variables, only:atm
   USE compare_struct
   USE General, only:knw,cell
   USE STRUTIL
   !corr USE profile
   USE background
   USE atom_type_util
   USE unit_cell
   USE profile_function
   USE constraints
   USE unit_cell
!
   implicit none
   integer, intent(in)                    :: Nact
   integer, intent(in)                    :: kpr
   type(refine_condition_type)            :: refcond
   character(len=*), intent(in), optional :: title
   integer                                :: i,j
   integer                                :: sigsh
   real, dimension(Nact)                  :: stdpar
   real                                   :: stdval
   integer                                :: ier
   integer, parameter                     :: lenp = 15 
   character(len=lenp), dimension(5)      :: stringp
   integer                                :: kconstr
   integer                                :: codepar
   real(DP), dimension(Nact), intent(in) :: sdpar
   logical :: printsd
!
   write(kpr,'(/1x,80("="))')
   if (present(title)) then
       write(kpr,'(a)') centra_str(title,80)
   endif
!
   call print_rvalues(kpr,title=.true.)
   select case (rfcond%typewei)
     case (1)
       write(kpr,'(/,1x,a)') 'Weighting scheme: w=1.0'
     case (2)
       write(kpr,'(/,1x,a)') 'Weighting scheme: w=1.0/ycount'
     case (3)
       write(kpr,'(/,1x,a)') 'Weighting scheme: w=1.0/ycount^2'
   end select
   write(kpr,'(1x,a,f0.6,/)') 'Convergence criterium: eps=',rfcond%eps
!
   sigsh = 5
!
!  Calcola le std
   !call solve_linlsq(jacob,-funcls,shiftx,ier,grad,hess,.true.,matinv)
   !esdd = chisquarednow/(refcond%lim%ncu+nrestraints(resvr)-Nact)
   !do i=1,Nact
   !   stdpar(i) = sqrt(matinv(i,i)*esdd)
   !enddo
     !call calculate_sd(jacob,funcls,chisquarednow,Nact,sdpar,ier)
     !call set_sd(sdpar)
!corr        write(0,*)'cell esd:',cellr%sd
   stdpar(:) = real(sdpar(:))
   
      !write(0,'(a,*(f10.4))')'SD2=',sdpar
!
!  Stampa informazioni sul background
!corr   select case (rfcond%typeback)
   select case (backriet%btype)
      case (POLY)
!corr        write(kpr,'(/1x,a,i0)')'Background modelled by a polynomial of order ',rfcond%nback-1
        write(kpr,'(/1x,a,i0)')'Background modelled by a polynomial of order ',backriet%ncoef-1

      case (CHEBY)
!corr        write(kpr,'(/1x,a,i0)')'Background modelled by a Chebyshev polynomial of degree ',rfcond%nback
        write(kpr,'(/1x,a,i0)')'Background modelled by a Chebyshev polynomial of degree ',backriet%ncoef

      case (FOUR_SERIES)
        write(kpr,'(/1x,a,i0)')'Background modelled by a cosine Fourier series of degree ',backriet%ncoef

   end select
!corr   if (rfcond%typeback==POLY) write(kpr,'(9x,a,t20,a)')'Origin',trim(string_sigf(thminbkg,7))
   if (backriet%btype==POLY) write(kpr,'(9x,a,t20,a)')'Origin',trim(string_sigf(thminbkg,7))
   printsd = .true.
!
!  Background
   do i=1,backriet%ncoef
      call print_parameter(coefbkg(i),sigsh,kpr,printsd)
   enddo
   write(kpr,*)
!
!  Zero
   call print_parameter(zero,sigsh,kpr,printsd,separ=.true.)
!
!  Sample displacement and transparency
   call print_parameter(sdisp,sigsh,kpr,printsd,separ=.true.)
   call print_parameter(stran,sigsh,kpr,printsd,separ=.true.)
!
!  Scale
   call print_parameter(scalr,sigsh,kpr,printsd)
!
!  Cella
   write(kpr,'(/1x,a)')'Lattice parameters'
!corr   do i=1,6
!corr      cellr(i)%val = cell(i)
!FIXME-cell      call print_parameter(cellr(i),sigsh,kpr,printsd)
!corr   enddo
   !call print_cell(cellph%par,cellph%sd,kpr=lo)
   call cellph%write(kpr)
!
!  Volume della cella
   !vol = cell_volume(cell(1:6))
!corr   vol = cellph%volume()
!corr   stdvol = cell_volume_std(vol,cellr(:)%val,cellr(:)%sd)
!corr   write(kpr,'(9x,a,t20,a)')'volume',trim(string_esd(vol,stdvol))
!
!  Informazioni sul profilo
   select case (rfcond%typefun)

     case (PEARSON7)
       write(kpr,'(/1x,a)') 'Profile shape function: Pearson VII'
       call print_parameter(pear%u,sigsh,kpr,printsd)
       call print_parameter(pear%v,sigsh,kpr,printsd)
       call print_parameter(pear%w,sigsh,kpr,printsd)
       call print_parameter(pear%b0,sigsh,kpr,printsd)
       call print_parameter(pear%b1,sigsh,kpr,printsd)
       call print_parameter(pear%b2,sigsh,kpr,printsd)
       call print_parameter(pear%asym,sigsh,kpr,printsd)

     case (PVOIG)
       write(kpr,'(/1x,a)') 'Profile shape function: Pseudo-Voigt '
       call print_parameter(pvoi%u,sigsh,kpr,printsd)
       call print_parameter(pvoi%v,sigsh,kpr,printsd)
       call print_parameter(pvoi%w,sigsh,kpr,printsd)
       call print_parameter(pvoi%eta0,sigsh,kpr,printsd)
       call print_parameter(pvoi%eta1,sigsh,kpr,printsd)
       call print_parameter(pvoi%eta2,sigsh,kpr,printsd)
       call print_parameter(pvoi%asym,sigsh,kpr,printsd)

     case (TCHZ)
       write(kpr,'(/1x,a)') 'Profile shape function: TCH Pseudo-Voigt '
       call print_parameter(tch%u,sigsh,kpr,printsd)
       call print_parameter(tch%v,sigsh,kpr,printsd)
       call print_parameter(tch%w,sigsh,kpr,printsd)
       call print_parameter(tch%z,sigsh,kpr,printsd)
       call print_parameter(tch%x,sigsh,kpr,printsd)
       call print_parameter(tch%y,sigsh,kpr,printsd)
       call print_parameter(tch%asym,sigsh,kpr,printsd)

   end select
!
   if (rinfo%refine_structure) then
       write(kpr,'(13x,a)')centra_str('x',lenp)//centra_str('y',lenp)//centra_str('z',lenp)//    &
                     centra_str('B',lenp)//centra_str('occ',lenp)
       do i=1,numatomtype(atm)
          do j=1,3
             stringp(j) = ' '
             kconstr = par_in_constraint(constrv,i,j)
             if (kconstr > 0) then            
                 codepar = constrv(kconstr)%ref
                 if (codepar > 0) stdval = stdpar(codepar)*abs(coef_of_constraint(constrv(kconstr),i,j))
             else
                 codepar = atm(i)%rcod(j)
                 if (codepar > 0) stdval = stdpar(codepar)
             endif
             if (codepar > 0) then
                 stringp(j) = trim(string_esd(atm(i)%xc(j),stdval))
             else
                 write(stringp(j),'(f0.6)',iostat=ier)atm(i)%xc(j)
             endif
          enddo
          stringp(4) = ' '
          kconstr = par_in_constraint(constrv,i,4)
          if (kconstr > 0) then            
              codepar = constrv(kconstr)%ref
              if (codepar > 0) stdval = stdpar(codepar)*abs(coef_of_constraint(constrv(kconstr),i,4))
          else
              codepar = atm(i)%rcod(4)
              if (codepar > 0) stdval = stdpar(codepar)
          endif
          if (codepar > 0) then
              stringp(4) = trim(string_esd(atm(i)%biso,stdval))
          else
              write(stringp(4),'(f0.6)',iostat=ier)atm(i)%biso
          endif
          stringp(5) = ' '
          if (atm(i)%rcod(5) > 0) then
              stdval = stdpar(atm(i)%rcod(5))
              stringp(5) = trim(string_esd(atm(i)%och,stdval))
          else
              write(stringp(5),'(f0.6)',iostat=ier)atm(i)%och
          endif
          write(kpr,'(3x,a)')atm(i)%lab//catsvector(stringp)
       enddo
   else
       call print_atoms(atm(:),title='Structure')
   endif
   call stampa_restraints(kpr,resvr,atm,set_cell_type(cell))
!
   write(kpr,'(2x,80("="))')
!
!  Orientazione preferenziale
   if (kopcorr > 0) then
       write(kpr,'(/,a,3i3/)') '  Preferred orientation on plane: ',hklpo(:)
       write(kpr,'(9x,a,t20,a)')trim(gpo%str),trim(string_sigf(gpo%val,7))
   endif
!
   end subroutine printfin_esd

!----------------------------------------------------------------------

   subroutine rietveldexe(refine,init)
   USE RefinecomRef
   USE variables
   USE stat
   USE TestModel
   USE compare_struct
   USE General, only: fname,OUT_FILE
   USE fileutil
   USE progtype, only: DISTRIBUTED_VER
!
   logical, intent(in), optional :: init
   character(len=*), intent(in)  :: refine
   integer                       :: i
!
   if (present(init)) then
       if (init)  call init_rietveld(.true.,.true.,.true.)
   endif
!
!corr   write(0,*)'refine: '//trim(refine)
   select case(trim(refine))
      case ('all')
        rfcond%raction = 0
        rfcond%auto_prof = .true.
        rfcond%auto_stru = .true.
        call riet_new(0)

      case ('structure')
        rfcond%raction = 0
        rfcond%auto_prof = .false.
        rfcond%auto_stru = .true.
        call riet_new(0)

      case ('profile')
        rfcond%raction = 1
        !rfcond%sprint = 'SVW'  !; rfcond%raction = 2   !!test
        rfcond%auto_prof = .true.
        rfcond%auto_stru = .false.
        if (rfcond%raction > 0) then
            call setRInt(ref)
            call Count0()
        endif
!
        if (rfcond%raction == 1) then
            do i=1,3
               rfcond%auto_prof = .true.
               rfcond%auto_stru = .false.
               call init_fobs()
               call riet_new(0)
            enddo
        else
            call pawley(1)
            call init_fobs()
            do i=1,3
               rfcond%auto_prof = .true.
               rfcond%auto_stru = .false.
               call riet_new(0)
            enddo
            call pawley(0)
        endif
        !call scrivi_lista6()
   end select
!
   call exitriet()
   if (.not.DISTRIBUTED_VER) then
       call to_coord_ref(1)
       write(6,'(a,t30,a,3f10.3,a,i6,a,i0,2f10.4)')'TB1'//file_get_name(fname(OUT_FILE)),        &
                              ' R:',rinfo%rwp,rinfo%rp,Rvalue(ref%fv,ref%fo),                      &
                              ' trov:',int(vett_test(3)),'/',int(vett_test(1)),vett_test(4),rmsd
   endif
!
   end subroutine rietveldexe

!-------------------------------------------------------------------------------------------------------------

   subroutine print_structure(atom,constr,kpr)
!
!  Stampa solo i parametri affinati
!
   USE genlsq, only:shiftx, step_length
   USE STRUTIL
   !USE RRESTR
   USE atom_type_util
   !USE RIETV, only: constrv
   USE constraints
   implicit none
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   type(constraint_type), dimension(:), allocatable, intent(in) :: constr
   integer, intent(in)               :: kpr
   integer, parameter                :: lenp = 13  ! lunghezza del campo in cui scrivere i parametri
   character(len=lenp), dimension(5) :: stringp
   character(len=lenp), dimension(5) :: strings
   character(len=20)                 :: stringl
   integer                           :: i,j
   integer                           :: ier
   real                              :: shiftp
   integer                           :: codepar
   integer, dimension(5)             :: id_constr, id_par
   logical, dimension(5)             :: id_constr_ref
   real :: coefc
!
   stringl = ' '
   write(kpr,'(13x,a)')centra_str('x',lenp)//centra_str('y',lenp)//centra_str('z',lenp)//    &
                     centra_str('B',lenp)//centra_str('occ',lenp)
   do i=1,numatomtype(atom)
!
!     Check if par. is in constraint (id_constr,id_par) and if the constraint is refined (id_constr_ref)
      do j=1,5
         id_constr(j) = par_in_constraint(constr,i,j,id_par(j))
         if (id_constr(j) > 0) then
             id_constr_ref(j) = (constr(id_constr(j))%ref > 0) 
         else
             id_constr_ref(j) = .false.
         endif
      enddo
      if (any(atom(i)%rcod > 0) .or. any(id_constr_ref)) then
          stringl = ' '
          write(stringl,'(i0,a)')i,' ('//trim(atom(i)%lab)//')'
!
!         Coordinate
          do j=1,3
             stringp(j) = ' '
!
!            prendo il codice di affinamento del parametro
             if (id_constr(j) > 0) then            
                 codepar = constr(id_constr(j))%ref
                 coefc = constr(id_constr(j))%coef(id_par(j))
             else
                 codepar = atom(i)%rcod(j)
                 coefc = 1.0
             endif
             if (codepar > 0) then
                 !write(stringp(j),'(f0.6)',iostat=ier)atom(i)%xc(j)
                 stringp(j) = r_to_s(atom(i)%xc(j),6)
                 stringp(j) = centra_str(stringp(j),lenp)
                 !write(strings(j),'(f0.6)',iostat=ier)shiftx(codepar)*coefc
                 strings(j) = r_to_s(real(shiftx(codepar)*coefc),6)
                 strings(j) = centra_str(strings(j),lenp)
             else
                 stringp(j) = centra_str('-',lenp)
                 strings(j) = ' '
             endif
          enddo
!
!         Fattori termici
          stringp(4) = ' '
          if (id_constr(4) > 0) then            
              codepar = constr(id_constr(4))%ref
          else
              codepar = atom(i)%rcod(4)
          endif
          if (codepar > 0) then
              !write(stringp(4),'(f0.6)',iostat=ier)atom(i)%biso
              stringp(4) = r_to_s(atom(i)%biso,6)
              stringp(4) = centra_str(stringp(4),lenp)
              if (id_constr(4) > 0) then
                  shiftp = step_length * shiftx(codepar) * constr(id_constr(4))%coef(id_par(4))
              else
                  shiftp = shiftx(codepar)  
              endif
              write(strings(4),'(a)',iostat=ier)trim(string_sigf(shiftp,7))
              strings(4) = centra_str(strings(4),lenp)
          else
              stringp(4) = centra_str('-',lenp)
              strings(4) = ' '
          endif
!
!         Occupanze
          stringp(5) = ' '
!corr          id_constr(5) = par_in_constraint(constr,i,5)
          if (id_constr(5) > 0) then            
              codepar = constr(id_constr(5))%ref
          else
              codepar = atom(i)%rcod(5)
          endif
          if (codepar > 0) then
              !write(stringp(5),'(f0.6)',iostat=ier)atom(i)%och
              stringp(5) = r_to_s(atom(i)%och,6)
              stringp(5) = centra_str(stringp(5),lenp)
              if (id_constr(5) == 0) then
                  !write(strings(5),'(f0.6)',iostat=ier)shiftx(atom(i)%rcod(5))
                  strings(5) = r_to_s(real(shiftx(atom(i)%rcod(5))),6)
                  strings(5) = centra_str(strings(5),lenp)
              else
                  strings(5) = ' '  ! non scrivere lo shift se in constrain 
              endif
          else
              stringp(5) = centra_str('-',lenp)
              strings(5) = ' '
          endif
          write(kpr,'(3x,a)')stringl(1:10)//catsvector(stringp)
          write(kpr,'(13x,a)')catsvector(strings)
      endif
   enddo
!
   end subroutine print_structure

!-------------------------------------------------------------------------------------------------------------

#if 0
   subroutine save_background_bin(unitp)
   USE counts
   integer, intent(in) :: unitp
!
   if (allocated(backgr)) then
       write(unitp) backgr(:)
       if (allocated(pointb)) then
           write(unitp) pointb
       endif
       write(unitp) coefbkg

   endif
!
   end subroutine save_background_bin
#endif
!--------------------------------------------------------------------------------
!corr
!corr   subroutine make_constraints(atom,legm,constr,spg,cell,riding_model,adp_factor)
!corr   USE constraints
!corr   USE unit_cell
!corr   USE spginfom
!corr   USE atom_type_util
!corr   USE setref
!corr   USE connect_mod
!corr!
!corr   implicit none
!corr   type(atom_type), dimension(:), allocatable, intent(inout)       :: atom
!corr   type(bond_type), dimension(:), allocatable, intent(in)          :: legm
!corr   type(constraint_type), dimension(:), allocatable, intent(inout) :: constr
!corr   type(spaceg_type), intent(in)                                   :: spg
!corr   type(cell_type), intent(in)                                     :: cell
!corr   logical, intent(in)                                             :: riding_model
!corr   real, intent(in)                                                :: adp_factor
!corr   integer :: i
!corr!
!corr!  Cancella i constraints di programma
!corr   do i=1,3
!corr      call delete_all_constraints(constr,code=i,typec=2)
!corr   enddo
!corr!
!corr   call set_constr_on_position(constr,atom,spg,cell)
!corr   !call print_lsqcond()
!corr!
!corr   call set_lsq_conditions(atom,spg,cell)  ! necessari per la set_constr_on_sof
!corr   call set_constr_on_sof(atom,constr,cell)
!corr!
!corr   if (riding_model) then
!corr       call add_constraints_for_riding(constr,atom,legm,spg,cell,adp_factor)
!corr   endif
!corr!
!corr!  Isolate independent equations, e.g. B constr + constr of riding; B constr + constr. on site
!corr   call check_constraints(constr)
!corr   !call print_constraints(constr,atom,0,is_all=.true.)
!corr!
!corr   end subroutine make_constraints
!corr
!------------------------------------------------------------------------

   subroutine prerefine_structure(atom,refl,constr,nkref)
   USE RefinecomRef
   USE GENLSQ
   USE atom_type_util
   USE reflection_type_util
   USE constraints
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom
   type(reflection_type), dimension(:), allocatable          :: refl
   type(constraint_type), dimension(:), allocatable          :: constr
   integer, intent(inout)                                    :: nkref
   character(4), dimension(5)                                :: atomstring = ['x   ','y   ','z   ','B   ','occ.']
   character(len=20)                                         :: strat
   integer                                                   :: i,j,kk,iv
   integer                                                   :: kpar
   real                                                      :: bfacmax
   integer                                                   :: katom
   real                                                      :: coefc
   integer                                                   :: npar
!
!  Struttura
   if (rinfo%refine_structure) then
       bfacmax = 35. / maxval(refl%slaq)   ! 35 = 30*ln(10) / 2
       do j=1,numatomtype(atom)
          write(strat,'(i0,a)')j,' ('//trim(atom(j)%lab)//')'
          do kk=1,3        ! x,y,z
             if (atom(j)%rcod(kk) > 0) then
                nkref = nkref + 1
                par_string(nkref) = trim(atomstring(kk))//strat
                xpar(nkref) = atom(j)%xc(kk)
                atom(j)%rcod(kk) = nkref
                typex(nkref) = 10.0
             endif
          enddo
          if (atom(j)%rcod(4) > 0) then
             nkref = nkref + 1
             par_string(nkref) = trim(atomstring(4))//strat
             xpar(nkref) = atom(j)%biso
             atom(j)%rcod(4) = nkref
             typex(nkref) = bfacmax
          endif
          if (atom(j)%rcod(5) > 0) then
             nkref = nkref + 1
             par_string(nkref) = trim(atomstring(5))//strat
             xpar(nkref) = atom(j)%och
             atom(j)%rcod(5) = nkref
          endif
       enddo
   endif
!
   do i=1,nconstraints(constr)
      if (constr(i)%ref > 0) then
          select case (constr(i)%code)
            case (1)  ! constrain sui b, un 1 par. indipendente
              nkref = nkref + 1 
              write(par_string(nkref),'(a,i3)')' co.',i
              !katom = constr(i)%vat(1)
              katom = constr(i)%vat(size(constr(i)%vat))
              xpar(nkref) = atom(katom)%biso

            case (2)  ! constrain sulle coordinate, un 1 par. indipendente
              nkref = nkref + 1
              write(par_string(nkref),'(a,i3)')' co.',i
              katom = constr(i)%vat(1)
              kpar = constr(i)%vpar(1)
              coefc = constr(i)%coef(1)
              xpar(nkref) = atom(katom)%xc(kpar) / coefc

            case (3)  ! constrain sulle occ, npar-1 par. indipendenti
              npar = constr(i)%npar
              do iv=1,npar-1
                 nkref = nkref + 1
                 write(par_string(nkref),'(a,i3,a,i3)')' co.',i,' par.',iv
                 katom = constr(i)%vat(iv)
                 xpar(nkref) = atom(katom)%och
              enddo
              
          end select
          constr(i)%ref = nkref
      endif
   enddo
!
   end subroutine prerefine_structure

!--------------------------------------------------------------------

   subroutine setstagestru_new(rfc,atom,legm,cell,spg,elem,constr,sstage)
   USE elements
   USE unit_cell
   USE atom_type_util
   USE setref
   USE spginfom
   USE connect_mod
   USE constraints
   USE arrayutil
!
   implicit none
   type(refine_condition_type), intent(in)                         :: rfc
   type(atom_type), dimension(:), allocatable, intent(inout)       :: atom
   type(bond_type), dimension(:), allocatable, intent(in)          :: legm
   type(cell_type), intent(inout)                                  :: cell
   type(spaceg_type), intent(in)                                   :: spg
   type(element_type), dimension(:), allocatable, intent(in)       :: elem
   type(constraint_type), dimension(:), allocatable, intent(inout) :: constr
   type(refinement_stage_type), intent(in)                         :: sstage
!
   call calcola_occ(atom,spg,cell)   ! coordinates are modified
   call azzeraref(atom,cell,spg,constr)
   call make_constraints(atom,legm,constr,spg,cell,rfc%riding_model,rfc%adp_factor)
   if (size_array(sstage%vet) > 0) then
       call set_rcode_atoms(atom,cell,spg,constr,sstage%spar,1,zspec=sstage%vet)
   else
       call set_rcode_atoms(atom,cell,spg,constr,sstage%spar,1)
   endif
!
   end subroutine setstagestru_new

!--------------------------------------------------------------------

   subroutine setstageprof_new(stage,cell,spg,refop,rfc,atom,legm,elem,constr,radtype,ref)
   USE strutil
   USE setref
   USE RefinecomRef
   USE spginfom
   USE unit_cell
   USE profile_function
   USE arrayutil
   USE atom_type_util
   USE reflection_type_util
   USE elements
   USE connect_mod
   USE constraints
   implicit none
   type(refinement_stage_type), intent(in) :: stage
   type(spaceg_type), intent(in)           :: spg
   type(cell_type), intent(inout)          :: cell
   integer, intent(in)                     :: refop
   type(refine_condition_type), intent(in)                         :: rfc
   type(atom_type), dimension(:), allocatable, intent(inout)       :: atom
   type(bond_type), dimension(:), allocatable, intent(in)          :: legm
   type(element_type), dimension(:), allocatable, intent(in)       :: elem
   type(constraint_type), dimension(:), allocatable, intent(inout) :: constr
   integer, intent(in)                                             :: radtype
   type(reflection_type), dimension(:), allocatable, intent(in)    :: ref
   character(len=:), allocatable :: sts  
   character(len=10), dimension(100) :: wordv
   integer                                      :: nword,i,k
   type(gref_type), allocatable, dimension(:) :: gref
   
   if (rfc%raction == PAWLEY_DECOMP) then
       call make_ref_groups(ref,gref,sogi=0.6)
   endif

   sts = s_u2b(stage%spar)
   call get_words(sts,wordv,nword)
   do i=1,nword
      wordv(i) = lower(wordv(i))
         !write(6,*)'WORD=',trim(wordv(i))
      select case (wordv(i))
        case ('zero')
           zero%rcod = 1

        case ('sd')
           sdisp%rcod = 1

        case ('st')
           stran%rcod = 1

        case ('scale') 
           scalr%rcod = 1

        case ('back')
           coefbkg%rcod = 1

        case ('cell')
           call cell%set_refine(spg,1)

        case ('u','v','w','asym')
           select case(rfcond%typefun)
             case(PEARSON7)
                call set_rcode_function(pear,sref=wordv(i),code=1)
             case(PVOIG)
                call set_rcode_function(psvoig=pvoi,sref=wordv(i),code=1)
             case(TCHZ)
                call set_rcode_function(tchh=tch,sref=wordv(i),code=1)
           end select

        case ('beta0','beta1','beta2')
           call set_rcode_function(pear,sref=wordv(i),code=1)

        case ('eta0','eta1','eta2')
           call set_rcode_function(psvoig=pvoi,sref=wordv(i),code=1)

        case ('xp','yp','zp')
           call set_rcode_function(tchh=tch,sref=wordv(i)(1:1),code=1)

        case ('all')
           select case(rfcond%typefun)
             case(PEARSON7)
                call set_rcode_function(pear,sref=wordv(i),code=1)
             case(PVOIG)
                call set_rcode_function(psvoig=pvoi,sref=wordv(i),code=1)
             case(TCHZ)
                call set_rcode_function(tchh=tch,sref=wordv(i),code=1)
                call set_rcode_function(tchh=tch,sref='z',code=0)
           end select

        case ('xyz','xyzb','b')
           call calcola_occ(atom,spg,cell)   ! coordinates are modified
           call azzeraref(atom,cell,spg,constr)
           call make_constraints(atom,legm,constr,spg,cell,rfc%riding_model,rfc%adp_factor)
           if (size_array(stage%vet) > 0) then
               call set_rcode_atoms(atom,cell,spg,constr,wordv(i),1,zspec=stage%vet)
           else
               call set_rcode_atoms(atom,cell,spg,constr,wordv(i),1)
           endif

        case ('allp')
           call refine_all(atom,legm,cell,spg,elem,radtype,constr,refop)

        case ('fo1')
           fopar(:)%rcod = 1
        
        case ('fo2')
           do k=1,size(fopar)
              if (fopar(k)%val > 0.000) then
                  fopar(k)%rcod = 1
              else
                  write(6,*)'riflesso non affinato:',k
              endif
           enddo

        case ('fo3')
           where(fopar(:)%val <= 0.000) fopar(:)%rcod = 1
  
        case ('fo4')
           do k=1,size(gref)
              if (gref(k)%code == 1) then
                  fopar(gref(k)%vref(1))%rcod = 1
              endif
           enddo
  
        case ('fo5')
           do k=1,size(gref)
              if (gref(k)%code >= 2) then
                  fopar(gref(k)%vref(:))%rcod = 1
              endif
           enddo
  
        case ('fo6')
           do k=1,size(gref)
              if (gref(k)%code >= 2 .and. gref(k)%code <=4) then
                  !do j=1,gref(k)%code
                  !   if (fopar(gref(k)%vref(j))%val > 0.00)fopar(gref(k)%vref(j))%rcod = 1
                  !enddo
                  fopar(gref(k)%vref(:))%rcod = 1
              endif
           enddo
  
        case ('fo7')
           do k=1,size(gref)
              if (gref(k)%code >= 5) then
                  fopar(gref(k)%vref(:))%rcod = 1
              endif
           enddo

      end select
   enddo
   if (refop > 0) gpo%rcod=1
   
   end subroutine setstageprof_new

!-----------------------------------------------------------------

   subroutine refine_all(atom,legm,cell,spg,elem,radtype,constr,refop)
   USE RefinecomRef, only:rfcond,zero,coefbkg,scalr,gpo,pear,pvoi,tch,sdisp,stran
   USE messagemod
   USE elements
   USE unit_cell
   USE atom_type_util
   USE profile_function
   USE setref
   USE spginfom
   USE constraints
   USE connect_mod
   implicit none
   type(atom_type), dimension(:), allocatable, intent(inout)       :: atom
   type(bond_type), dimension(:), allocatable, intent(in)          :: legm
   type(cell_type), intent(inout)                                  :: cell
   type(spaceg_type), intent(in)                                   :: spg
   type(element_type), dimension(:), allocatable, intent(in)       :: elem
   type(constraint_type), dimension(:), allocatable, intent(inout) :: constr
   integer, intent(in)                                             :: radtype
   integer, intent(in) :: refop
   integer         :: i
!
   call calcola_occ(atom,spg,cell)
   call make_constraints(atom,legm,constr,spg,cell,rfcond%riding_model,rfcond%adp_factor)
   zero%rcod = 1
   sdisp%rcod = 1
   stran%rcod = 1
   coefbkg%rcod = 1
   scalr%rcod = 1
   if (refop > 0) gpo%rcod=1
   call cell%set_refine(spg,1)
   select case (rfcond%typefun)
      case (PEARSON7)
        call set_rcode_function(pear,sref='all',code=1)
      case (PVOIG)
        call set_rcode_function(psvoig=pvoi,sref='all',code=1)
      case (TCHZ)
        call set_rcode_function(tchh=tch,sref='all',code=1)
        call set_rcode_function(tchh=tch,sref='z',code=0)
   end select
!
!  Refine all species excluded H-atoms
   do i=1,numelem(elem)
      !if ((neutro == 0) .and. (elem(i)%z /= H_at)) then
      if ((radtype == RX_SOURCE) .and. (elem(i)%z /= H_at)) then
          call set_rcode_atoms(atom,cell,spg,constr,'XYZB',1,zspec=[elem(i)%z])
      endif
   enddo
   !msgpar = 'all parameters'
   !write(pgstring(kstager),'(i3,a,a20)')kstager,'-',msgpar
   !write(msginfo,'(a,i0,a,i0,a)')'Automatic profile refinement (step ',kstager,'/',kstagemax,').'
   !write(msginfo,'(a,i0,a,i0,a)')'Automatic structure refinement (step ',kstager,'/',kstagemax,').'
   !call write_message(trim(msginfo),pos=2)
!
   end subroutine refine_all

!-----------------------------------------------------------------

   function sample_displacement(tt,dis,geom) result(corr)
!
!  2theta = -2s*cos(theta)*(180/pi)/R = dis*cos(theta)*(180/pi)  [in degree]
!
   USE trig_constants
   real, dimension(:), intent(inout) :: tt   ! 2theta in degree
   real, intent(in)                  :: dis  ! refined parameter = -2s/R
   integer, intent(in)               :: geom ! geometry
   real, dimension(size(tt))         :: corr
!
   select case (geom)
     case (BB_GEOMETRY)
       corr = dis * cos(dtor*tt/2) * rtod

     case (DS_GEOMETRY)    !!!TODO
   end select
!
   end function sample_displacement

!-----------------------------------------------------------------

   function sample_transparency(tt,trasp,geom) result(corr)
!
!  2theta = (1/2*mu*R)*sin(2theta)*(180/pi) = trasp*sin(2theta)*(180/pi)  [in degree]
!
   USE trig_constants
   real, dimension(:), intent(inout) :: tt    ! 2theta in degree
   real, intent(in)                  :: trasp ! refined parameter = 1/2*mu*R
   integer, intent(in)               :: geom  ! geometry
   real, dimension(size(tt))         :: corr
!
   select case (geom)
     case (BB_GEOMETRY)
       corr = trasp * sin(dtor*tt) * rtod

     case (DS_GEOMETRY)    !!!TODO
   end select
!
   end function sample_transparency

 END MODULE RIETV

!------------------------------------------------------------------------------------------
   subroutine rietveld()
   USE RefinecomRef
   USE variables, only:atm,lconn
   USE Molcom
   USE General
   USE genlsq
   USE ilmviewmod
   USE atom_type_util
   USE sinterface
   USE enable_amb
   USE rietv
   USE unit_cell
!
   implicit none
   character(16) :: SpaceGroup
   logical       :: modif
   type(rparam_type), dimension(6) :: cellg
!
   if (jscreen == 0 .or. ifAutomatic == 1) then   
!      Rietveld run without GUI or in automatic mode
       call rietveldexe(refine='all',init=.true.)
       return
   endif
!
   if (.not.is_cell_defined_list()) then
       call set_cell_and_spg(modif)
       if (modif .and. jav_opened()) then
           call ilmview_draw(atm,lconn)
       endif
   endif
!
   if (numatomtype(atm) == 0) then
       call open_file_selector('Import starting model for Rietveld Refinement'//char(0))
       return
   endif
!
   call init_rietveld(.true.,.true.,.true.)
!
   if (.not. jav_opened()) call ilmview_open(StructureName)
   call ilmview_draw(atm,lconn)
   call jav_set_modify(3)   ! abilita solo i menu' di info
!
   SpaceGroup = spaceg_string()
!
!  Apri finestra grafica
!corr   call getputvar(1)
   call abilita_tasti('rietveld')
   cellg(:)%val =  cellph%get_par()
   cellg(:)%rcod = cellph%get_cod()
   call open_rietdlg(atm, rfcond, coefbkg, zero, scalr, cellg,   &
              numatomtype(atm), pear, pvoi, tch, gpo, kopcorr, hklpo,   &
              SpaceGroup // char(0), backriet)
!
   end subroutine rietveld

!------------------------------------------------------------------------------------------
#if 0
   subroutine getputvar(code)
   USE refinecomref
   USE General, only:cell
!
   implicit none
   integer, intent(in) :: code
!
   if (code == 0) then                 !prendi variabili dalla grafica
!
!      Cella
       cell(:6) = cellr(:)%val
              
   elseif (code == 1) then             !passa variabili alla grafica
!
!      Cella
       cellr%val = cell(:6)
       
   endif
   end subroutine getputvar
#endif
  !-------------------------------------------------------------------------

   subroutine init_rietveld(kprin,initfo,scalefo)
   USE RefinecomRef
   USE PatternRef
   USE Molcom
   USE General
   USE PhaseModule
   USE Symmetry
   USE Atomi
   USE GENLSQ
   USE RIETV
   USE PRNUTIL
   USE VIEW
   USE Counts
   USE background
   USE variables, only:atm,ref,lconn
   USE RRESTR
   USE constraints
   USE unit_cell
   USE atom_type_util
   USE reflection_type_util
   USE messagemod
   USE compare_struct
                     USE Riflessi
   USE proginterface
   USE strutil
   USE profile_function
   USE Conteggi, only: npunti
   USE setref
   USE pointmod
   USE calculate_pdp
!
   implicit none
   logical, intent(in) :: kprin
   logical, intent(in) :: initfo
   logical, intent(in) :: scalefo
   integer             :: nsize
   integer             :: posini,posfin
   integer             :: ier
   integer             :: ierp
   integer             :: i
   type(cell_type) :: cellt
!
   if (kprin) call write_title(lo,(/'Rietveld Refinement'/))
!
   iriet = 1
   ncytot = 0   
   call clear_constraints(constrv)
!  
   kalphar = 1
!
   if (numatomtype(atm) > 0) then
       call delete_ghosts(atm,lconn)
       nats = numatomtype(atm)
   endif
   call cellph%set(cell)
   call cellph%set_refine(spacegr,0)
!
   if (iFirstRietveld == 0) call leggi_profilo(ier)
!
!  I riflessi
   if (.not.allocated(ref) .or. iFirstRietveld == 0) then
       if  (nrefl == 0) then
            call generate_reflections(thmin,thmax,alambda)
            call AllocaRiflessi(ALLOCA)  !!! forse non serve
       endif
       if (knw > 0) then
           do i=1,nrefl
              ipub(i) = IndHKL(i,5)/32
              khl(1:3,i) = IndHKL(i,1:3)
           enddo
       endif
       call fillref(ref)
       call calculate_2theta(ref,cell,alambda)
       if (npunti == 0) then
           thmin = minval(ref(:)%tthd)
           thmax = maxval(ref(:)%tthd)
       endif
   endif
   ref(:)%pk = 20
!corr   call set_rcode_cell(cellr,0)
!
   call AllocaRietnew(ALLOCA,nrefl)
   if (npunti > 0) call Allocafuncls(ALLOCA,npunti+nrestraints(resvr))
!
!corr   if (knw==1 .or. ModelToCompare /= 0)call AllocaTestStuff(1)
   if (iFirstRietveld == 0) call inizializza_rietveld()
!
   if (npunti > 0) then
       if(iFirstRietveld == 0) then
          iFirstRietveld = 1
          posini = 1
          posfin = rfcond%lim%ncu  
          !call genera_background     &
          !(ttetaczero(posini:posfin),ycountoss(posini:posfin),backgr(:posfin-posini+1),   &
          ! coefbkg%val,thminbkg,pointb,npointback,backriet)
          !   !call bruckner_back(ycountoss(posini:posfin),backgr(:posfin-posini+1),40,4)
!!!!!!!!!!!!!! temporary
           call backrtv%set_cond(backriet)   !!!!FIXME
           call backrtv%make(ttetaczero(posini:posfin),ycountoss(posini:posfin),backgr(:posfin-posini+1))
           call backrtv%get_points(pointb); npointback = numpoints(pointb)
           thminbkg = backrtv%zeroback()
           call backrtv%get_coef(coefbkg%val)
           call backrtv%get_cond(backriet)
!!!!!!!!!!!!!! temporary
          call CalcI(ref)
          call setwei
          call Count0()
          if (initfo) call init_fobs()
!         during rietveld ref. scale Fo on Fc scale to avoid scale problem switching from and to LeBail refinement
          if (scalefo .and. numatomtype(atm)>0) ref%fo = ref%fo*(sum(ref%fc)/sum(ref%fo))  
!
!         leggi direttive di rietveld solo la prima volta
          call riet_directories()
!
       else
          posini = 1
          posfin = rfcond%lim%ncu  
   !       ttetaczero(1:rfcond%lim%ncu) = ttetacount - 2.0 * zero%val
          call CalcI(ref)
          call setwei
          !call Count0()
   !       call Count2()
          call calculate_powder_pattern(rfcond,ttetaczero,ycounttot,ref,scalr%val,kalphar,0.0,pear,pvoi,tch,ierp)
          call erreacc(0)
          if (numatomtype(atm) > 0) call set_rcode_atoms(atm,set_cell_type(cell),spacegr,constrv,'XYZBO',0)  ! reset ref code for atoms, shared with SA
       endif
       !call make_ref_groups(ref,gref)
!
       if (kprin) then
           call init_messages()
           call write_message('Rietveld Refinement',pos=1)
           call write_message('Rp=',rnum=rinfo%rp,pos=3)
           call write_message('Rwp=',rnum=rinfo%rwp,pos=2)
!
!          Disegna pattern
           if (jscreen /= 0) then
               kcount_status = 0; jjj=100
               call vedinew(8,1,rfcond%lim%nctot,posini,posfin)
           endif
!
!          questa routine modifiche le coord. e va evitata in SA
           if(numatomtype(atm) > 0) then
              if (knw == 1 .or. ModelToCompare /= 0) then
                  call to_coord_ref(-1)
                  call write_message_found(4)
              endif
              cellt = set_cell_type(cell)
              call calcola_occ(atm,spacegr,cellt)
              call make_constraints(atm,lconn,constrv,spacegr,cellt,rfcond%riding_model,rfcond%adp_factor)
           endif
       endif
   endif
!
   call defco(nsize)
   call inptf
   call presearch
!!!!!   call to_coord(1)
!!!!prova
!             call abs_corr()
!             call calcola_correzioneAbs(1)
!!!!prova
!
   end subroutine init_rietveld

!-------------------------------------------------------------------

   subroutine riet_directories()
   USE General, only:card
   USE comandiexpo
   USE rrestr
   USE variables, only:atm
   USE strutil
   USE RIETV, only:resvr
   USE profile_function
   USE commandsmod
   implicit none
   integer :: i
   integer :: icmax
   integer :: icom
   integer :: ier
   type(restraint_type) :: res
   character(len=80) :: strdir,word
   integer           :: nlongs,nlongw
   real, dimension(100)                        :: vet
   integer, dimension(100)                     :: ivet
   integer                                     :: iv
!
!  Controlla se è stato dato il comando '%rietveld' (code=24)
   icom = 0
   do i=1,size(icomq(:,:),1)
      if (icomq(i,1) == RIET_CMD) then
          icom = i
          exit
      elseif (icomq(i,1) == 0) then
          exit
      endif
   enddo
!
!  Leggi le direttive
   if (icom /= 0) then
       icmax = icomq(icom,2)
       do i=1,icmax
          ipdir = ipdir + 1
          strdir = trim(card(ipdir))
          call Cutst(strdir,nlongs,word,nlongw)
          nlongw = min(4,nlongw)     ! bastano 4 caratteri per individuare la direttiva
          word = upper(word)
          select case(word(:nlongw))
             case ('LOAD')        ! leggi i restraints da file esterno
               call read_restraints_from_file(strdir,resvr)

             case ('WEID')        ! peso distanze
               call Getnum(strdir(:nlongs),vet,ivet,iv)
               if (iv > 0) wqq(1) = vet(1)

             case ('WEIA')        ! peso angoli
               call Getnum(strdir(:nlongs),vet,ivet,iv)
               if (iv > 0) wqq(2) = vet(1)

             case ('WEIP')        ! peso piani
               call Getnum(strdir(:nlongs),vet,ivet,iv)
               if (iv > 0) wqq(3) = vet(1)

             case default
               call get_restraint_from_string(card(ipdir),atm(:),res,ier)
               if (ier == 0) then
                   call add_restraint_to_list(resvr,res)       
              !     !!!!nrest = size(resvr)
              ! else
              !     write(0,*)'error on reading line:'//trim(adjustl(card(ipdir)))
               endif
          end select
       enddo
   endif
!
   end subroutine riet_directories
!-------------------------------------------------------------------
   subroutine finishriet( catm, rfc, ccbkg, czero, cscal, ccell,  &
                          cpear, cpvoi, ctch, cgpo, ckopcorr, chklpo, backc)
   USE RefinecomRef
   USE PatternRef
   USE progtype
   USE variables, only:atm
   USE RIETV
   USE General, only:is_extraction
   USE background
   USE constraints
   USE atom_type_util
             USE rrestr
   implicit none
   type (refine_condition_type) :: rfc   
   type(rparam_type), dimension(NMAXCOEFB) :: ccbkg 
   type(rparam_type) :: czero 
   type(rparam_type) :: cscal 
   type(rparam_type) :: cgpo 
   type(rparam_type) :: ccell(6)
   type(pearson_function_type) :: cpear 
   type(pvoigt_function_type) :: cpvoi 
   type(tch_function_type) :: ctch 
   type(atom_type)               ::catm(*)
   integer                      :: i, ckopcorr
   integer                      :: chklpo(3)
   integer                      :: refcod
   type(back_condition_type) :: backc
!
   !call getputvar(0)       
!!!!!prova - pawley
        !if(rfcond%raction == 1) call pawley(1)
   refcod = gpo%rcod
   call riet_new(refcod)
        !if(rfcond%raction == 2) call pawley(0)
   !call getputvar(1)
   is_extraction = is_extraction .or. rfcond%raction > 0
!
!mr  restituisco i valori al C
   do i=1, numatomtype(atm)
      catm(i) = atm(i)
   end do
!!mr   rfc  = rfcond
   czero = zero
   cscal = scalr
   cpear = pear
   cpvoi = pvoi
   ctch  = tch
   ckopcorr = kopcorr
   do i=1, NMAXCOEFB
      ccbkg(i) = coefbkg(i)
   end do
!corr   do i=1, 6
!corr      ccell(i) = cellr(i)
!corr   end do
   ccell(:)%val = cellph%get_par()
   ccell(:)%rcod = cellph%get_cod()
   cgpo = gpo
   do i=1, 3
      chklpo(i) = hklpo(i)
   end do
   backriet = backc
!
   end subroutine finishriet
!-------------------------------------------------------------------
   subroutine ripassa_valori( catm, rfc, ccbkg, czero, cscal, ccell,  &
                              cpear, cpvoi, ctch, cgpo, ckopcorr, chklpo, backc)
   USE RefinecomRef
   USE PatternRef
   USE atom_basic, only: atom_type
   USE variables, only:atm
   USE General, only:cell
   USE background
   USE atom_type_util
   USE profile_function
   USE RIETV
   implicit none
   type (refine_condition_type) :: rfc   
   type(rparam_type), dimension(NMAXCOEFB) :: ccbkg 
   type(rparam_type) :: czero 
   type(rparam_type) :: cscal 
   type(rparam_type) :: cgpo 
   type(rparam_type) :: ccell(6)
   type(pearson_function_type) :: cpear 
   type(pvoigt_function_type) :: cpvoi 
   type(tch_function_type) :: ctch 
   type(atom_type)               ::catm(*)
   integer                      :: i, ckopcorr
   integer                      :: chklpo(3)
   type(back_condition_type) :: backc
!
   rfcond = rfc
   do i=1, numatomtype(atm)
      atm(i) = catm(i)
   end do
   call cellph%set(ccell%val)
   do i=1, 6
!!!!!FIXME - refinement code for cell
      cell(i) = ccell(i)%val
      call cellph%set_cod(i,ccell(i)%rcod)
!corr      cellr(i) = ccell(i)
   end do
   do i=1, NMAXCOEFB
      coefbkg(i)%rcod = ccbkg(i)%rcod
   end do
   kopcorr = ckopcorr
   zero = czero
   scalr = cscal
   gpo = cgpo
   do i=1, 3
      hklpo(i) = chklpo(i)
   end do
   select  case (rfcond%typefun)
   case (PEARSON7)
         pear = cpear
   case (PVOIG)
         pvoi = cpvoi
   case (TCHZ)
         tch = ctch
   end select
   backriet = backc

   end subroutine ripassa_valori
!-------------------------------------------------------------------
   subroutine exitriet()
   USE RefinecomRef
   USE PatternRef
   USE General
   USE Molcom
   USE progtype, only:DEALLOCA
   USE genlsq
   USE grexpomod
   USE RIETV
   USE variables
   USE drawatoms, only:draw_atoms
   USE ilmviewmod
   USE enable_amb
   USE messagemod
   USE exportfiles
   USE profile_function
   USE atom_type_util
!
   implicit none
   integer :: iExCode
!
   call clear_plot(grplot,1)
   call gr_close_viewer()
!
!!   if (jscreen > 0 .and. ifAutomatic == 0) then
!!       Msg = 'Do you want to save parameters?'
!!       call MsgWinErr('WARNING',Msg,QUEST_WINDOW_YES_NO,iExCode)
!!   else
       iExCode = 1
!!   endif
!
   if (iExCode.eq.1) then
       iFirstRietveld = 1
       if (numatomtype(atm) > 0) then
           call scrivi_lista15()
           call export_files()
       endif
       cell(1:6) = cellph%get_par()
       call scrivi_cella
!prova
       !call scrivi_lista6  ! i riflessi
!prova
   endif
   call AllocaRietnew (DEALLOCA,0)
   call Allocafuncls(DEALLOCA)
!corr   call AllocaTestStuff(2)
!  
   if (jscreen > 0 .and. numatomtype(atm) > 0) then
       call draw_atoms(atm,lconn)
       call jav_set_modify(1)
   endif
!
   call clear_messages()
!
   kcount_status = 1
   iriet = 0
   call abilita_tasti('structure+pattern')
!
   end subroutine exitriet
!-------------------------------------------------------------------
   subroutine riet_new(refop)
   USE RefinecomRef
   USE General, only: ERR_WINDOW, knw, cell
   USE Molcom, only : ModelToCompare
   USE RRESTR, only:nrestraints
   USE GENLSQ
   USE Counts
   USE progtype, only:ALLOCA,DEALLOCA
   USE compare_struct
   USE variables, only:ref
   USE grexpomod, only:fortran_work
   USE RIETV
   USE enable_amb
   USE messagemod
   USE ilmviewmod
   USE unit_cell
!
   implicit none
   integer, intent(in) :: refop  ! se 1 OP e' incluso nell'affinamento automatico
   integer             :: nrefuser
   integer             :: nExitCode
   logical             :: kautomatic
   integer             :: i
   integer             :: exitcode
   integer             :: numres
!
   if (kopcorr == 1) call calcola_correzioneOP(ref(:rfcond%lim%nrefu),gpo%val,hklpo,set_cell_type(cell))
   kautomatic = rfcond%auto_prof .or. rfcond%auto_stru
!
   call jav_set_modify(0)   ! disabilita menu'
   call abilita_tasti('run')
   if (kautomatic) then
       if (ncytot == 0 .and. index(rfcond%sprint,'S') > 0)  &
       call printfin(6,'Starting Rietveld parameters')    ! stampa parametri+restraints
       fortran_work = .true.
!!!!!!!       refop = gpo%rcod  ! se 1 l'op e' inclusa nell'aff. automatico
       call progress_bar_init('Starting refinement')
       do i=1,rfcond%ncauto
          ncurrent_cycle = i
          call gesref(exitcode,refop)
          if (exitcode == 2) exit
       enddo
       call progress_bar_close()
       if (knw == 1 .or. ModelToCompare /= 0) then
           call to_coord_ref(-1)
           call write_message_found(9)
       endif
       rfcond%auto_stru = .false.
       rfcond%auto_prof = .false.
       fortran_work = .false.
   else
       call conta_parametri(nrefuser,rfcond)
       if (nrefuser == 0)  then        ! Se non affino niente ...
           call MsgWinErr('ERROR','User Refinement Stage. Please set the strategy before going on !',ERR_WINDOW,nExitCode)
           return
       else
           if (ncytot == 0 .and. index(rfcond%sprint,'S') > 0)  &
               call printfin(6,'Starting Rietveld parameters')    ! stampa parametri+restraints
!
!          Se nel frattempo ho introdotto attraverso la grafica dei restraints devo riallocare funcls
           numres = nrestraints(resvr)
           if (size(funcls) /= rfcond%lim%ncu + numres .and. rinfo%refine_structure) then
               call Allocafuncls(ALLOCA,rfcond%lim%ncu+numres)
               call setwei() ! il peso va riempito nuovamente
           endif
           call UserRef()
           if (knw == 1 .or. ModelToCompare /= 0) then
               call to_coord_ref(-1)
               call write_message_found(9)
           endif
       endif
   endif
   call abilita_tasti('rietveld')
   call jav_set_modify(3)   ! disabilita menu'
!
   end subroutine riet_new
!--------------------------------------------------------------------
   subroutine checkOP(kop,cell,ier)
   USE RefinecomRef
   USE General, only: lo
   USE unit_cell
   USE variables, only:ref
   USE CGEOM
   USE unit_cell
   USE prog_constants
   implicit none
   integer, intent(in)  :: kop
   type(cell_type), intent(in) :: cell
   integer, intent(out) :: ier
   integer              :: i
   real                 :: angolo
   integer              :: nExitCode
   real, dimension(3,3) :: gr
   character(len=300)   :: msg
!
!  Controllo per il piano di tipo 0,0,0
!
   if (sum(abs(hklpo(:))) == 0) then
       ier = -1
   else
!
!     Controlla che il piano esista
      gr = matrice_mreciproca(cell%get_par())
      do i=1,rfcond%lim%nrefu    
         angolo = angle_vectors(ref(i)%hkl,hklpo(:),gr)
         if (angolo < 1.0e-6 .or. abs(angolo-180.) < 1.0e-06) then
             write(lo,*)i,angolo,ref(i)%hkl
             ier = 0
             return
         endif
      enddo
      ier = -1
   endif
   if (ier == -1) then
!corr       Inte = 'Rietveld Refinement'
       write(msg,100) kop,hklpo(:)
   100     format('Error on Preferred Orientation Plane',1x,i3,1x,'(',3(i3,1x),')')
       call MsgWinErr('Rietveld Refinement',Msg,WARN_WINDOW,nExitCode)
   endif
!
   return
   end
!--------------------------------------------------------------------
   subroutine setwei
   USE RefinecomRef
   USE GENLSQ
   USE Counts
   implicit none
   integer :: i
!
   select case (rfcond%typewei)

     case(1)
       weilsq(:rfcond%lim%ncu) = 1.0

     case(2)
       do i=1,rfcond%lim%ncu
          weilsq(i) = 0.0
          if (ycountoss(i) > 0.0) weilsq(i) = 1.0/ycountoss(i)
       enddo

     case(3)
       do i=1,rfcond%lim%ncu
          weilsq(i) = 0.0
          if (ycountoss(i) > 0.0) weilsq(i) = 1.0/(ycountoss(i)**2)
       enddo

   end select
!
   end subroutine setwei
!----------------------------------------------------------------------------------
!----------------------------------------------------------------------------------
!
!     SUBROUTINES DI STAMPA
!
!----------------------------------------------------------------------------------
!----------------------------------------------------------------------------------
   subroutine stampa_risultati (kpr)
   USE RefinecomRef
   USE RIETV
   USE genlsq
           USE spginfom
           USE cif_frm
           USE sinterface
           USE general, only:celem, package_alt_name, alambda, neutro
           USE variables, only:atm,lconn
   USE unit_cell
   USE type_constants, only: DP
   implicit none
   integer, intent(in) :: kpr
   logical             :: stdprint,fprint,cprint,mprint
   real(DP), dimension(rinfo%npar) :: sdpar
   integer :: ier
!corr   type(cell_type) :: cellt
!
   stdprint = index(rfcond%sprint,'E') > 0
   fprint = index(rfcond%sprint,'S') > 0
   cprint = index(rfcond%sprint,'C') > 0
   mprint = index(rfcond%sprint,'M') > 0
   if (stdprint .or. fprint) then
       if (stdprint) then
           !call calculate_sd(jacob,funcls,chisquarednow,rinfo%npar,sdpar,ier)  !FIXME-why chisquared was here?
           call calculate_sd(jacob,funcls,rinfo%npar,sdpar,ier)
           if (ier == 0) then
               call set_sdval(sdpar)
               call printfin_esd(kpr,rinfo%npar,rfcond,'Final Rietveld parameters and stdv',sdpar)
!!!!!FIXME - call export cif file
!corr                cellt = set_cell_type(cellr%val)
                call create_ciffile('riet.cif',cell=cellph,atom=atm,progname=package_alt_name,    &
                spg=init_spaceg_type(spaceg_string()),elem=celem,legm=lconn,stdcell=cellph%get_sd(),wave=alambda,source_type=neutro)
           endif
       else
           call printfin(kpr,'Final Rietveld parameters')
       endif
   endif
   if (cprint) call stampa_correlation(kpr)
   if (mprint) call stampa_matrici_lsq(kpr)
!
   end subroutine stampa_risultati
!-------------------------------------------------------------------------------
   subroutine stampa_correlation(kpr)
   USE RefinecomRef
   USE PRNUTIL
   USE genlsq   !!!!, only:matinv,par_string
   USE type_constants, only: DP
!
   implicit none
   integer, intent(in)                                   :: kpr
   real(DP), dimension(size(matinv,1),size(matinv,1)) :: mat_corr
   integer                                               :: Nact
   integer                                               :: i,j
   integer                                               :: ier
!
   call solve_linlsq(jacob,-funcls,shiftx,ier,grad,hess,.true.,matinv)
   if (ier == 0) then
       Nact = size(matinv,1)
       forall (i=1:Nact, j=1:Nact) mat_corr(i,j) = matinv(i,j)/sqrt(matinv(i,i)*matinv(j,j))
       call stampa_matrice(mat_corr,par_string,4,kpr,'Correlation Matrix')
   endif
!
   end subroutine stampa_correlation

  !--------------------------------------------------------------------------------

   subroutine UserBack()     !!!! subroutine da sistemare
   USE RefinecomRef
   USE background
   USE counts
   USE RIETV
!
   implicit none
   real :: thmin0,thmax0
!
   if (backriet%auto) then
       call autobackground(ttetaczero,backgr,backriet%btype,pointb(:npointback),thminbkg,coefbkg%val,backriet%ncoef)
   else
       thmin0 = ttetaczero(1)
       thmax0 = ttetaczero(rfcond%lim%ncu)
       call param_back(backriet%btype,pointb(:npointback),backriet%ncoef,coefbkg%val,thmin0,thmax0,thminbkg)
       !call getbackground(backriet%btype,backgr(:rfcond%lim%ncu),ttetaczero,coefbkg(:backriet%ncoef)%val,thmin0,  &
       !thmax0,thminbkg)
       call getbackground(backriet%btype,backgr(:rfcond%lim%ncu),ttetaczero,coefbkg(:backriet%ncoef)%val,thminbkg)
   endif
!
   end subroutine Userback

  !--------------------------------------------------------------------------------

   integer function update_background(xpixel,ypixel) result(ier)
   USE RefinecomRef
   USE Molcom
   USE GREXPOMOD, only : scalex,dispx,scaley,dispy,gr_clear_all_area,iselview
   USE VIEW
   USE pointmod
   USE genlsq, only:funcls
   USE plotstyle
   USE counts, only:backgr
   USE proginterface
   USE arrayutil
!
   implicit none
   integer, intent(in) :: xpixel,ypixel
   character(len = 80) :: Message
   integer             :: i
   integer             :: ira
   integer             :: iix2, iiy2
   type(point_type)    :: padd
   integer             :: ipos
!
   if (button_ev == 1) then      ! aggiungo punti col tasto sinistro
!
!      Set background is not available
       if (.not.allocated(backgr)) then
           call fillcounts()
           call back_for_peaksearch(.true.)
       endif
       npointback = npointback + 1
       padd%x = (xpixel - dispx(iselview)) / scalex(iselview)
       padd%y = (ypixel - dispy(iselview)) / scaley(iselview)
       ipos = clocate(pointb(:)%x,padd%x)
       call add_point(pointb,padd,ipos)
       ier = 0
   elseif (button_ev == 3) then  ! elimino punti col tasto destro
       ira=5
       ier = -1
       do i=1, npointback
          iix2 = pointb(i)%x*scalex(iselview) + dispx(iselview)
          iiy2 = pointb(i)%y*scaley(iselview) + dispy(iselview)
              if (i < 10) write(0,*)'BACK=',i,iabs(xpixel-iix2), iabs(ypixel-iiy2),ira
          !if(iabs(xpixel-iix2) < ira .and. iabs(ypixel-iiy2) < ira) then
          if(iabs(xpixel-iix2) <= ira .and. iabs(ypixel-iiy2) <= ira) then
             call Beep(200,200)
             call del_point(pointb,i)
             npointback = npointback - 1  
                   write(0,*)'ELIMINO AT ',i
             ier = 0
             exit
          end if
       enddo
   endif
!
   if (ier == 0) then
       call UserBack()      ! ricalcola il background
!
       if (allocated(funcls)) then   ! calcola e mostra Rwp se  funcls e' allocato
!corr           call erreacc(rfcond%nback)
           call erreacc(backriet%ncoef)
           write(Message,'(" Rp = ",f8.3," Rwp = ",f8.3)') rinfo%rp,rinfo%rwp
           call PrintMess(Message)
       endif
!
       if (style(STYLE_PEAKS)%vis == 1) then  ! peaksearch
           !call peaksearch(shiftzero,ier)
           call run_peaksearch()
       else
           call vedinew()
       endif
   endif
!
   end function update_background

!----------------------------------------------------------------------------------------
   subroutine abs_corr
   USE assorbimento
   USE RefinecomRef
!
!  Questa subroutine determina un'espressione della correzione di assorbimento
!  in funzione di sen(theta) utilizzando un metodo proposto sulle tabelle internazionali
!  rif. Tibbals,Acta Cryst.(1982).A38,161-163
!
   implicit none
   integer :: i,j
   real, dimension(7) :: abs
   real               :: muR
   real               :: sum_abs
!
!  Calcolo la correzione di assorbimento (abs) per theta=0,15,30,45,60,75,90 gradi
!
   muR = 0.1
   write(6,*)'muR = ',muR
   do j=1,7
      sum_abs = 0.0
      do i=1,7
         sum_abs = sum_abs + cabs(i,j)*muR**i
      enddo
      abs(j) = exp (sum_abs)
   enddo
!
!  Calcolo coefficienti coef della formula di correzione
!
   do j=1,7
      abscoef(j) = sum(abs*cinv(j,:)/3.0)
   enddo
   write(0,*)'OLD',abscoef(1:7)
!
   return
   end subroutine abs_corr
! ----------------------------------------------------------------------
   subroutine print_all_parameters(kpr,refcond)
   USE RefinecomRef
   USE GENLSQ
   USE RIETV
   USE RRESTR
   USE variables, only:atm
   USE compare_struct
   USE General, only:knw,cell,lo
   USE STRUTIL
   USE constraints
   USE unit_cell
   USE profile_function
!
   implicit none
   integer, intent(in)         :: kpr
   type(refine_condition_type) :: refcond
   integer                     :: i
   integer                     :: back_code
   real                        :: shiftp
   integer                     :: sigsh
   character(len=*), dimension(6), parameter :: scell= ['a    ','b    ','c    ','alpha','beta ','gamma']
!
   sigsh = 5
!
!  Stampa intestazione
   write(kpr,'(2x,76("="))')
   if (refcond%auto_prof .or. refcond%auto_stru) then
      write(kpr,'(16x,a10,i4,a24,i4,a1,i0/)') '  Cycle # ',ncytot,        &
      '       Step refinement: ',kstager,'/',kstagemax
   else
      write(kpr,'(26x,a10,i4/)') '  Cycle # ',ncytot
   endif
!
   write(kpr,'(9x,a,i0)')'Number of refined parameters:',rinfo%npar
!
!  Stampa titolo colonne
   back_code = count(coefbkg(:)%rcod > 0) 
   if (back_code+zero%rcod+sdisp%rcod+stran%rcod+scalr%rcod+gpo%rcod > 0 .or. rinfo%refine_profile &
       .or. rinfo%refine_cell) then
      write(kpr,'(a)')'                    Parameter            Shift'
   endif
!
!  Background
   if (back_code > 0) then
       do i=1,backriet%ncoef
          call print_parameter(coefbkg(i),sigsh,kpr)
       enddo
       write(kpr,*)
   endif
!
!  Zero
   call print_parameter(zero,sigsh,kpr,separ=.true.)
!
!  Sample displacement and transparency
   call print_parameter(sdisp,sigsh,kpr,separ=.true.)
   call print_parameter(stran,sigsh,kpr,separ=.true.)
!
!  Scale
   call print_parameter(scalr,sigsh,kpr)
!
!  Informazioni sul profilo
   select case (rfcond%typefun)

     case (PEARSON7)
       call print_parameter(pear%u,sigsh,kpr)
       call print_parameter(pear%v,sigsh,kpr)
       call print_parameter(pear%w,sigsh,kpr)
       call print_parameter(pear%b0,sigsh,kpr)
       call print_parameter(pear%b1,sigsh,kpr)
       call print_parameter(pear%b2,sigsh,kpr)
       call print_parameter(pear%asym,sigsh,kpr)

     case (PVOIG)
       call print_parameter(pvoi%u,sigsh,kpr)
       call print_parameter(pvoi%v,sigsh,kpr)
       call print_parameter(pvoi%w,sigsh,kpr)
       call print_parameter(pvoi%eta0,sigsh,kpr)
       call print_parameter(pvoi%eta1,sigsh,kpr)
       call print_parameter(pvoi%eta2,sigsh,kpr)
       call print_parameter(pvoi%asym,sigsh,kpr)

     case (TCHZ)
       call print_parameter(tch%u,sigsh,kpr)
       call print_parameter(tch%v,sigsh,kpr)
       call print_parameter(tch%w,sigsh,kpr)
       call print_parameter(tch%z,sigsh,kpr)
       call print_parameter(tch%x,sigsh,kpr)
       call print_parameter(tch%y,sigsh,kpr)
       call print_parameter(tch%asym,sigsh,kpr)

   end select
   if (rinfo%refine_profile)write(kpr,*)
!
!  Orientazione preferenziale
   call print_parameter(gpo,sigsh,kpr,separ=.true.)
!
!  Cella
   do i=1,3
      if (cellph%get_cod(i) > 0) then
          shiftp = shiftx(cellph%get_cod(i))  
          !write(kpr,'(9x,a,t20,f0.7,t40,a)')trim(cellr(i)%str),cell(i),trim(string_sigf(shiftp,sigsh))
          write(kpr,'(9x,a,t20,f0.7,t40,a)')trim(scell(i)),cellph%get_par(i),trim(string_sigf(shiftp,sigsh))
      endif
   enddo
   do i=4,6
      if (cellph%get_cod(i) > 0) then
          shiftp = shiftx(cellph%get_cod(i))  
          !write(kpr,'(9x,a,t20,f0.4,t40,a)')trim(cellr(i)%str),cell(i),trim(string_sigf(shiftp,sigsh))
          write(kpr,'(9x,a,t20,f0.4,t40,a)')trim(scell(i)),cellph%get_par(i),trim(string_sigf(shiftp,sigsh))
      endif
   enddo
   if (rinfo%refine_cell)write(kpr,*)
!
!  Struttura
   if (rinfo%refine_structure) then
       call print_structure(atm,constrv,kpr)
       write(kpr,*)
   endif
!
!  Constraints su x,y,z
   if (nconstraints(constrv) > 0) then
       call print_constraints(constrv,atm,lo)
   endif
!
   if (refcond%algo == GAUNEW) then
       write(kpr,'(3x,a,a,t42,a,f0.5)') 'Chisquared = ',trim(string_sig(chisquarednow,10)),'Step lenght = ',step_length
   else
       write(kpr, '(3x,a,a)')  'Chisquared =',string_sig(chisquarednow,10)
   endif
   if (nrestraints(resvr) > 0 .and. rinfo%refine_structure) call stampa_restraints(kpr,resvr,atm,set_cell_type(cell))
   call print_rvalues(kpr,title=.false.)
   if (rinfo%refine_structure) call to_coord_ref(0)
!
   end subroutine print_all_parameters

!-------------------------------------------------------------------------------------------------------------

   subroutine init_fobs_old()
   USE Refinecomref
   USE Counts
   USE variables, only:ref
   USE arrayutil
   implicit none
   integer             :: posref
   integer             :: i
   integer             :: pini,pfin
   real                :: ycvmax
!!!!!   integer             :: npp
   integer, parameter  :: NPPX = 5
!!!!!   real, dimension(NPPX*2+1) :: ycb,xcb
!!!!!   integer, dimension(1) :: cloc
   !integer, dimension(rfcond%lim%nrefu) :: cvloc
   !real, dimension(rfcond%lim%nrefu) :: coef
!
   !     coef(:) = 1.0
   !do i=1,rfcond%lim%nrefu 
   !   posref = clocate(ttetacount,ref(i)%tthd)
   !   pini = max(posref-10,1)
   !   pfin = min(posref+10,rfcond%lim%ncu)
   !   cloc(:) = maxloc(ycountoss(pini:pfin)-backgr(pini:pfin))
   !   cvloc(i) = cloc(1) + pini - 1
   !enddo
   !   do i=2,rfcond%lim%nrefu 
   !      if (abs(cvloc(i) - cvloc(i-1)) < 4) coef(i) = 0.5
   !         write(6,*)i,'cvloc=',cvloc(i),coef(i)
   !   enddo
!
   do i=1,rfcond%lim%nrefu 
      posref = clocate(ttetacount,ref(i)%tthd)
      pini = max(posref-NPPX,1)
      pfin = min(posref+NPPX,rfcond%lim%ncu)
!!!!new
      !npp = pfin-pini+1
      !ycb(:npp) = ycountoss(pini:pfin) - backgr(pini:pfin)
      !xcb(:npp) = ttetaczero(pini:pfin)
      !ycvmax = 0.5*SUM((xcb(2:npp)-xcb(1:npp-1))*(ycb(2:npp) + ycb(1:npp-1)))
!!!!new
      ycvmax = maxval(ycountoss(pini:pfin)-backgr(pini:pfin))
      if (ycvmax < 0) ycvmax = 0.01
                !ycvmax = ycvmax * coef(i)
      !ref(i)%fo = sqrt(maxval((ycountoss(pini:pfin)-backgr(pini:pfin))/scalr%val)/(ref(i)%m*ref(i)%lp))
      ref(i)%fo = sqrt(ycvmax/(ref(i)%m*ref(i)%lp))
          !write(6,*)i,'refo=',ref(i)%fo
   enddo
!
   end subroutine init_fobs_old
!-----------------------------------------------------------------------
   subroutine init_fobs()
   USE Refinecomref
   USE Counts
   USE variables, only:ref
   USE arrayutil
   USE reflection_type_util
   USE math_util
   USE conteggi, only: npunti
   implicit none
   integer                                    :: i,j
   integer                                    :: pini,pfin
   real                                       :: yarea
   integer                                    :: npp
   real                                       :: deltath
   real, dimension(npunti)                    :: ycb    !!!!,xcb
   real, dimension(size(ref))                 :: xref
   type(gref_type), dimension(:), allocatable :: gref
   integer                                    :: nref1,nref2,ngref,nrefmin,nrefmax
   integer, dimension(1)                      :: cloc
   real                                       :: tmed,ycv,const
!
   deltath = 0.25
!
   call make_ref_groups(ref(:rfcond%lim%nrefu),gref)   ! estrai gruppi
!
   do i=1,size(gref)
!
!     Individua i conteggi pini e pfin che delimitano il gruppo
      ngref = gref(i)%code
      nref1 = gref(i)%vref(1)
      nref2 = gref(i)%vref(ngref)
      cloc(:) = minloc(ref(nref1:nref2)%tthd)
      nrefmin = cloc(1) + nref1 - 1   ! primo riflesso del gruppo
      cloc(:) = maxloc(ref(nref1:nref2)%tthd)
      nrefmax = cloc(1) + nref1 - 1   ! ultimo riflesso del gruppo
      pini = clocate(ttetacount,ref(nrefmin)%tthd-deltath)
      pfin = clocate(ttetacount,ref(nrefmax)%tthd+deltath)
      !write(6,*)'gruppo n.',i,'ngr=',ngref,nref1,nref2
      !write(6,*)'min=',ttetaczero(pini)
      !write(6,*)'max=',ttetaczero(pfin)
!
!     Calcola integrale dell'area  tra pini e fin
      npp = pfin-pini+1
      ycb(:npp) = ycountoss(pini:pfin) - backgr(pini:pfin)
!!      if (npp > 1) then
!!          xcb(:npp) = ttetacount(pini:pfin)
!!          yarea = 0.5*SUM((xcb(2:npp)-xcb(1:npp-1))*(ycb(2:npp) + ycb(1:npp-1))) 
!!      else
!!          yarea =  ycb(1)
!!      endif
!!        write(0,*)'AREA=',yarea,integrate(ttetacount(pini:pfin),ycb,1,npp)
      yarea = integrate(ttetacount(pini:pfin),ycb,1,npp)
!
      if (yarea < 0) then
          yarea = 0.01
          ref(nref1:nref2)%fo = sqrt(yarea/(ref(nref1:nref2)%m*ref(nref1:nref2)%lp))
      else
          if (ngref > 1) then
!
!             Ripartisci intensita' in basa alla distanza dal massimo
              cloc(:) = maxloc(ycb(:npp)) + pini - 1
              tmed = ttetacount(cloc(1))
              xref(nref1:nref2) = 1.0/(0.001 + abs(ref(nref1:nref2)%tthd - tmed))
              const = yarea/sum(xref(nref1:nref2))
              !write(6,*)'max at =',tmed
              do j=nref1,nref2
                 ycv = const*xref(j)
                 !write(6,*)'int=',j,ycv,yarea,ref(j)%tthd
                 ref(j)%fo = sqrt(ycv/(ref(j)%m*ref(j)%lp))
              enddo
          else
              ref(nref1)%fo = sqrt(yarea/(ref(nref1)%m*ref(nref1)%lp))
          endif
      endif
   enddo
!
   end subroutine init_fobs
!-----------------------------------------------------------------------
   subroutine applyriet_new(modzero,modcell,modop,modxyz,modfun,modwei)
!
!  Esegue determinate modifiche dopo aver modificato i parametri 
!
   USE RefinecomRef
   USE General
   USE Counts
   USE View
   USE background
   USE unit_cell
   USE variables, only:atm,ref  !!!,lconn
   USE RIETV
   USE atom_type_util
   USE reflection_type_util
   USE unit_cell
   !USE constraints
   USE setref
!
   implicit none
   integer, intent(in) :: modzero  ! modifica sullo zero
   integer, intent(in) :: modcell  ! modifica sulla cella
   integer, intent(in) :: modop    ! modifica sull'orientazione preferenziale
   integer, intent(in) :: modxyz   ! modifiche sui parametri strutturali x,y,z,B,och
   integer, intent(in) :: modfun   ! modifica sulla funzione di profilo o sulla scala
   integer, intent(in) :: modwei   ! modifica sul tipo di peso
   logical             :: calcback,calcfc,calcrapi
   type(cell_type) :: cellt
!
   calcback = .false.
   calcfc = .false.
   calcrapi = .false.
!
!  Se ho modificato lo zero-shift ...
   if (modzero == 1) then
       ttetaczero = ttetacount - 2.0 * zero%val              ! ricalcola i 2-theta corretti   !!!!!TODO: add sample displacement
       calcback = .true.                                     ! devo ricalcolare il background
   endif
!
!  Se ho modificato la cella ...
   if (modcell > 0) then
       call calculate_2theta(ref,cell,alambda)  ! rigenera riflessi
!       call calcola_correzioneLP(ref)                        ! ricalcola la correzione LP
       ref(:)%lp = LPcorrection(ref%tthd,neutro,nsyn)
!corr       xgg = matrice_metrica(cell)                           ! ricalcola matrici
!corr       ortom = orthomatrix(cell)
!corr       ortoi = orthomatrixi(cell)
!corr       call xspinb()                                         ! riaggiorna aaa usato dalla to_coord per le distanze
       calcfc = .true.                                       ! devo ricalcolare gli Fc
   endif
!
!  Se ho modificato il parametro G o il piano OP ...
   if (modop == 1) then
       kopcorr = 1                     
       cellt = set_cell_type(cell)
       call calcola_correzioneOP(ref(:rfcond%lim%nrefu),gpo%val,hklpo,cellt)     ! ricalcola OP
       calcrapi = .true.                                     ! devo ricalcolare i rapporti di intensita'
   endif
!
!  Se ho modificato i parametri strutturali x,y,z,B,occ ...
   if (modxyz > 0) then
       call fillatom(atm)                                    ! aggiorno xyzs
       calcfc = .true.                                       ! devo ricalcolare gli Fc
       calcrapi = .true.                                     ! devo ricalcolare i rapporti di intensita'
   endif
!
!  Se ho cambiato il peso ..
   if (modwei == 1) call setwei()
!
!  Ricalcolo il background   !!!TODO: add sample displacement
   !if(calcback)call getbackground(backriet%btype,backgr(:rfcond%lim%ncu),ttetaczero,coefbkg(:backriet%ncoef)%val, &
   !                                       rfcond%lim%ttmin-2.0*zero%val,rfcond%lim%ttmax-2.0*zero%val,thminbkg)
   if(calcback)call getbackground(backriet%btype,backgr(:rfcond%lim%ncu),ttetaczero,coefbkg(:backriet%ncoef)%val,thminbkg)
!
!  Ricalcolo gli Fc
   if (calcfc) call sfcalc(ref,atm,spacegr,celem)
!
!  Ricalcolo i rapporti di intensita'
   if (calcrapi) call setRInt(ref)
!
!  Ricalcolo il profilo se ho modifica qulcosa
   if (modzero+modcell+modop+modxyz+modfun+modwei > 0) then
       call PrintMess('User Modification applied')
       call Count2()
       call vedinew()
       write(Msg,'(a6,f8.3,a7,f8.3)')' Rp = ',rinfo%rp,' Rwp = ',rinfo%rwp
       call PrintMess(Msg)
   endif
!
   end subroutine applyriet_new

!-----------------------------------------------------------------------

   subroutine gesref(exitcode,refop)
   USE RefinecomRef
   USE PatternRef
   USE General
   USE GENLSQ
   USE Molcom  
   USE RIETV
   USE variables, only : updgr,atm,lconn,ref
   USE progtype,only:DEALLOCA,ALLOCA
   USE RRESTR 
   USE prnutil
   USE grexpomod
   USE messagemod
   USE ilmviewmod
   USE atom_type_util
   USE connect_mod
   USE unit_cell
   USE setref
   USE constraints
!
   implicit none
   integer, intent(in) :: refop   ! se 1 anche l'OP va affinata
   integer       :: Nactot
   integer       :: kvolte1,kvolte2
   integer       :: modes
   external frietlm
   external frietgauss
   integer, intent(out) :: exitcode
   logical       :: winact
   integer       :: kstageprof
!   character(len=10), dimension(50) :: sstage
!   integer, dimension(50)           :: specstage
   integer                          :: nstage
   integer                          :: nstagestru
   integer                          :: stoplsq
   logical                          :: bothref   !!!!,continue_ref,resetcode
   integer                          :: nstageval
   integer                          :: numres
   character(len=200)               :: msgpar,msgstep
   character(len=200)               :: msginfo
   integer                          :: ierp
   type(cell_type) :: cellt
   type(refinement_strategy_type) :: riets
   !type(refinement_strategy_type) :: profs
!            integer, dimension(:), allocatable :: arr
!

   exitcode = 0
   call open_rietveld_viewer()
   ifBrekke = 0
   termcode = 0
   updgr = 5
   kstagemax = 0
   nstageval = 11
   bothref = rfcond%auto_prof .and. rfcond%auto_stru 
   numres = nrestraints(resvr)
   if (rfcond%raction == 2) nstageval = 35
   if (bothref) then
       kstagemax = nstageval  !!!!22  !!!11
       kstageprof = kstagemax
       !call define_strategy(1,sstage,specstage,nstage,numres)
       call riets%new()
       call define_strategy_prof(riets,rfcond%raction)
       call define_strategy_struct(1,riets,atm,neutro)
       call riets%add_stage('ALLP')
       !call riets%prn(0)
       !nstage = riets%get_num()
       !kstagemax = kstagemax+nstage+1
       kstagemax = riets%get_num()
       modes = 2
   else
       if (rfcond%auto_prof) then
           !kstagemax = nstageval  !!!!22  !!!!11
           modes = 2   !!!!1
           call riets%new()
           call define_strategy_prof(riets,rfcond%raction)
           kstagemax = riets%get_num()
       elseif (rfcond%auto_stru) then        
           !call define_strategy(1,sstage,specstage,nstage,numres)
           call riets%new()
           call define_strategy_struct(1,riets,atm,neutro)
           nstage = riets%get_num()
           kstagemax = kstagemax + nstage
           modes = 2
       endif
   endif
   call calcola_occ(atm,spacegr,set_cell_type(cell))
!
!!!corr   call scrper('Starting refinement',0,0)
!   call progress_bar_init('Starting refinement')
!
!corr   call DoSingleIteration()
   call clear_messages(ipos=2)
!
   nstagestru = 0
   call setwei() ! il peso va riempito nuovamente
   loop_stage: do kstager = 1,kstagemax
!
      kvolte1=1; kvolte2=3
!!!!!      call defrange(kvolte1,kvolte2)
!
    !!!!!  write(msgstep,'("(step ",i0,"/",i0,")")')kstager,kstagemax
      cellt = set_cell_type(cell)
!corr      if (bothref) then
         ! if (kstager == kstagemax) then
         !     call refine_all(atm,lconn,cellph,spacegr,celem,neutro,constrv,refop)
         !     msgpar = 'all'
         !     msgstep = 'Automatic refinement '   !!!!!!!//trim(msgstep)
         ! else
              call azzeraref(atm,cellph,spacegr,constrv,.false.)                                  
              call setstageprof_new(riets%stages(kstager),cellph,spacegr,refop,rfcond,atm,lconn,celem,constrv,neutro,ref)
              msgpar = riets%stages(kstager)%get_string()
              msgstep = 'Automatic refinement '    !!!TODO:set correctly this string
         ! endif
!corr         if (kstager <= kstageprof) then
!corr             !call setstageprof(msgpar,refop)
!corr             call azzeraref(atm,cellph,spacegr,constrv,.false.)                                  
!corr             call setstageprof_new(riets%stages(kstager),cellph,spacegr,refop,rfcond,atm,lconn,constrv)
!corr             msgpar = riets%stages(kstager)%get_string()
!corr             msgstep = 'Automatic profile refinement ' !!!!!!!//trim(msgstep)
!corr         elseif (kstager == kstagemax) then
!corr             call refine_all(refop)
!corr             msgpar = 'all'
!corr             msgstep = 'Automatic refinement '   !!!!!!!//trim(msgstep)
!corr         else
!corr             !nstagestru = nstagestru + 1
!corr             !call setstagestru(rfcond,atm,lconn,cellph,spacegr,celem,constrv,sstage(nstagestru),specstage(nstagestru),msgpar)
!corr               !write(6,*)'NSTAGE N.',nstagestru,riets%stages(nstagestru)%get_string()
!corr             !call setstagestru_new(rfcond,atm,lconn,cellph,spacegr,celem,constrv,riets%stages(nstagestru))
!corr             call setstageprof_new(riets%stages(kstager),cellph,spacegr,refop,rfcond,atm,lconn,constrv)
!corr             msgpar = riets%stages(kstager)%get_string()
!corr             msgstep = 'Automatic structure refinement '    !!!!!!//trim(msgstep)
!corr         endif
!corr      else
!corr         if (rfcond%auto_prof) then
!corr             !call setstageprof(msgpar,refop)
!corr!!!!!new
!corr             call azzeraref(atm,cellph,spacegr,constrv,.false.)                                  
!corr             call setstageprof_new(riets%stages(kstager),cellph,spacegr,refop,rfcond,atm,lconn,celem,constrv,neutro)
!corr!!!!!new
!corr             msgpar = 'test'
!corr             msgstep = 'Automatic profile refinement '   !!!!!!!//trim(msgstep)
!corr         elseif (rfcond%auto_stru) then
!corr             !nstagestru = nstagestru + 1
!corr               !write(6,*)'NSTAGE N.',nstagestru,riets%stages(nstagestru)%get_string()
!corr             !call setstagestru(rfcond,atm,lconn,cellph,spacegr,celem,constrv,sstage(nstagestru),specstage(nstagestru),msgpar)
!corr             !call setstagestru_new(rfcond,atm,lconn,cellph,spacegr,celem,constrv,riets%stages(nstagestru))
!corr             call azzeraref(atm,cellph,spacegr,constrv,.false.)                                  
!corr             call setstageprof_new(riets%stages(kstager),cellph,spacegr,refop,rfcond,atm,lconn,celem,constrv,neutro)
!corr             msgpar = riets%stages(kstager)%get_string()
!corr             msgstep = 'Automatic structure refinement '   !!!!!//trim(msgstep)
!corr         endif
!corr      endif
      write(msginfo,'(a,i0,a,i0,a)')trim(msgstep)//' (step ',kstager,'/',kstagemax,').'
      call write_message(trim(msginfo),pos=2)
      call write_message('Parameters: '//trim(msgpar),pos=3)
      !call progress_bar_update(cval=kstager,fval=kstagemax)
!
      loop_restart: do
          call conta_parametri(Nactot,rfcond)
          if (Nactot == 0) cycle loop_stage           ! solo se ho atomi in posizione speciale
          if (rinfo%refine_structure) then
              call Allocafuncls(ALLOCA,rfcond%lim%ncu+numres)
          else
              call Allocafuncls(ALLOCA,rfcond%lim%ncu)
          endif
          call setwei() ! il peso va riempito nuovamente
          call AllocaLSQ(DEALLOCA)
          if (numres > 0 .and. rinfo%refine_structure) then
              call AllocaLSQ(ALLOCA,Nactot,rfcond%lim%ncu+numres)
          else
              call AllocaLSQ(ALLOCA,Nactot,rfcond%lim%ncu)
          endif
          call prerefine(scalr)
!
          call calcola_profilo(Nactot,ierp)
!
          if (rfcond%algo == LMDER) then
              call levmarq_algo(frietlm,Nactot,rfcond%lim%ncu,modes,winact)
              stoplsq = 0
          else
              call gaussn_algo(frietgauss,Nactot,rfcond%maxcy,rfcond%eps,rfcond%sprint,exitcode,stoplsq)
              if (exitcode == 1) cycle loop_stage  ! skip stage from user
              if (exitcode == 2) exit loop_stage   ! exit from user
          endif
!
          if (kstager == kstagemax) call stampa_risultati(lo)   
          if (stoplsq == 0) exit
      enddo loop_restart
!
                             call calcola_profilo_leb(Nactot) !prova
   enddo loop_stage
                             call calcola_profilo_leb(Nactot) !prova
        
!
!  Ricalcola le occ. se hai affinato la struttura
   if (rinfo%refine_structure) then
       cellt = set_cell_type(cell)
       call calcola_occ(atm,spacegr,cellt)
       call bond_distance_update(atm,lconn,cellt%get_g())
       call ilmview_draw(atm,lconn)
       call update_pattern(ncytot)
   endif
!
!!!   call scrper(' ',0,-1)
!!!!   call progress_bar_close()
!
!!corr   if (rfcond%winmess) call message_from_lsq(continue_ref,resetcode)
!
   call AllocaLSQ(DEALLOCA)
!
   call azzeraref(atm,cellph,spacegr,constrv)         ! riazzera irefmat per evitare selezioni attive con la grafica
   call make_constraints(atm,lconn,constrv,spacegr,cellph,rfcond%riding_model,rfcond%adp_factor)
!
   end subroutine gesref

!-----------------------------------------------------------------------------------------
   subroutine print_short_info(kpr)
   USE RefinecomRef, only:rinfo
   integer, intent(in) :: kpr
   if (rinfo%refine_structure) then
       write(kpr,'(a,f10.3)')'@',rinfo%rwp
   endif
   end subroutine print_short_info
!-----------------------------------------------------------------------------------------
   subroutine message_from_lsq(msgcontinue,resetcode)
   USE RefinecomRef
   USE prnutil
   USE genlsq
   USE General
   USE RIETV
   logical, intent(out) :: msgcontinue
   logical, intent(out) :: resetcode
   integer              :: exitcode
!
   msgcontinue = .false.
   resetcode = .false.
   write(Inte,'(a20,1x,i2)')'Termination criteria',termcode
   select case (termcode)
       case (0)
         if (rfcond%auto_stru .or. rfcond%auto_prof) then
             Msg = 'Automatic Refinement completed.'//char(10)//'Convergence NOT reached'
             call MsgWinErr(Inte,Msg,WARN_WINDOW,exitcode)
         else
             Msg = ' Convergence NOT reached'//char(10)//' Do you want to continue?'
             call MsgWinErr(Inte,Msg,QUEST_WINDOW_NO_YES,exitcode)
             if (exitcode == 1) msgcontinue = .true.
         endif
       case (-1,1,2,3)
         if (rfcond%auto_stru .or. rfcond%auto_prof) then
             Msg = 'Automatic Refinement completed.'//char(10)//'Convergence reached'
         else
             Msg = 'Refinement completed.'//char(10)//'Convergence reached'
         endif
         call MsgWinErr(Inte,Msg,WARN_WINDOW,exitcode)
         resetcode = .true.
       case (-2) 
         call MsgWinErr('Warning','Hessian matrix is not positive definite',WARN_WINDOW,exitcode)
         !call stampa_matrice(hess,par_string,7,6)
   end select
!
   end subroutine message_from_lsq
!-----------------------------------------------------------------------------------------
   subroutine open_rietveld_viewer()
   USE RefinecomRef
   USE grexpomod
   USE molcom  !for colors
   integer :: xdim,ydim
!
   if (index(rfcond%sprint,'W') > 0 .and. jscreen > 0) then
       call gr_create_graph(grgraph)
       call gr_set_graph(grgraph,stringx='cycles',stringy='Rwp',xscale=10000.0)
       if (.not.allocated(grplot)) then
            allocate(grplot(1))
            call gr_new_plot(grplot(1),1000)
            call gr_set_plot(grplot(1),existmarkers=.TRUE.,colorline=color_LIGHTRED,colormarkers=color_BLACK)
       endif
       call new_viewer_size(xdim,ydim)
       call gr_open_viewer(xdim,ydim,'CF vs. number of moves',newv=.true.)
       call gr_select_viewer(1)
       call gr_setbackground(color_BRIGHTWHITE)
       call gr_select_viewer(0)
   endif
!
   end subroutine open_rietveld_viewer
!-----------------------------------------------------------------------------------------
   subroutine UserRef()
   USE RefinecomRef
   USE PatternRef
   USE General
   USE Molcom, only : ifBrekke
   USE GENLSQ
   USE RIETV
   USE variables, only : updgr,atm,lconn
   USE progtype,only:DEALLOCA,ALLOCA
   USE RRESTR
   USE prnutil
   USE grexpomod
   USE messagemod
      USE ilmviewmod
      USE atom_type_util
   USE connect_mod
   !USE constraints
   USE unit_cell
   USE setref
!
   implicit none
   integer       :: Nactot
   integer       :: exitcode
   integer       :: stoplsq
   logical       :: continue_ref,resetcode
   integer       :: numres
   integer       :: ierp
   type(cell_type) :: cellt
   external frietlm
   external frietgauss
!
   fortran_work = .true.
   call open_rietveld_viewer()
   ifBrekke = 0
   termcode = 0
   updgr = 1
!
!  Inizializzazioni utili per la gestione della progressbar
   kstagemax = 1
   kstager = 1
   ncurrent_cycle = 1
   rfcond%ncauto = 1
!
   call progress_bar_init('Starting refinement')
   call clear_messages(ipos=2)
!corr   call conta_parametri(Nactot,rfcond)
   if (rinfo%refine_structure) call calcola_occ(atm,spacegr,set_cell_type(cell))
   do 
!
       resetcode = .false.
       call conta_parametri(Nactot,rfcond)
       call AllocaLSQ(DEALLOCA)
       if (Nactot == 0) exit
!
       numres = nrestraints(resvr)
       if (numres > 0 .and. rinfo%refine_structure) then
           call AllocaLSQ(ALLOCA,Nactot,rfcond%lim%ncu+numres)
       else
           call AllocaLSQ(ALLOCA,Nactot,rfcond%lim%ncu)
       endif
       call prerefine(scalr)
!
       call calcola_profilo(Nactot,ierp)
       if (rfcond%algo == LMDER) then
           call levmarq_algo(frietlm,Nactot,rfcond%lim%ncu,2,.TRUE.)
       else
           do 
              call gaussn_algo(frietgauss,Nactot,rfcond%maxcy,rfcond%eps,rfcond%sprint,exitcode,stoplsq)
              if (stoplsq > 0) exit
              call message_from_lsq(continue_ref,resetcode)
              if (.not.continue_ref) exit
           enddo
       endif
       if (stoplsq == 0) exit
!corr       call conta_parametri(Nactot,rfcond)  ! recompute number of parameters before new refinement
   enddo
!
!  Ricalcola le occ. se hai affinato la struttura
   cellt = set_cell_type(cell)
   if (rinfo%refine_structure) then
       call calcola_occ(atm,spacegr,cellt)
       call bond_distance_update(atm,lconn,cellt%get_g())
       call ilmview_draw(atm,lconn)
       call update_pattern(ncytot)
   endif
!
   call stampa_risultati(lo)   
   if (resetcode) then
       call azzeraref(atm,cellph,spacegr,constrv)
       call make_constraints(atm,lconn,constrv,spacegr,cellph,rfcond%riding_model,rfcond%adp_factor)
   endif
!corr   call delete_all_constraints(constrv,code=2,typec=2)
!corr   call set_constr_on_position(constrv,atm)
   call progress_bar_close()
   call AllocaLSQ(DEALLOCA)
   fortran_work = .false.
!
   end subroutine UserRef
!--------------------------------------------------------------
   subroutine update_pattern(nls)
   USE Molcom, only:jscreen
   USE VIEW
   USE variables, only:updgr
   USE RefinecomRef, only:rfcond
   implicit none
   integer, intent(in) :: nls
   integer             :: kresto
!
   if (jscreen == 0) return      ! Rietveld senza grafica
!
!  updgr --> variabile che definisce dopo quanti cicli va rifatto il grafico
!
!  Il grafico viene aggiornato solo dopo veti_graph(2) cicli o nell'ultimo ciclo
!
   kresto = mod(nls,updgr)
   if (kresto == 0 .or. nls == rfcond%maxcy) then
       if (kzoomn) then
           call vedinew()  ! mantiene lo zoom
       else
           call vedinew(8,1,rfcond%lim%nctot,1,rfcond%lim%ncu,lsetstyle=.false.)
       endif
   endif
!
   end subroutine update_pattern
!--------------------------------------------------------------
   subroutine conta_parametri(Nactot,refcond)
   USE RefinecomRef
   USE variables, only:atm
   USE RRESTR
   USE atom_type_util
   USE constraints
   USE profile_function
   USE rietv
!
   implicit none
   type(refine_condition_type) :: refcond
   integer, intent(out) :: Nactot
   integer              :: Ncell,Nprof,Nstru
   integer              :: i
   integer              :: nconstra
!
   Nactot = 0
!
!  Fo per Pawley
   if (rfcond%raction == 2) then
       do i=1,rfcond%lim%nrefu
          if (fopar(i)%rcod > 0) then
              Nactot = Nactot + 1
          endif
       enddo
   endif
!
!  Cella
   Ncell = count(cellph%get_cod() > 0) 
   rinfo%refine_cell = Ncell > 0
!
!  Profilo
   Nprof = 0
   select case (refcond%typefun)
     case (PEARSON7)
       if (pear%u%rcod > 0) Nprof = Nprof + 1
       if (pear%v%rcod > 0) Nprof = Nprof + 1
       if (pear%w%rcod > 0) Nprof = Nprof + 1
       if (pear%b0%rcod > 0) Nprof = Nprof + 1
       if (pear%b1%rcod > 0) Nprof = Nprof + 1
       if (pear%b2%rcod > 0) Nprof = Nprof + 1
       if (pear%asym%rcod > 0) Nprof = Nprof + 1

     case (PVOIG)
       if (pvoi%u%rcod > 0) Nprof = Nprof + 1
       if (pvoi%v%rcod > 0) Nprof = Nprof + 1
       if (pvoi%w%rcod > 0) Nprof = Nprof + 1
       if (pvoi%eta0%rcod > 0) Nprof = Nprof + 1
       if (pvoi%eta1%rcod > 0) Nprof = Nprof + 1
       if (pvoi%eta2%rcod > 0) Nprof = Nprof + 1
       if (pvoi%asym%rcod > 0) Nprof = Nprof + 1

     case (TCHZ)
       if (tch%u%rcod > 0) Nprof = Nprof + 1
       if (tch%v%rcod > 0) Nprof = Nprof + 1
       if (tch%w%rcod > 0) Nprof = Nprof + 1
       if (tch%z%rcod > 0) Nprof = Nprof + 1
       if (tch%x%rcod > 0) Nprof = Nprof + 1
       if (tch%y%rcod > 0) Nprof = Nprof + 1
       if (tch%asym%rcod > 0) Nprof = Nprof + 1

   end select
   if (scalr%rcod > 0) Nprof = Nprof + 1
   rinfo%refine_profile = Nprof > 0
!
!  Struttura
   Nstru = 0
   do i=1,numatomtype(atm)
      Nstru = Nstru + count(atm(i)%rcod > 0)
   enddo
   nconstra = 0
   do i=1,nconstraints(constrv)
      if (constrv(i)%ref > 0) nconstra = nconstra + constrv(i)%npari
   enddo
   rinfo%refine_structure = Nstru + nconstra > 0
!
   Nactot = Nactot + Ncell + Nprof + Nstru
!
   if (zero%rcod > 0) Nactot = Nactot + 1
   if (sdisp%rcod > 0) Nactot = Nactot + 1
   if (stran%rcod > 0) Nactot = Nactot + 1
   if (gpo%rcod > 0) Nactot = Nactot + 1
   Nactot = Nactot + count(coefbkg(:backriet%ncoef)%rcod > 0) + nconstra
   rinfo%npar = Nactot
!
   end subroutine conta_parametri
!--------------------------------------------------------------
   subroutine calcola_profilo (Nactot,ierp)
!
   USE RefinecomRef
   USE Counts
   USE variables, only: ref
   USE calculate_pdp
   implicit none
   integer, intent(in) :: Nactot
   integer             :: ncall
   integer, intent(out) :: ierp
!
   ierp = 0
   ncall = gpo%rcod+rfcond%raction
   if (zero%rcod > 0) ncall = ncall + 1
   if (sdisp%rcod > 0) ncall = ncall + 1
   if (stran%rcod > 0) ncall = ncall + 1
   if (rinfo%refine_cell .or. rinfo%refine_structure .or. rinfo%refine_profile) ncall = 1 
   if (ncall > 0) call calculate_powder_pattern(rfcond,ttetaczero,ycounttot,ref,scalr%val,kalphar,0.0,pear,pvoi,tch,ierp)
!
   call erreacc(Nactot)
!
   end subroutine calcola_profilo
!--------------------------------------------------------------
   subroutine erreacc(Nactot)
   USE Conteggi
   USE RefinecomRef
   USE GENLSQ
   USE Counts
   USE RRESTR, only:nrestraints,resapp
   USE RIETV, only:resvr
   USE General, only: cell
   USE unit_cell
!
   implicit none
   integer, intent(in) :: Nactot
   integer             :: numrexp
   real                :: sumdeltay,sumyow,sdeltayw,sumyow1
   real, dimension(rfcond%lim%ncu) :: diff
   integer             :: ncc
   integer             :: numres
   type(cell_type)     :: cellt
!
   ncc = rfcond%lim%ncu
!
!  Calcola contributo dei conteggi
   funcls(:ncc) = sqrt(weilsq(:ncc))*(ycountoss(:ncc)-backgr(:ncc)-ycounttot(:ncc))
   sdeltayw  = SUM(funcls(:ncc)**2)
!
!  Calcola il contributo dei restraints se si affinano i parametri strutturali
   numres = nrestraints(resvr)
   if (numres > 0 .and. rinfo%refine_structure) then
       cellt = set_cell_type(cell)
       call resapp(cellt,resvr,ncc,0,coderwi=2)  
       chisquarednow=sdeltayw+SUM(funcls(ncc+1:ncc+numres)**2)         ! Somma il contributo dovuto ai restraints
   else
       chisquarednow = sdeltayw                                       ! Chisquared
   endif
!
   sumyow    = SUM(weilsq(:ncc)*ycountoss(:ncc)**2)
   sumyow1   = SUM(weilsq(:ncc)*(ycountoss(:ncc)-backgr(:ncc))**2)
   sumdeltay = SUM(abs(ycountoss(:ncc)-backgr(:ncc)-ycounttot(:ncc)))

   rinfo%rp     = (sumdeltay/SUM(ycountoss(:ncc)))*100.               ! Rp (Conventional)
   errep1    = (sumdeltay/SUM(ycountoss(:ncc)-backgr(:ncc)))*100.     ! Rp'(Back. subtrac.)
!
   rinfo%rwp         = sqrt(sdeltayw/sumyow) *100.                    ! Rwp (Conventional)
   errepw1        = sqrt(sdeltayw/sumyow1) *100.                      ! Rwp'(Back. subtrac.)
   numrexp        = rfcond%lim%ncu - Nactot
   errexp         = sqrt(numrexp/sumyow) *100.                        ! Re (Conventional)
   errexp1        = sqrt(numrexp/sumyow1) *100.                       ! Re'(Back. subtrac.)
   chi2           = (rinfo%rwp/errexp)**2                             ! Chi-2
   diff = weilsq(:ncc)*(ycountoss(:ncc)-backgr(:ncc)-ycounttot(:ncc))
   rinfo%dw       = sum((diff(2:ncc) - diff(1:ncc-1))**2)/sum(diff**2)
     !write(0,*)'SDELTA=',sdeltayw,sum(ycounttot),sum(backgr),minval(backgr),maxval(backgr)
!
   end subroutine erreacc
!------------------------------------------------------------------------
   subroutine prerefine(scal)
   USE RefinecomRef
   USE GENLSQ
   USE variables, only:ref,atm
   USE RRESTR
   USE rietv
   USE profile_function
   implicit none
   type(rparam_type), intent(inout) :: scal
   integer           :: i,j,nkref
   character(len=*), dimension(6), parameter :: scell= ['a    ','b    ','c    ','alpha','beta ','gamma']
!
!  Questa subroutine riempie i vettori krefp,par_string,typex
!
!  krefp(i)      = puntatore su irefmat
!  par_string(i) = stringa associata al parametro
!  typex(i)      = valore massimo
!  i = puntatore al parametro nella matrice di affinamento
!
   nkref = 0
   typex = 1.0
!
!  Affinamento fattori di struttura nel pawley
   if (rfcond%raction == 2) then
       do i=1,rfcond%lim%nrefu
          if (fopar(i)%rcod > 0) then
              nkref = nkref + 1
              fopar(i)%rcod = nkref
              !xpar(nkref) = ref(i)%fo
              xpar(nkref) = ref(i)%fo**2
              par_string(nkref) = fopar(i)%str
          endif
       enddo
   endif
!
!  Zero
   if (zero%rcod > 0) then
       nkref = nkref + 1
       par_string(nkref) = zero%str
       xpar(nkref) = zero%val
       typex(nkref) = 0.001
       zero%rcod = nkref
   endif
!
!  Sample displacement
   if (sdisp%rcod > 0) then
       nkref = nkref + 1
       par_string(nkref) = sdisp%str
       xpar(nkref) = sdisp%val
       typex(nkref) = 0.001
       sdisp%rcod = nkref
   endif
!
!  Sample transparency
   if (stran%rcod > 0) then
       nkref = nkref + 1
       par_string(nkref) = stran%str
       xpar(nkref) = stran%val
       typex(nkref) = 0.001
       stran%rcod = nkref
   endif
!
!  Background
!
   if (coefbkg(1)%rcod > 0) then
       do i=1,backriet%ncoef
          nkref = nkref + 1
          par_string(nkref) = coefbkg(i)%str
          xpar(nkref) = coefbkg(i)%val
          coefbkg(i)%rcod = nkref
       enddo
   endif
!
!  Cella
   if(rinfo%refine_cell) then
      do j=1,6
         if (cellph%get_cod(j) > 0) then
             nkref = nkref + 1
             !par_string(nkref) = cellr(j)%str
             par_string(nkref) = scell(j)
             !xpar(nkref) = cell(j)
             xpar(nkref) = cellph%get_par(j)
             !cellph%get_cod(j) = nkref
             call cellph%set_cod(j,nkref)
             call cellph%set_sd(j,0.)   ! reset sd for refined par.
         endif
      enddo
   endif
!
!  Profilo
   select case (rfcond%typefun)
     case (PEARSON7)
        if (pear%u%rcod > 0) then 
            nkref = nkref + 1
            par_string(nkref) = pear%u%str
            xpar(nkref) = pear%u%val
            pear%u%rcod = nkref
        endif
        if (pear%v%rcod > 0) then 
            nkref = nkref + 1
            par_string(nkref) = pear%v%str
            xpar(nkref) = pear%v%val
            pear%v%rcod = nkref
        endif
        if (pear%w%rcod > 0) then 
            nkref = nkref + 1
            par_string(nkref) = pear%w%str
            xpar(nkref) = pear%w%val
            pear%w%rcod = nkref
        endif
        if (pear%b0%rcod > 0) then
            nkref = nkref + 1
            par_string(nkref) = pear%b0%str
            xpar(nkref) = pear%b0%val
            pear%b0%rcod = nkref
        endif
        if (pear%b1%rcod > 0) then
            nkref = nkref + 1
            par_string(nkref) = pear%b1%str
            xpar(nkref) = pear%b1%val
            pear%b1%rcod = nkref
        endif
        if (pear%b2%rcod > 0) then
            nkref = nkref + 1
            par_string(nkref) = pear%b2%str
            xpar(nkref) = pear%b2%val
            pear%b2%rcod = nkref
        endif
        if (pear%asym%rcod > 0) then
            nkref = nkref + 1
            par_string(nkref) = pear%asym%str
            xpar(nkref) = pear%asym%val
            pear%asym%rcod = nkref
            typex(nkref) = 0.5
        endif

     case (PVOIG)
        if (pvoi%u%rcod > 0) then 
            nkref = nkref + 1
            par_string(nkref) = pvoi%u%str
            xpar(nkref) = pvoi%u%val
            pvoi%u%rcod = nkref
        endif
        if (pvoi%v%rcod > 0) then 
            nkref = nkref + 1
            par_string(nkref) = pvoi%v%str
            xpar(nkref) = pvoi%v%val
            pvoi%v%rcod = nkref
        endif
        if (pvoi%w%rcod > 0) then 
            nkref = nkref + 1
            par_string(nkref) = pvoi%w%str
            xpar(nkref) = pvoi%w%val
            pvoi%w%rcod = nkref
        endif
        if (pvoi%eta0%rcod > 0) then
            nkref = nkref + 1
            par_string(nkref) = pvoi%eta0%str
            xpar(nkref) = pvoi%eta0%val
            pvoi%eta0%rcod = nkref
        endif
        if (pvoi%eta1%rcod > 0) then
            nkref = nkref + 1
            par_string(nkref) = pvoi%eta1%str
            xpar(nkref) = pvoi%eta1%val
            pvoi%eta1%rcod = nkref
        endif
        if (pvoi%eta2%rcod > 0) then
            nkref = nkref + 1
            par_string(nkref) = pvoi%eta2%str
            xpar(nkref) = pvoi%eta2%val
            pvoi%eta2%rcod = nkref
        endif
        if (pvoi%asym%rcod > 0) then
            nkref = nkref + 1
            par_string(nkref) = pvoi%asym%str
            xpar(nkref) = pvoi%asym%val
            pvoi%asym%rcod = nkref
        endif

     case (TCHZ)
        if (tch%u%rcod > 0) then 
            nkref = nkref + 1
            par_string(nkref) = tch%u%str
            xpar(nkref) = tch%u%val
            tch%u%rcod = nkref
        endif
        if (tch%v%rcod > 0) then 
            nkref = nkref + 1
            par_string(nkref) = tch%v%str
            xpar(nkref) = tch%v%val
            tch%v%rcod = nkref
        endif
        if (tch%w%rcod > 0) then 
            nkref = nkref + 1
            par_string(nkref) = tch%w%str
            xpar(nkref) = tch%w%val
            tch%w%rcod = nkref
        endif
        if (tch%z%rcod > 0) then
            nkref = nkref + 1
            par_string(nkref) = tch%z%str
            xpar(nkref) = tch%z%val
            tch%z%rcod = nkref
        endif
        if (tch%x%rcod > 0) then
            nkref = nkref + 1
            par_string(nkref) = tch%x%str
            xpar(nkref) = tch%x%val
            tch%x%rcod = nkref
        endif
        if (tch%y%rcod > 0) then
            nkref = nkref + 1
            par_string(nkref) = tch%y%str
            xpar(nkref) = tch%y%val
            tch%y%rcod = nkref
        endif
        if (tch%asym%rcod > 0) then
            nkref = nkref + 1
            par_string(nkref) = tch%asym%str
            xpar(nkref) = tch%asym%val
            tch%asym%rcod = nkref
        endif

   end select
!
!  Scala
   if (scal%rcod > 0) then
       nkref = nkref + 1
       par_string(nkref) = scal%str
       xpar(nkref) = scal%val
       scal%rcod = nkref
   endif
!
!  Orientazione preferenziale
   if (gpo%rcod > 0) then
       nkref = nkref + 1
       par_string(nkref) = gpo%str
       xpar(nkref) = gpo%val
       gpo%rcod = nkref
   endif
!
   call prerefine_structure(atm,ref,constrv,nkref)
!
   end subroutine prerefine

!------------------------------------------------------------------------

   subroutine rietjacobian(ierp)
   USE Counts
   USE RefinecomRef
   USE General, only : dtor,cell,nsyn,neutro,alambda,celem,spacegr,alambda
   USE nr
   USE GENLSQ
   USE background
   USE RRESTR
   USE progtype, only:deratom
   USE variables, only:atm,ref  
   USE atom_type_util
   USE reflection_type_util
   USE pearsonf
   USE psvoigtf
   USE tchzf
   USE Symmetry, only:jsys
   USE constraints
   USE unit_cell
   USE profile_function
   USE rietv
!
   implicit none
   integer, intent(out)                       :: ierp
   integer                                    :: nref
   integer                                    :: numb
   integer                                    :: i,j,k,kk
   integer                                    :: kka
   integer                                    :: katom
   real                                       :: derzb
   real                                       :: thy,xche
   real                                       :: doc
   real, dimension(36)                        :: tt
   real                                       :: aa,bb
   type(deratom), dimension(:,:), allocatable :: dera
   type(deratom), dimension(:,:), allocatable :: derca
   real         , dimension(:),   allocatable :: dery2tcorr
   real         , dimension(:,:), allocatable :: dfwhm
   real         , dimension(:,:), allocatable :: derpf
   real         , dimension(:,:), allocatable :: dercl
   real         , dimension(:), allocatable   :: derpG
   real         , dimension(:), allocatable   :: derfct
   real         , dimension(:), allocatable   :: derOP
   real         , dimension(:), allocatable   :: fstr
   real, dimension(:,:), allocatable          :: dycount
   real                                       :: rapporto_int
   real                                       :: difth
   integer                                    :: kp
   integer                                    :: kpar
   real                                       :: coef
   integer                                    :: np,kpind,npar,natom
   type(cell_type)                            :: cellt
!
   ierp = 0
   nref = rfcond%lim%nrefu
   numb = rfcond%lim%ncu
   rapporto_int = 1.0
   cellt = set_cell_type(cell)
!
   natom = numatomtype(atm)
   allocate(derfct(nref))
   if (rinfo%refine_cell .or. rinfo%refine_structure) then
       if (rfcond%raction == 0) then
           call fillatom(atm)
           allocate(dera(nref,natom),derca(rfcond%lim%ncu,natom))
           !call fcalcadern(ref,atm(:natom),spacegr,al,bs,cl,nk,alambda,dera,derfct)
           call fcalcadern(ref,atm(:natom),spacegr,celem,alambda,dera,derfct)
       else
           derfct = 0.0
       endif
       call setRInt(ref)
   endif
!
   allocate(fstr(nref))
   if (rfcond%raction > 0) then
       fstr(:) = ref(:nref)%fo
       if (rfcond%raction == 2) then
           allocate(dycount(nref,numb))
           call decomponi_pearson(pear,ref(:nref),ttetaczero(:numb),scalr%val,dycount,kalphar,rapporto_int,ierp)
       endif
   else
       fstr(:) = ref(:nref)%fc
   endif
   if (kopcorr > 0) then
       allocate(derOP(nref))
       call calcola_derivataOP(ref,gpo%val,hklpo,cellt,derOP)
   endif
!
   select case(rfcond%typefun)
      case(PEARSON7)
       allocate(dery2tcorr(numb),dfwhm(numb,3),derpf(numb,4),dercl(numb,6),derpG(numb))
       call derpears(fstr,pear,1,nref,ttetaczero(:numb),ref,cell,derfct,derOP,dery2tcorr,dfwhm,derpf(:,:),dercl,   &
                     derpG,rapporto_int,scalr,rinfo,kalphar,gpo,neutro,nsyn,alambda,jsys)
       if (rinfo%refine_structure) call derpearsa(ref,pear,1,nref,ttetaczero(:numb),dera,derca,kalphar,rapporto_int,scalr)

      case(PVOIG)
       allocate(dery2tcorr(numb),dfwhm(numb,3),derpf(numb,4),dercl(numb,6),derpG(numb))
       call derpsvoi(pvoi,1,nref,ttetaczero(:numb),ref,cell,derfct,derOP,dery2tcorr,dfwhm,derpf,dercl,derpG,rapporto_int,  &
                     scalr,rinfo,kalphar,gpo,neutro,nsyn,alambda,jsys)
       if (rinfo%refine_structure) call derpsvoia(ref,pvoi,1,nref,ttetaczero(:numb),dera,derca,kalphar,rapporto_int,scalr)

      case(TCHZ)
       allocate(dery2tcorr(numb),dfwhm(numb,6),derpf(numb,4),dercl(numb,6),derpG(numb))
       !call derpsvoi_tchz(tch,1,nref,ttetaczero(:numb),cell,derfct,derOP,dery2tcorr,dfwhm,derpf,dercl,derpG,rapporto_int)
       !if (rinfo%refine_structure) call dertchza(tch,1,nref,ttetaczero(:numb),dera,derca,rapporto_int)
       call derpsvoi_tchz(tch,1,nref,ttetaczero(:numb),ref,cell,derfct,derOP,dery2tcorr,dfwhm,derpf,dercl,derpG,rapporto_int,  &
                     scalr,rinfo,kalphar,gpo,neutro,nsyn,alambda,jsys)
       if (rinfo%refine_structure) call dertchza(ref,tch,1,nref,ttetaczero(:numb),dera,derca,kalphar,rapporto_int,scalr)
   end select
!
   LOOP_CONTEGGI : do i=1,numb
      kka = 0
!
!     Affinamento fo con pawley
      if (rfcond%raction == 2) then
          do kk=1,rfcond%lim%nrefu
             if (fopar(kk)%rcod > 0) then
                 kka = fopar(kk)%rcod
                 !jacob(i,kka) = 2.0*dycount(kk,i)*ref(kk)%fo 
                 jacob(i,kka) = dycount(kk,i)
             endif
          enddo
      endif
!
!     Derivata dello zero.
      if (zero%rcod > 0) then
          kka = zero%rcod
!
!         Calcolo del contributo derzb del background alla derivato dello zero
          select case (backriet%btype)

            case (POLY)
              derzb=coefbkg(2)%val
              do k=3,backriet%ncoef
                 derzb = derzb + (k-1)*coefbkg(k)%val*(ttetaczero(i)/thminbkg - 1.0)**(k-2)
              enddo
              derzb = -2.0*derzb / thminbkg

            case (CHEBY)
              aa = -1.0
              bb = 1.0
              difth = (rfcond%lim%ttmax-rfcond%lim%ttmin)
              xche = 2.0*(ttetacount(i) - rfcond%lim%ttmin)/difth - 1.0
              derzb = -4.0*chebev(aa,bb,chder(aa,bb,coefbkg(:backriet%ncoef)%val),xche)/difth

            case (FOUR_SERIES)
              derzb = 0.0

          end select
          jacob(i,kka) = -2.0 * dery2tcorr(i) + derzb
      endif
!
!     Derivative of sample displacement
      if (sdisp%rcod > 0) then
          kka = sdisp%rcod
          select case (diffr_geom)
            case (BB_GEOMETRY)
              jacob(i,kka) = dery2tcorr(i) * cos(dtor*ttetacount(i)/2)*rtod

            case (DS_GEOMETRY)  !!TODO
          end select
      endif
!
!     Derivative of sample transparency
      if (stran%rcod > 0) then
          kka = stran%rcod
          select case (diffr_geom)
            case (BB_GEOMETRY)
              jacob(i,kka) = dery2tcorr(i) * sin(dtor*ttetacount(i))*rtod

            case (DS_GEOMETRY)  !!TODO
          end select
      endif
!
!     Derivate del background
      if (coefbkg(1)%rcod > 0) then

          select case (backriet%btype)

             case (POLY)
               thY=ttetaczero(i)/thminbkg - 1.0
               jacob(i,kka+1) = 1.0
               do kk=2,backriet%ncoef
                  jacob(i,kk+kka) = thY**(kk-1)
               enddo

             case (CHEBY)
               xche = 2.0*(ttetacount(i) - rfcond%lim%ttmin)/(rfcond%lim%ttmax-rfcond%lim%ttmin) - 1.0
               tt(1) = 1.0
               tt(2) = xche
               do j=2,backriet%ncoef-1
                  tt(j+1) = 2.*xche*tt(j) - tt(j-1)
               enddo
               do kk=1,backriet%ncoef
                   jacob(i,kka+kk) = tt(kk)
               enddo

             case (FOUR_SERIES)
               do kk=1,backriet%ncoef
                  jacob(i,kk+kka) = cos((kk-1)*ttetaczero(i)*dtor)
               enddo

          end select
          kka = kka + backriet%ncoef

      endif
!
!
!     Derivate della cella
      if(rinfo%refine_cell) then
         do j=1,6
            if (cellph%get_cod(j) > 0) then
                kka = kka + 1
                jacob(i,kka) = dercl(i,j)
            endif
            if (cellph%get_cod(j) == -1) jacob(i,kka) = jacob(i,kka) + dercl(i,1) !constraints a=b o a=b=c
         enddo
      endif
!
!     Derivate del profilo
      if (rinfo%refine_profile) then
          select case (rfcond%typefun) 
            case (PEARSON7)
              if (pear%u%rcod > 0) then
                  kka = kka + 1
                  jacob(i,kka) = dfwhm(i,1)
              endif
              if (pear%v%rcod > 0) then
                  kka = kka + 1
                  jacob(i,kka)= dfwhm(i,2)
              endif
              if (pear%w%rcod > 0) then
                  kka = kka + 1
                  jacob(i,kka) = dfwhm(i,3)
              endif
              if (pear%b0%rcod > 0) then
                  kka = kka + 1
                  jacob(i,kka)= derpf(i,2)
              endif
              if (pear%b1%rcod > 0) then
                  kka = kka + 1
                  jacob(i,kka)= derpf(i,3)
              endif
              if (pear%b2%rcod > 0) then
                  kka = kka + 1
                  jacob(i,kka)= derpf(i,4)
              endif
              if (pear%asym%rcod > 0) then
                  kka = kka + 1
                  jacob(i,kka) = derpf(i,1)
              endif

            case (PVOIG)
              if (pvoi%u%rcod > 0) then
                  kka = kka + 1
                  jacob(i,kka) = dfwhm(i,1)
              endif
              if (pvoi%v%rcod > 0) then
                  kka = kka + 1
                  jacob(i,kka)= dfwhm(i,2)
              endif
              if (pvoi%w%rcod > 0) then
                  kka = kka + 1
                  jacob(i,kka) = dfwhm(i,3)
              endif
              if (pvoi%eta0%rcod > 0) then
                  kka = kka + 1
                  jacob(i,kka)= derpf(i,2)
              endif
              if (pvoi%eta1%rcod > 0) then
                  kka = kka + 1
                  jacob(i,kka)= derpf(i,3)
              endif
              if (pvoi%eta2%rcod > 0) then
                  kka = kka + 1
                  jacob(i,kka)= derpf(i,4)
              endif
              if (pvoi%asym%rcod > 0) then
                  kka = kka + 1
                  jacob(i,kka) = derpf(i,1)
              endif

            case (TCHZ)
              if (tch%u%rcod > 0) then
                  kka = kka + 1
                  jacob(i,kka) = dfwhm(i,1)
              endif
              if (tch%v%rcod > 0) then
                  kka = kka + 1
                  jacob(i,kka)= dfwhm(i,2)
              endif
              if (tch%w%rcod > 0) then
                  kka = kka + 1
                  jacob(i,kka) = dfwhm(i,3)
              endif
              if (tch%z%rcod > 0) then
                  kka = kka + 1
                  jacob(i,kka) = dfwhm(i,4)
              endif
              if (tch%x%rcod > 0) then
                  kka = kka + 1
                  jacob(i,kka) = dfwhm(i,5)
              endif
              if (tch%y%rcod > 0) then
                  kka = kka + 1
                  jacob(i,kka) = dfwhm(i,6)
              endif
              if (tch%asym%rcod > 0) then
                  kka = kka + 1
                  jacob(i,kka) = derpf(i,1)
              endif
          end select
          if (scalr%rcod > 0) then
              kka = kka + 1
              jacob(i,kka) = ycounttot(i) / scalr%val
          endif
      endif
!
!     Derivate parametri di orientazione preferenziale
      if (gpo%rcod > 0) then
          kka = kka + 1
          jacob(i,kka) = derpG(i)
      endif
!
!     Derivate parametri strutturali
      if (rinfo%refine_structure) then
          do j=1,natom
             do kk=1,5
                if (atm(j)%rcod(kk) > 0) then
                    kka = kka + 1
                    if (kk.eq.1) doc = derca(i,j)%co(1)
                    if (kk.eq.2) doc = derca(i,j)%co(2)
                    if (kk.eq.3) doc = derca(i,j)%co(3)
                    if (kk.eq.4) doc = derca(i,j)%b
                    if (kk.eq.5) doc = derca(i,j)%occ
                    jacob(i,kka) = doc
                endif
             enddo
          enddo
      endif
!
      do j=1,nconstraints(constrv)
         if (constrv(j)%ref > 0) then
             select case (constrv(j)%code)
               case (CONSTR_B)  ! constrain on b, 1 solo par. indipendente
                  kka = kka + 1
                  jacob(i,kka) = 0
                  do kp=1,constrv(j)%npar
                     katom = constrv(j)%vat(kp)
                     kpar = constrv(j)%vpar(kp)
                     coef = constrv(j)%coef(kp)
                     jacob(i,kka) = jacob(i,kka) + coef*derca(i,katom)%b
                  enddo

                case (CONSTR_CO) ! constrain on coord., 1 solo par. indipendente
                  kka = kka + 1
                  jacob(i,kka) = 0
                  do kp=1,constrv(j)%npar
                     katom = constrv(j)%vat(kp)
                     kpar = constrv(j)%vpar(kp)
                     coef = constrv(j)%coef(kp)
                     jacob(i,kka) = jacob(i,kka) + coef*derca(i,katom)%co(kpar)
                  enddo

                case (CONSTR_SOF) ! constrain on occupanze, npar-1 par. indipendenti
                  npar = constrv(j)%npar
                  kpind= constrv(j)%vat(npar)  ! parametro indipendente
                  do np=1,npar-1
                     kka = kka + 1
                     katom = constrv(j)%vat(np)
!!!!TODO: if you use coef should be: jacob(i,kka) = derca(i,katom)%occ - derca(i,kpind)%occ*constrv(j)%coef(i)/constrv(j)%coef(kpind)
                     jacob(i,kka) = derca(i,katom)%occ - derca(i,kpind)%occ
                  enddo
             end select
         endif
      enddo
!
      jacob(i,:kka) = -sqrt(weilsq(i))*jacob(i,:kka)
   enddo LOOP_CONTEGGI
!
!  Introduce nello Jacobiano ed in funcls il contributo dei restraints
   if (nrestraints(resvr)>0 .and. rinfo%refine_structure) call resapp(cellt,resvr,numb,1,coderwi=2)
!
   end subroutine rietjacobian

!--------------------------------------------------------------------

   subroutine setstageprof(msgpar,refop)
   USE RefinecomRef
   USE General
   USE RIETV
   USE messagemod
   USE unit_cell
   USE variables
   USE profile_function
   implicit none
   character(len=*), intent(out) :: msgpar
   integer, intent(in)           :: refop
!
   if (rfcond%raction == 2) then
       call setstageprof_pawley()
       return
   endif
!
   call azzeraref(atm,cellph,spacegr,constrv,.false.)                                  
!corr   call make_constraints(atm,lconn,constrv,spacegr,set_cell_type(cell))
!
   select case(kstager)
     case(1)
       zero%rcod = 1
       msgpar = 'zero'

     case(2)
       zero%rcod = 1
       scalr%rcod = 1
       msgpar = 'zero, scale'

     case(3)
       zero%rcod = 1
       coefbkg%rcod = 1
       scalr%rcod = 1
       msgpar = 'zero, scale, background'

     case(4)
       scalr%rcod = 1
       !call set_rcode_cell(cellph%cod,spacegr,1)
       call cellph%set_refine(spacegr,1)
       msgpar = 'scale, cell parameters'

     case(5)
       zero%rcod = 1
       coefbkg%rcod = 1
       scalr%rcod = 1
       !call set_rcode_cell(cellph%cod,spacegr,1)
       call cellph%set_refine(spacegr,1)
       msgpar = 'zero, background, scale, cell parameters'

     case(6)
       scalr%rcod = 1
       select case(rfcond%typefun)
         case(PEARSON7)
            call set_rcode_function(pear,sref='w,beta0,asym',code=1)
         case(PVOIG)
            call set_rcode_function(psvoig=pvoi,sref='w,eta0,asym',code=1)
            !call set_rcode_function(psvoig=pvoi,sref='w',code=1)
         case(TCHZ)
            call set_rcode_function(tchh=tch,sref='w,x,y,asym',code=1)
       end select
       msgpar = 'scale, W, mixing, asymmetry'

     case(7)
       zero%rcod = 1
       coefbkg%rcod = 1
       scalr%rcod = 1
       !call set_rcode_cell(cellph%cod,spacegr,1)
       call cellph%set_refine(spacegr,1)
       select case(rfcond%typefun)
         case(PEARSON7)
            call set_rcode_function(pear,sref='w,beta0,asym',code=1)
         case(PVOIG)
            call set_rcode_function(psvoig=pvoi,sref='w,eta0,asym',code=1)
            !call set_rcode_function(psvoig=pvoi,sref='eta0',code=1)
         case(TCHZ)
            call set_rcode_function(tchh=tch,sref='w,x,y,z,asym',code=1)
       end select
       msgpar = 'zero, background, scale, cell, profile'

     case(8)
       scalr%rcod = 1
       select case(rfcond%typefun)
         case(PEARSON7)
            call set_rcode_function(pear,sref='u,v,w',code=1)
         case(PVOIG)
            call set_rcode_function(psvoig=pvoi,sref='u,v,w',code=1)
         case(TCHZ)
            call set_rcode_function(tchh=tch,sref='u,v,w',code=1)
       end select
       msgpar = 'scale, FWHM'

     case(9)
       zero%rcod = 1
       coefbkg%rcod = 1
       scalr%rcod = 1
       select case(rfcond%typefun)
         case(PEARSON7)
            call set_rcode_function(pear,sref='beta0,asym,u,v,w',code=1)
         case(PVOIG)
            call set_rcode_function(psvoig=pvoi,sref='eta0,asym,u,v,w',code=1)
         case(TCHZ)
            call set_rcode_function(tchh=tch,sref='x,y,asym,u,v,w',code=1)
       end select
       msgpar = 'zero, background, scale, cell, profile'
       if (refop > 0) gpo%rcod=1

     case(10)
       scalr%rcod = 1
       coefbkg%rcod = 1
       select case(rfcond%typefun)
         case(PEARSON7)
            call set_rcode_function(pear,sref='beta0,beta1,beta2,asym',code=1)
         case(PVOIG)
            call set_rcode_function(psvoig=pvoi,sref='eta0,eta1,eta2,asym',code=1)
         case(TCHZ)
            call set_rcode_function(tchh=tch,sref='x,y,z,asym',code=1)
       end select
       if (refop > 0) gpo%rcod=1
       msgpar = 'scale, background, profile'

     case(11)       
       zero%rcod = 1
       coefbkg%rcod = 1
       scalr%rcod = 1
       !call set_rcode_cell(cellph%cod,spacegr,1)
       call cellph%set_refine(spacegr,1)
       select case(rfcond%typefun)
         case(PEARSON7)
            call set_rcode_function(pear,sref='all',code=1)
         case(PVOIG)
            call set_rcode_function(psvoig=pvoi,sref='all',code=1)
         case(TCHZ)
            call set_rcode_function(tchh=tch,sref='all',code=1)
            call set_rcode_function(tchh=tch,sref='z',code=0)
       end select
       if (refop > 0) gpo%rcod=1
       msgpar = 'all'

   end select
!
   !write(msginfo,'(a,i0,a,i0,a)')'Automatic profile refinement (step ',kstager,'/',kstagemax,').'
   !call write_message(trim(msginfo),pos=2)
!
   end subroutine setstageprof


!--------------------------------------------------------------------

   subroutine setstageprof_pawley()
   USE RefinecomRef
   USE General
   USE RIETV
   USE variables
   USE reflection_type_util
   USE unit_cell
   USE profile_function
   implicit none
   character (len=100) :: msgpar
         integer :: i
   integer :: refinefo
   type(gref_type), allocatable, dimension(:) :: gref
!
     call make_ref_groups(ref,gref,sogi=0.6)
   refinefo = 0
              rfcond%maxcy = 30
   call azzeraref(atm,cellph,spacegr,constrv,.false.)                                  
!corr   call make_constraints(atm,lconn,constrv,spacegr,set_cell_type(cell))
                 scalr%rcod = 1
!
   select case(kstager)
     case(1)
       zero%rcod = 1
       msgpar = 'zero'

     case(2)
       refinefo = 4
       rfcond%maxcy = 1

     case(3)
       refinefo = 5
       rfcond%maxcy = 1

     case(4)
       zero%rcod = 1
       scalr%rcod = 1
       msgpar = 'zero, scale'

     case(5)
       refinefo = 4
       rfcond%maxcy = 1

     case(6)
       refinefo = 5
       !if (rfcond%raction == 2) fopar(:)%rcod = 1
       rfcond%maxcy = 1

     case(7)
       zero%rcod = 1
       coefbkg%rcod = 1
       scalr%rcod = 1
       msgpar = 'zero, scale, background'

     case(8)
       refinefo = 4
       rfcond%maxcy = 1

     case(9)
       refinefo = 5
       rfcond%maxcy = 1

     case(10)
       scalr%rcod = 1
       !call set_rcode_cell(cellph%cod,spacegr,1)
       call cellph%set_refine(spacegr,1)
       msgpar = 'scale, cell parameters'

     case(11)
       refinefo = 4   !!!!2
       rfcond%maxcy = 1

     case(12)
       refinefo = 5   !!!!2
       rfcond%maxcy = 1

     case(13)
       zero%rcod = 1
       coefbkg%rcod = 1
       scalr%rcod = 1
       !call set_rcode_cell(cellph%cod,spacegr,1)
       call cellph%set_refine(spacegr,1)
       msgpar = 'zero, background, scale, cell parameters'

     case(14)
       refinefo = 4
       rfcond%maxcy = 1

     case(15)
       refinefo = 5
       rfcond%maxcy = 1

     case(16)
       scalr%rcod = 1
       select case(rfcond%typefun)
         case(PEARSON7)
            call set_rcode_function(pear,sref='w,beta0,asym',code=1)
         case(PVOIG)
            call set_rcode_function(psvoig=pvoi,sref='w,eta0,asym',code=1)
            !call set_rcode_function(psvoig=pvoi,sref='w',code=1)
         case(TCHZ)
            call set_rcode_function(tchh=tch,sref='w,x,y,asym',code=1)
       end select
       msgpar = 'scale, W, mixing, asymmetry'

     case(17)
       refinefo = 4
       rfcond%maxcy = 1

     case(18)
       refinefo = 5
       rfcond%maxcy = 1

     case(19)
       zero%rcod = 1
       coefbkg%rcod = 1
       scalr%rcod = 1
       !call set_rcode_cell(cellph%cod,spacegr,1)
       call cellph%set_refine(spacegr,1)
       select case(rfcond%typefun)
         case(PEARSON7)
            call set_rcode_function(pear,sref='w,beta0,asym',code=1)
         case(PVOIG)
            call set_rcode_function(psvoig=pvoi,sref='w,eta0,asym',code=1)
            !call set_rcode_function(psvoig=pvoi,sref='eta0',code=1)
         case(TCHZ)
            call set_rcode_function(tchh=tch,sref='w,x,y,z,asym',code=1)
       end select
       msgpar = 'zero, background, scale, cell, profile'

     case(20)
       refinefo = 4
       rfcond%maxcy = 1

     case(21)
       refinefo = 5
       rfcond%maxcy = 1

     case(22)
       scalr%rcod = 1
       select case(rfcond%typefun)
         case(PEARSON7)
            call set_rcode_function(pear,sref='u,v,w',code=1)
         case(PVOIG)
            call set_rcode_function(psvoig=pvoi,sref='u,v,w',code=1)
         case(TCHZ)
            call set_rcode_function(tchh=tch,sref='u,v,w',code=1)
       end select
       msgpar = 'scale, FWHM'

     case(23)
       refinefo = 4
       rfcond%maxcy = 1

     case(24)
       refinefo = 5
       rfcond%maxcy = 1

     case(25)
       refinefo = 2
       zero%rcod = 1
       coefbkg%rcod = 1
       scalr%rcod = 1
       select case(rfcond%typefun)
         case(PEARSON7)
            call set_rcode_function(pear,sref='beta0,asym,u,v,w',code=1)
         case(PVOIG)
            call set_rcode_function(psvoig=pvoi,sref='eta0,asym,u,v,w',code=1)
         case(TCHZ)
            call set_rcode_function(tchh=tch,sref='x,y,asym,u,v,w',code=1)
       end select
       msgpar = 'zero, background, scale, cell, profile'

     case(26)
       refinefo = 4
       rfcond%maxcy = 1

     case(27)
       refinefo = 5
       rfcond%maxcy = 1

     case(28)
       refinefo = 2
       scalr%rcod = 1
       coefbkg%rcod = 1
       select case(rfcond%typefun)
         case(PEARSON7)
            call set_rcode_function(pear,sref='beta0,beta1,beta2,asym',code=1)
         case(PVOIG)
            call set_rcode_function(psvoig=pvoi,sref='eta0,eta1,eta2,asym',code=1)
         case(TCHZ)
            call set_rcode_function(tchh=tch,sref='x,y,z,asym',code=1)
       end select
       msgpar = 'scale, background, profile'
       if (rfcond%raction == 2) fopar(:)%rcod = 1

     case(29)
       refinefo = 4
       rfcond%maxcy = 1

     case(30)
       refinefo = 5
       rfcond%maxcy = 1

     case(31)
       refinefo = 1
       rfcond%maxcy = 1

     case(32)       
       refinefo = 2
       zero%rcod = 1
       coefbkg%rcod = 1
       scalr%rcod = 1
       !call set_rcode_cell(cellph%cod,spacegr,1)
       call cellph%set_refine(spacegr,1)
       select case(rfcond%typefun)
         case(PEARSON7)
            call set_rcode_function(pear,sref='all',code=1)
         case(PVOIG)
            call set_rcode_function(psvoig=pvoi,sref='all',code=1)
         case(TCHZ)
            call set_rcode_function(tchh=tch,sref='all',code=1)
            call set_rcode_function(tchh=tch,sref='z',code=0)
       end select
       msgpar = 'all'
       if (rfcond%raction == 2) fopar(:)%rcod = 1

     case(33)
       refinefo = 4
       rfcond%maxcy = 1

     case(34)
       refinefo = 6
       rfcond%maxcy = 1

     case(35)
       refinefo = 7
       rfcond%maxcy = 1

   end select
!
   select case (refinefo)
      case (1)   
       fopar(:)%rcod = 1

      case (2)
       do i=1,size(fopar)
          if (fopar(i)%val > 0.000) then
              fopar(i)%rcod = 1
          else
              !write(6,*)'riflesso non affinato:',i
          endif
       enddo

      case (3)
       where(fopar(:)%val <= 0.000) fopar(:)%rcod = 1

      case (4)
       do i=1,size(gref)
          if (gref(i)%code == 1) then
              fopar(gref(i)%vref(1))%rcod = 1
          endif
       enddo

      case (5)
       do i=1,size(gref)
          if (gref(i)%code >= 2) then
              fopar(gref(i)%vref(:))%rcod = 1
          endif
       enddo

      case (6)
       do i=1,size(gref)
          if (gref(i)%code >= 2 .and. gref(i)%code <=4) then
              !do j=1,gref(i)%code
              !   if (fopar(gref(i)%vref(j))%val > 0.00)fopar(gref(i)%vref(j))%rcod = 1
              !enddo
              fopar(gref(i)%vref(:))%rcod = 1
          endif
       enddo

      case (7)
       do i=1,size(gref)
          if (gref(i)%code >= 5) then
              fopar(gref(i)%vref(:))%rcod = 1
          endif
       enddo

   end select
!
   end subroutine setstageprof_pawley
!--------------------------------------------------------------------
   subroutine calcola_correzioneAbs()
!
!  Calcolo dellla correzione di assorbimento
!  abscoef = coefficienti calcolati nella subroutine abs_corr
!
   USE RefinecomRef
   USE General, only : dtor    !!!!,nrefl
   USE variables, only:ref
   implicit none
   integer :: numb
   integer :: i
   integer :: kal
   integer, dimension(7) :: mvet = (/0,2,4,6,8,10,12/)
!
   numb = size(ref)  !!!!nrefl   
   LOOP_KALPHA : do kal=1,kalphar
      do i=1,numb
         ref(i)%as = sum(abscoef*sin(ref(i)%tthd*dtor)**mvet)
      enddo
   enddo LOOP_KALPHA
!
   end subroutine calcola_correzioneAbs

!-----------------------------------------------------------------

   subroutine frietgauss(npar,iflag,nls,err)
   USE RefinecomRef
   USE Molcom, only : viewerOpened, jscreen, ModelToCompare
   USE GENLSQ
   USE variables, only:ref,atm,lconn
   USE General, only:lo,knw
   USE compare_struct
   USE RIETV
   USE grexpomod
   USE rrestr
   USE atom_type_util
   USE ilmviewmod
   USE messagemod
   USE stat, only:Rvalue
   USE constraints
   USE type_constants, only: DP
!
   implicit none
   integer, intent(in)            :: npar    ! numero di parametri da affinare
   integer, intent(in)            :: iflag   ! tipo di operazione da eseguire
   integer, intent(in), optional  :: nls     ! num. ciclo corrente
   integer, intent(out), optional :: err
   real(DP), dimension(size(xpar))    :: xparsav
     real :: rbragg, rf
   integer :: ierp
!
   ierp = 0
   select case (iflag)

     case (-1)                    ! calcolo iniziale delle funzioni
       call calcola_profilo(npar,ierp)
           !if (ierp /= 0) write(0,*)'IERP=',ierp

     case (0)                     ! calcola jacobiano
       call rietjacobian(ierp)

     case (1)                     ! applica gli shift e calcola il profilo
       xpar = xparold + step_length*shiftx
       call set_parameter(npar,xpar)
       call calcola_profilo(npar,ierp)
      !     if (ierprof /= 0) write(0,*)'IERProf=',ierprof

     case (2)                     ! stampa e aggiorna la grafica
       if (index(rfcond%sprint,'V') > 0) call print_all_parameters(lo,rfcond)
       if (index(rfcond%sprint,'M') > 0) call stampa_matrici_lsq(lo)
       call write_message('Cycle # ',inum=ncytot,pos=4)
       call write_message('Rp =    ',rnum=rinfo%rp,pos=5)
       call write_message('Rwp =   ',rnum=rinfo%rwp,pos=6)
       call get_rbragg(ref,rbragg,rf)
       call write_message('R-Bragg =   ',rnum=rbragg,pos=7)
       if (knw > 0) then
           call write_message('R-F =   ',rnum=Rvalue(ref%fv,ref%fo),pos=8)
       else
           call write_message('R-F =   ',rnum=rf,pos=8)
       endif
       call progress_bar_update(cval=(ncurrent_cycle-1)*kstagemax*rfcond%maxcy+(kstager-1)*rfcond%maxcy+nls,   &
                                fval=rfcond%maxcy*kstagemax*rfcond%ncauto)
       if (knw > 0 .or. ModelToCompare /= 0) then
           call to_coord_ref(-1)
           call write_message_found(9)
       endif
!
!      Output grafico su viewer secondario
       if (index(rfcond%sprint,'W') > 0 .and. jscreen > 0) then
!           
!          Aggiorna sempre punti da plottare
           call gr_add_point_plot(grplot(1),real(ncytot),rinfo%rwp)
!       
!          Se il viewer è aperto esegui plot
           if (viewerOpened == 1) then
               call gr_select_viewer(1)
               call gr_clear_area(0.0,1.0,0.0,1.0)
               call gr_area(0.08, 0.99, 0.10, 0.99)
               call gr_draw_graph(grgraph,grplot)
               call zflush()
               call gr_select_viewer(0)
           endif
       endif
!
       call update_pattern(ncytot)
       if (rinfo%refine_structure) call ilmview_draw(atm,lconn)
!
     case (3)                     
         !write(0,*)'XPAR=',xpar
       call check_bt2(xpar,atm,constrv,0.0,1000.,0,rfcond%sprint,ierp) ! check thermal factors
       !call check_occupancy(xpar,atm,constrv,0,rfcond%sprint,ierp) ! check occupancy factors
       if (rfcond%raction == 2) call check_fo(xpar,0,ierp)

     case (4)                     ! controlla e blocca i parametri
       xparsav = xpar
       step_length = 0.0
       xpar = xparold + step_length*shiftx
       call set_parameter(npar,xpar)
       call calcola_profilo(npar,ierp)
!
!      Check using parameter with shift (xparsav)
       call check_bt2(xparsav,atm,constrv,0.0,1000.,1,rfcond%sprint,ierp)
       !call check_occupancy(xparsav,atm,constrv,1,rfcond%sprint,ierp) ! check occupancy factors

     case (5)                     ! save parameters
       xparold(:) = xpar(:)
       if (nconstraints(constrv) > 0) then     ! se ci sono constraints e' necessario salvare anche gli atomi
           call copia_atoms(atm_sav,atm)
       endif

   end select
   if (present(err)) err=ierp
!
   end subroutine frietgauss
!-----------------------------------------------------------------
   subroutine frietlm(mfun,npar,x,fvec,fjac,ldfjac,iflag)
   USE Conteggi
   USE RefinecomRef
   USE Molcom, only : viewerOpened
   USE GENLSQ
   USE view
   USE grexpomod
   USE General, only:lo
   USE messagemod
   USE type_constants, only: DP
!
   implicit none
   integer,                   intent(in)              :: mfun   !numero di funzioni
   integer,                   intent(in)              :: npar   !numero di parametri (n<=m)
   integer,                   intent(in)              :: ldfjac
   integer,                   intent(inout)           :: iflag
   real(DP), dimension(mfun),        intent(out) :: fvec   !funzioni che entrano nella somma di quadrati
   real(DP), dimension(ldfjac,npar), intent(out) :: fjac   !jacobiano di fvec
   real(DP), dimension(npar),        intent(in)  :: x      !i parametri da affinare
   integer :: ierp
!!!!!!!   character(80)                                      :: string
!
   select case (iflag)

     case (0)
       nclev = nclev + 1
       if (nclev == 0) then
       elseif (nclev <= rfcond%maxcy) then
           ncytot = ncytot + 1
           shiftx = xpar-xparold
           if (index(rfcond%sprint,'V') > 0) call print_all_parameters(lo,rfcond)
           !write(string,'(a6,f8.3,a9,f8.3,7x,a8,i4)') ' Rp = ',rinfo%rp,' - Rwp = ',rinfo%rwp,'Cycle # ',ncytot
           !call scrper(string,nclev,rfcond%maxcy)
           call progress_bar_update(cval=nclev,fval=rfcond%maxcy)
!
!          Output grafico su viewer secondario
           if (index(rfcond%sprint,'W') > 0) then
!           
!              Aggiorna sempre punti da plottare
               call gr_add_point_plot(grplot(1),real(ncytot),rinfo%rwp)
!       
!              Se il viewer è aperto esegui plot
               if (viewerOpened == 1) then
                   call gr_select_viewer(1)
                   call gr_clear_area(0.0,1.0,0.0,1.0)
                   call gr_area(0.08, 0.99, 0.10, 0.99)
                   call gr_draw_graph(grgraph,grplot)
                   call zflush()
                   call gr_select_viewer(0)
               endif
           endif
           call vedinew()
           if (nclev == rfcond%maxcy) iflag = -1
       endif
       chisquared_old = chisquarednow

     case (1)                  ! calcola le funzioni in x
       call set_parameter(npar,x)
       call calcola_profilo(npar,ierp)
       fvec = funcls
       xparold = xpar

     case (2)                  ! calcola lo jacobiano
       call rietjacobian(ierp)
       fjac = jacob

   end select
!
   end subroutine frietlm
!-----------------------------------------------------------------
   subroutine set_parameter(Nact,parx)
   USE RefinecomRef
   USE Conteggi
   USE Counts
   USE background
   USE unit_cell
   USE variables, only:ref,atm
   USE General, only:cell,alambda,spacegr,celem,neutro,nsyn
   USE RRESTR
   USE genlsq, only:shiftx,step_length
   USE RIETV
   USE atom_type_util
   USE reflection_type_util
   USE constraints
   USE unit_cell
   USE profile_function
   USE spginfom
   USE type_constants, only: DP
!
   implicit none
   integer              , intent(in) :: Nact
   real(DP), dimension(Nact), intent(in) :: parx
   integer                           :: j
   integer                           :: i
   integer                           :: nkref
   integer                           :: icalcback = 0
   integer                           :: icalcFc = 0
   integer                           :: icalcrapI = 0
   integer                           :: kk
   integer                           :: katom,kparam
   integer                           :: kp
   real                              :: coefc
   integer                           :: kcon
   type(gref_type), allocatable, dimension(:) :: gref
   integer                                    :: ng
   integer                                    :: code
   integer                                    :: npar
   real                                       :: sumocc
   !real :: thmin0, thmax0
!
   nkref = 0
!
!  Fo se pawley
   if (rfcond%raction == 2) then
              call make_ref_groups(ref,gref,sogi=0.3)
       do i=1,rfcond%lim%nrefu
          if (fopar(i)%rcod > 0) then
              nkref = nkref + 1
              fopar(i)%rcod = nkref
              !fopar(i)%val = parx(nkref)
              if (parx(nkref) < 0.000) then
                  ng = number_of_group(i,gref)
                  if (gref(ng)%code == 1) then    ! riflesso singolo
                      fopar(i)%val = 0.000
                  else                            ! media delle intensita
                      fopar(i)%val = sum(ref(gref(ng)%vref)%fo) / gref(ng)%code
                  endif
              else
                  fopar(i)%val = sqrt(parx(nkref))
              endif
              ref(i)%fo = fopar(i)%val
          endif
       enddo
       icalcRapI = 1
   endif
!
!  Zero
   if (zero%rcod > 0) then
       nkref = nkref + 1
       zero%val = parx(nkref)
       !ttetaczero = ttetacount - 2.0 * zero%val
       !icalcback = 1          ! se affino lo zero ricalcola il background
   endif
!
!  Sample displacement
   if (sdisp%rcod > 0) then
       nkref = nkref + 1
       sdisp%val = parx(nkref)
   endif
 !
!  Sample transparency
   if (stran%rcod > 0) then
       nkref = nkref + 1
       stran%val = parx(nkref)
   endif   
!
!  Apply shift !!!!TODO: better to move after
   if (zero%rcod > 0 .or. sdisp%rcod > 0 .or. stran%rcod > 0) then
       ttetaczero = ttetacount - 2.0 * zero%val + sample_displacement(ttetaczero,sdisp%val,diffr_geom)   &
                                                + sample_transparency(ttetaczero,stran%val,diffr_geom)

       icalcback = 1          ! se affino lo zero ricalcola il background
   endif
!
!  Background
   if (coefbkg(1)%rcod > 0) then
       do i=1,backriet%ncoef
          nkref = nkref + 1
          coefbkg(i)%val = parx(nkref)
       enddo
       icalcback = 1
   endif
!
!  Cell parameters
   if(rinfo%refine_cell) then
      do j=1,6
         if (cellph%get_cod(j) > 0) then
             nkref = nkref + 1
             cell(j) = parx(nkref)
         endif
      enddo
      select case (spacegr%csys_code)
         case (CS_Tetragonal, CS_Hexagonal)  ! a=b/=c
          cell(2) = cell(1)

         case (CS_Trigonal)         
          if (index(spacegr%symbol_xhm,':R') > 0) then   ! Rhombohedral setting: a=b=c,al=bet=gam/=90
              cell(2:3) = cell(1)
              cell(5:6) = cell(4)
          else
              cell(2) = cell(1)
          endif

         case (CS_Cubic)                     ! a=b=c
          cell(2:3) = cell(1)

      end select
      call cellph%set(cell)
!
!     rigenera i 2theta 
      call calculate_2theta(ref,cell,alambda)
!
!     ricalcola le correzioni
      ref(:)%lp = LPcorrection(ref%tthd,neutro,nsyn)
      if (kopcorr == 1) call calcola_correzioneOP(ref(:rfcond%lim%nrefu),gpo%val,hklpo,set_cell_type(cell))
!
      icalcFc = 1
      icalcrapI = 1
   endif
!
!  Profilo
   select case(rfcond%typefun)
     case (PEARSON7)
       if (pear%u%rcod > 0) then
           nkref = nkref + 1
           pear%u%val = parx(nkref)
       endif
       if (pear%v%rcod > 0) then
           nkref = nkref + 1
           pear%v%val = parx(nkref)
       endif
       if (pear%w%rcod > 0) then
           nkref = nkref + 1
           pear%w%val = parx(nkref)
       endif
       if (pear%b0%rcod > 0) then
           nkref = nkref + 1
           pear%b0%val = parx(nkref)
       endif
       if (pear%b1%rcod > 0) then
           nkref = nkref + 1
           pear%b1%val = parx(nkref)
       endif
       if (pear%b2%rcod > 0) then
           nkref = nkref + 1
           pear%b2%val = parx(nkref)
       endif
       if (pear%asym%rcod > 0) then
           nkref = nkref + 1
           pear%asym%val =  parx(nkref)
       endif

     case (PVOIG)
       if (pvoi%u%rcod > 0) then
           nkref = nkref + 1
           pvoi%u%val = parx(nkref)
       endif
       if (pvoi%v%rcod > 0) then
           nkref = nkref + 1
           pvoi%v%val = parx(nkref)
       endif
       if (pvoi%w%rcod > 0) then
           nkref = nkref + 1
           pvoi%w%val = parx(nkref)
       endif
       if (pvoi%eta0%rcod > 0) then
           nkref = nkref + 1
           pvoi%eta0%val = parx(nkref)
       endif
       if (pvoi%eta1%rcod > 0) then
           nkref = nkref + 1
           pvoi%eta1%val = parx(nkref)
       endif
       if (pvoi%eta2%rcod > 0) then
           nkref = nkref + 1
           pvoi%eta2%val = parx(nkref)
       endif
       if (pvoi%asym%rcod > 0) then
           nkref = nkref + 1
           pvoi%asym%val =  parx(nkref)
       endif

     case (TCHZ)
       if (tch%u%rcod > 0) then
           nkref = nkref + 1
           tch%u%val = parx(nkref)
       endif
       if (tch%v%rcod > 0) then
           nkref = nkref + 1
           tch%v%val = parx(nkref)
       endif
       if (tch%w%rcod > 0) then
           nkref = nkref + 1
           tch%w%val = parx(nkref)
       endif
       if (tch%z%rcod > 0) then
           nkref = nkref + 1
           tch%z%val = parx(nkref)
       endif
       if (tch%x%rcod > 0) then
           nkref = nkref + 1
           tch%x%val = parx(nkref)
       endif
       if (tch%y%rcod > 0) then
           nkref = nkref + 1
           tch%y%val = parx(nkref)
       endif
       if (tch%asym%rcod > 0) then
           nkref = nkref + 1
           tch%asym%val =  parx(nkref)
       endif
   end select

!
!  Scala
   if (scalr%rcod > 0) then
       nkref = nkref + 1
       scalr%val= parx(nkref)
   endif
!
!  Orientazione preferenziale
   if (gpo%rcod > 0) then
       nkref = nkref + 1
       gpo%val = parx(nkref)
       call calcola_correzioneOP(ref(:rfcond%lim%nrefu),gpo%val,hklpo,set_cell_type(cell))
       icalcrapI = 1
   endif
!
!  Struttura
   if (rinfo%refine_structure) then
       do j=1,numatomtype(atm)
          do kk=1,3 
             if (atm(j)%rcod(kk) > 0) then
                nkref = nkref + 1
                atm(j)%xc(kk) = parx(nkref)
             endif
          enddo
          if (atm(j)%rcod(4) > 0) then
              nkref = nkref + 1
              atm(j)%biso = parx(nkref)
          endif
          if (atm(j)%rcod(5) > 0) then
              nkref = nkref + 1
              atm(j)%och = parx(nkref)
          endif
        enddo
!
       do kcon=1,nconstraints(constrv)
          if (constrv(kcon)%ref > 0) then
              code = constrv(kcon)%code
                     !write(6,*)'XPAR=',parx(nkref),shiftx(nkref)
              select case(code)
                 case (CONSTR_B)   ! constraints on b
                   nkref = nkref + 1
                   do kp = 1,constrv(kcon)%npar
                      katom = constrv(kcon)%vat(kp)
                      coefc = constrv(kcon)%coef(kp)
                      atm(katom)%biso =  atm_sav(katom)%biso + coefc*shiftx(nkref)*step_length
                   enddo

                  case (CONSTR_CO)  ! constraints on coord.
                   nkref = nkref + 1
                   do kp = 1,constrv(kcon)%npar
                      katom = constrv(kcon)%vat(kp)
                      kparam = constrv(kcon)%vpar(kp)
                      coefc = constrv(kcon)%coef(kp)
                      atm(katom)%xc(kparam) = atm_sav(katom)%xc(kparam) + coefc*shiftx(nkref)*step_length
                   enddo

                  case (CONSTR_SOF)  ! constraints on sof
                   npar = constrv(kcon)%npar
                   sumocc = 0.0
                   do i=1,npar-1  ! loop sui par. indipendenti
                      nkref = nkref + 1
                      katom = constrv(kcon)%vat(i)
                      !atm(katom)%och = atm_sav(katom)%och + shiftx(nkref)*step_length
                      !   write(0,*)'OCC=',atm(katom)%och,' XPAR=',parx(nkref)
                      atm(katom)%och = parx(nkref)
                      sumocc = sumocc + atm(katom)%och
                   enddo
!corr                   coefc = 1.0  ! va definito dall'utente in una futura implementazione
                   katom = constrv(kcon)%vat(npar)   ! parametro dipendente
                      !write(0,*)'CONSTR_OCC=',sumocc,constrv(kcon)%sumc
                   atm(katom)%och = constrv(kcon)%sumc - sumocc
                   
              end select
          endif
       enddo
       icalcFc = 1
       icalcrapI = 1
       call fillatom(atm)
   endif
!corr   if (icalcFc == 1 .and. rfcond%raction == 0) call fcalcan(ref,atm)
   if (icalcFc == 1 .and. rfcond%raction == 0) call sfcalc(ref,atm,spacegr,celem)
   if (icalcrapI == 1) call setRInt(ref)
!
!corr   if (icalcback == 1) call getbackground(rfcond%typeback,backgr(:rfcond%lim%ncu),ttetaczero,coefbkg(:rfcond%nback)%val, &
   !if (icalcback == 1) call getbackground(backriet%btype,backgr(:rfcond%lim%ncu),ttetaczero,coefbkg(:backriet%ncoef)%val, &
   !                                       rfcond%lim%ttmin-2.0*zero%val,rfcond%lim%ttmax-2.0*zero%val,thminbkg)
   if (icalcback == 1) then
       !thmin0 = ttetaczero(1)
       !thmax0 = ttetaczero(rfcond%lim%ncu)
       call getbackground(backriet%btype,backgr(:rfcond%lim%ncu),ttetaczero,coefbkg(:backriet%ncoef)%val,thminbkg)
   endif
!
   end subroutine set_parameter
! ----------------------------------------------------------------------
   subroutine Count0
   USE RefinecomRef
   USE Counts
!corr   USE RIETV
   USE variables, only: ref
   USE calculate_pdp
!
   implicit none
   real    :: sumoss
   real    :: sumcalc
   real    :: scal
   integer :: ierp
!
!  calcolo profilo
   scalr%val = 1.0
   call calculate_powder_pattern(rfcond,ttetaczero,ycounttot,ref,scalr%val,kalphar,0.0,pear,pvoi,tch,ierp)
!
!  Riscalamento su profilo osservato
   sumoss = sum (ycountoss(:rfcond%lim%ncu) - backgr(:rfcond%lim%ncu))
   sumcalc = sum (ycounttot(:rfcond%lim%ncu))
   if (sumcalc > 0.0) then
       scal = sumoss/sumcalc
   else
       scal = 1.0
   endif
   scalr%val = scal
   ycounttot = ycounttot * scalr%val
!
   call erreacc(0)
!
   end subroutine Count0
!----------------------------------------------------------------------------------
   subroutine calcgradn(np,x,epsfcn)
   USE RefinecomRef
   USE nr
   USE PatternRef, only : iriet
   USE GENLSQ
   !USE nrtype
   USE type_constants
   implicit none
        INTERFACE
                FUNCTION func(x)
                USE nrtype; USE nrutil
                IMPLICIT NONE
                REAL(SP), DIMENSION(:), INTENT(IN) :: x
                REAL(SP), DIMENSION(size(x)) :: func
                END FUNCTION func
        END INTERFACE
!
   integer, intent(in) :: np
   real(DP), intent(in)    :: epsfcn
   real(DP), dimension(np)  :: x
   real                :: temp
   real(DP)       :: h
   integer             :: i
   integer, parameter  :: npoin=80
   integer(i4b), parameter  :: ncof=20  !!!!15
   real, dimension(ncof)  :: parceb
   real(sp) :: aa, bb
   integer :: ierp
!
   chisquared_old = chisquarednow
   nparfun = np
   write(0,'(/,a21,i3,2x,a6,e15.7)')'Gradiente al ciclo n.',ncytot,'chi2 =',chisquared_old
   write(0,*)'   Parametro   chi2 stimato       der. numerica      der. analitica'
   do i=1, np
      temp = x(i)
      h= abs(sqrt(epsfcn)*max(temp,1.0))
      ippar = i
!
      aa = temp-npoin*h
      bb = temp+npoin*h
      parceb = chebft(aa,bb,ncof,func)
      write(0,'(1x,a13,3(e15.7,4x))')par_string(i),chebev(aa,bb,parceb,temp), &
                                     chebev(aa,bb,chder(aa,bb,parceb),temp),grad(i)
      x(i) = temp
   enddo
   chisquarednow = chisquared_old
   call set_parameter(np,x)
   call calcola_profilo(np,ierp)
! scommenta anche in func per affinamento su F
!!! call calcerrefs(npref,kcscal,scalr%val)
!
   end subroutine calcgradn
!----------------------------------------------------------------------------------
   function func(x)
   USE RefinecomRef
   USE nrtype
   USE GENLSQ
!!!!corr   USE PatternRef, only : iriet
!
   implicit none
   real(sp), dimension(:), intent(in) :: x
   real(sp), dimension(size(x)) :: func
   integer :: i
   integer :: nx
   integer :: ierp
!
   nx = size(x)
   do i=1,nx
      xpar(ippar) = x(i)
      call set_parameter(nparfun,xpar)
      call calcola_profilo(nparfun,ierp)
! scommenta anche in calcgradn per affinamento su F
! call calcerrefs(npref,kcscal,scalr%val)
      func(i) = chisquarednow
   !  write(0,*)xpar(ippar),chisquarednow
   enddo
!
   end function func
!----------------------------------------------------------------------------------
!corr   subroutine check_shift()
!corr   USE RefinecomRef
!corr!
!corr   implicit none
!corr!
!corr   call check_bt()
!corr!
!corr   end subroutine check_shift
!----------------------------------------------------------------------------------
!corr   subroutine check_bt()
!corr   USE RefinecomRef
!corr   USE GENLSQ
!corr!
!corr   implicit none
!corr   real :: btmax,btmin
!corr!!!!   integer :: nparb
!corr!!!!   real               :: maxb,minb
!corr!
!corr   btmax = 100.0
!corr   btmin = -100.0
!corr!!!!   nparb = checkbt(1)
!corr   !!! prova 1
!corr!!!!!   if (nparb > 0) then
!corr!!!!!       maxb = maxval( xparold(checkbt(2:nparb+1))+shiftx(checkbt(2:nparb+1)) )
!corr!!!!!       minb = minval( xparold(checkbt(2:nparb+1))+shiftx(checkbt(2:nparb+1)) )
!corr!!!!!       if (maxb > btmax .or. minb < btmin) then
!corr!!!!!           shiftx = shiftx * btmax/maxb
!corr!!!!!            write(6,*)'SHIFT RIDOTTO DI ',btmax/maxb,maxb
!corr!!!!!           !write(0,*)'SHIFT RIDOTTO DI ',btmax/maxb,maxb
!corr!!!!!       endif
!corr!!!!!   endif
!corr   !!! prova 2
!corr!!!!!     ierprof = 0
!corr!!!!!     if (nparb > 0) then
!corr!!!!!         if (any(xparold(checkbt(2:nparb+1))+shiftx(checkbt(2:nparb+1)) < 0.0)) ierprof = 1
!corr!!!!!     endif
!corr   !!! prova 3
!corr!!!    if (nparb > 0) then
!corr!!!        where(xparold(checkbt(2:nparb+1))+shiftx(checkbt(2:nparb+1)) < 0) shiftx(checkbt(2:nparb+1)) = 0.0
!corr!!!    endif
!corr!
!corr   end subroutine check_bt
!----------------------------------------------------------------------------------
   FUNCTION func1(x)
   USE nrtype
   USE GENLSQ
   USE RefinecomRef
   IMPLICIT NONE
   REAL(SP), INTENT(IN) :: x
   REAL(SP) :: func1
   integer :: ierp
!
   xpar(ippar) = x
   call set_parameter(nparfun,xpar)
   call calcola_profilo(nparfun,ierp)
   func1 = chisquarednow
!
   END FUNCTION func1
!----------------------------------------------------------------------------------
   subroutine calcgrad2(np,x,h)
   USE nr
   USE RefinecomRef
   USE GENLSQ
   USE type_constants, only: DP
!
   implicit none
        INTERFACE
                FUNCTION func1(x)
                USE nrtype
                IMPLICIT NONE
                REAL(SP), INTENT(IN) :: x
                REAL(SP) :: func1
                END FUNCTION func1
        END INTERFACE
   integer, intent(in) :: np
   real, intent(in)    :: h
   real(DP), dimension(np) :: x
   integer             :: i
   real                :: dx
   real                :: err
   integer             :: ierp
!
   chisquared_old = chisquarednow
   nparfun = np
   write(0,*)'Gradiente al ciclo n.',ncytot,'eps =',h
   write(0,*)'   Parametro           der. numerica        der. analitica'
   do i=1, np
      ippar = i
      dx = dfridr(func1,real(x(i)),h,err)
      write(0,'(4x,a13,3e15.7)')par_string(i),dx,-2.0*grad(i),err
   enddo
   chisquarednow = chisquared_old
   call set_parameter(np,x)
   call calcola_profilo(np,ierp)
!
   end subroutine calcgrad2
!----------------------------------------------------------------------------------
!!!!!   subroutine calcdern(np,h)
!!!!!   USE nr
!!!!!   USE RefinecomRef
!!!!!   USE GENLSQ
!!!!!!
!!!!!   implicit none
!!!!!        INTERFACE
!!!!!                FUNCTION func2(x)
!!!!!                USE nrtype
!!!!!                IMPLICIT NONE
!!!!!                REAL(SP), INTENT(IN) :: x
!!!!!                REAL(SP) :: func2
!!!!!                END FUNCTION func2
!!!!!        END INTERFACE
!!!!!   integer :: i
!!!!!   real, intent(in)    :: h
!!!!!   real                :: err
!!!!!   integer, intent(in) :: np
!!!!!   real                :: dx
!!!!!!
!!!!!   do i=1,20
!!!!!      ippar = i
!!!!!      dx = dfridr(func2,real(xpar(np)),h,err)
!!!!!      write(0,'(4x,i3,3e15.7)')i,dx,derxph(i,1,1)*0.5/refph(1,i)%fc,err
!!!!!   enddo
!!!!!!
!!!!!   end subroutine calcdern
!----------------------------------------------------------------------------------
   function func2(x)
   USE nrtype
   USE GENLSQ
   USE RefinecomRef
   USE variables, only:ref
   IMPLICIT NONE
   REAL(SP), INTENT(IN) :: x
   REAL(SP) :: func2
!
   xpar(1) = x
   call set_parameter(1,xpar)
   func2 = ref(ippar)%fc
!
   end function func2
!----------------------------------------------------------------------------------
   subroutine controlla2theta()
   USE RefinecomRef
  ! USE General, only: nrefl
   USE variables, only: ref
!
   implicit none
   integer :: i
!
   do i=2,size(ref)
      if (ref(i)%tthd < ref(i-1)%tthd) then
          write(0,*) 'WARN THETA',i,ref(i)%tthd,i-1,ref(i-1)%tthd
      endif
   enddo
!
   end subroutine controlla2theta

 !-----------------------------------------------------------------------

   subroutine Count2()
   USE RefinecomRef
   USE Counts
   USE background
   USE patternref, only:thmin,thmax
   USE variables, only: ref
   USE calculate_pdp
!
   implicit none
   integer :: posini,posfin
   integer :: ierp
!
   posini = 1
   posfin = rfcond%lim%ncu  
   !call getbackground(backriet%btype,backgr(:posfin-posini+1),ttetaczero(posini:posfin),   &
   !     coefbkg(:backriet%ncoef)%val,thmin-2.0*zero%val,thmax-2.0*zero%val,thminbkg)
   call getbackground(backriet%btype,backgr(:posfin-posini+1),ttetaczero(posini:posfin),coefbkg(:backriet%ncoef)%val,thminbkg)
   call calculate_powder_pattern(rfcond,ttetaczero,ycounttot,ref,scalr%val,kalphar,0.0,pear,pvoi,tch,ierp)
!
   call erreacc(0)
!
   end subroutine Count2

!---------------------------------------------------------------------------------------------   

   subroutine ParProf_daextra()          !!!!da sistemare
   USE RefinecomRef
   USE variables, only:ref
!
   common /direx/ncycle,icycle,nosing,izero,zeroprof,asym,scal,frac,noint,ialt,sigma,if92,nocell
   integer :: ncycle,icycle,nosing,izero,noint,ialt,if92,nocell
   real    :: zeroprof,asym,scal,sigma
   real    :: frac
!
   ref(:)%pk = frac                         ! range di picco da extra
   pear%w%val = ref(1)%fwhm**2         ! calcolo w dal primo riflesso
   pvoi%w%val = ref(1)%fwhm**2         ! calcolo w dal primo riflesso
   tch%w%val = ref(1)%fwhm**2
!
   end subroutine ParProf_daextra
            
            
!--------------------------------------------------------------------------------

  subroutine get_constraints() 
!
!  passa al c i constraints
!
  USE RRESTR
  USE RefinecomRef, only: constrv
  USE constraints
  implicit none
  integer ::   i
  integer :: nconstr
!
  nconstr = nconstraints(constrv)
  call passa_num_constraints(nconstr)
  do i=1, nconstr
     call passa_constr(constrv(i)%code, constrv(i)%npar, constrv(i)%ref,   &
                  constrv(i)%vpar,constrv(i)%vat,constrv(i)%coef,constrv(i)%sumc,i,constrv(i)%typec)
  end do
!
  end subroutine get_constraints
            
!--------------------------------------------------------------------------------

  subroutine put_num_constraints(knconstr) 
!
!  prende dal c il numero di constraints
!
  USE constraints
  USE RefinecomRef, only: constrv
  implicit none
  integer::        knconstr
!
! rialloca constrv
   call resize_constraints(constrv,knconstr)
!
   end subroutine put_num_constraints
            
!--------------------------------------------------------------------------------

  subroutine put_constr(code, npar, ref, vpar, vat, coef, sumc, j, typec)
!
!  prende dal C il j-esimo constraint
!
  USE constraints
  USE RefinecomRef, only: constrv
  implicit none
  integer, intent(in)                  :: j
  integer, intent(in)                  :: ref
  integer, intent(in)                  :: code 
  integer, intent(in)                  :: npar
  integer, dimension(npar), intent(in) :: vpar
  integer, dimension(npar), intent(in) :: vat
  real, dimension(npar), intent(in)    :: coef
  real, intent(in)                     :: sumc
  integer, intent(in)                  :: typec
!
  constrv(j+1) = constraint_type(code,vpar,vat,coef,sumc,npar,1,ref,typec)
!
  end subroutine put_constr
            
!--------------------------------------------------------------------------------

   subroutine get_info(rinfo_var)
! 
!  Prende dal C il j-esimo constraint
! 
   USE RefinecomRef
   USE progtype, only: refine_info_type
! 
   implicit none
   type(refine_info_type), intent(out) :: rinfo_var
! 
   rinfo_var = rinfo
! 
   end subroutine get_info
            
!--------------------------------------------------------------------------------

   subroutine genera_lista_restraints(code)
   USE rrestr
   USE variables, only: atm, lconn
   USE rietv, only: resvr
   USE unit_cell
   USE General, only: cell
   USE errormod
   USE OBmodule
   USE atom_type_util
   USE arrayutil
   implicit none
   type(cell_type) :: cellt
   integer, intent(in) :: code
   type(restraint_type), dimension(:), allocatable :: reslist
   integer, dimension(:), allocatable              :: vetres
   integer                                         :: i
   integer                                         :: nrl
   integer                                         :: pos
   integer, parameter :: BOND_TAB=1,CURRENT=2,MOL_MEC=3
   integer :: gen_mode = 1
   type(atom_type),  dimension(:), allocatable :: atom
   type(error_type) :: err
   integer :: nuser
!
   cellt = set_cell_type(cell)
   call copia_atoms(atom,atm)
   if (gen_mode == MOL_MEC) then
       call OBOptimizeGeometry(atom, lconn, cellt, err)
   endif
!
   call conn_to_restraints(lconn,atom,cellt,code,reslist,gen_mode)
!
   if (gen_mode == MOL_MEC) then
!
!      set current value from non-optimized model
       do i=1,nrestraints(reslist)
          call set_value_for_restraint(reslist(i),atm,cellt)
       enddo
   endif
!
!  Genera vettore vetres che dice quali checks sono attivi
   nrl = nrestraints(reslist)
          !call stampa_restraints(0,reslist,atm,cellt)
   if (nrl > 0) then
       allocate(vetres(nrl))
       vetres(:) = 0
!
!      Now update array resvr containing only the active restraints
       if (nrestraints(resvr) > 0) then
           do i=1,nrestraints(resvr)
              if (resvr(i)%code /= code) cycle
              pos = restraint_position(reslist,resvr(i)) 
              if (pos > 0) then
                  vetres(pos) = 1
                  reslist(pos) = resvr(i)      ! usa target e esd del restraint in memoria
              else
!!!!TODO: test plane
                  if (code == RESPLANE) then   ! check if restraints is compatible with num. of atoms
                      if (any(resvr(i)%na > numatomtype(atom))) then
                          resvr(i)%code = DELETED_RES  ! mark as deleted
                      endif
                  else
                      resvr(i)%code = DELETED_RES  ! mark as deleted
                  endif
              endif
           enddo
           call delete_restraints_code(resvr)
       endif
   endif
!
!  For plane set by user to_genera is called from C
   nuser = 0
   do i=1,nrestraints(resvr)
      if (resvr(i)%code == RESPLANE) then
          nuser = nuser + 1
          call add_restraint_to_list(reslist,resvr(i))
      endif
   enddo
   if (nuser > 0) then
       nrl = nrestraints(reslist)
       call resize_array(vetres,nrl)
       vetres(nrl-nuser+1:nrl) = 1  ! user restraints are active
                                    ! TODO: user restraints desappear if deactivated
   endif
!
!  aggiunte da mr 240709
   call c_deallocate_res()
   call passa_nres(nrl)
   if (nrl > 0) then
       call passa_vetres(nrl, vetres)
       do i=1, nrl
          call put_info_newrest(i-1,reslist(i)%code,reslist(i)%targ,reslist(i)%sigma,   &
                                reslist(i)%val,size(reslist(i)%na),reslist(i)%na)
       end do
   endif
!
   end subroutine genera_lista_restraints
!-----------------------------------------------------------------------
   subroutine init_newrestraints(cwqq)          !  aggiunta da mr 240709
   USE RRESTR
   real    :: cwqq(3)  
   integer :: i      
!      
   do i=1,3
      cwqq(i) = wqq(i)
   enddo
!
   return
   end subroutine init_newrestraints

!-----------------------------------------------------------------------

   subroutine get_restraints_from_c(active,code,val,targ,esd, nna, na) 
!
!  Function called by C every time that a restraint is selected
!
   USE rrestr
   implicit none
   integer, intent(in)              :: code
   integer, intent(in)              :: nna
   integer, intent(in)              :: na(*)
   integer, intent(in)              :: active
   type(restraint_type)             :: restr
   real, intent(in)                 :: val, targ, esd
!
   restr%code  = code
   restr%val   = val
   restr%targ  = targ
   restr%sigma = esd
   allocate(restr%na(nna))
   restr%na(1:nna) = na(1:nna)
   call get_restraints_from_graphic(active,restr)  ! update global variable resvr
!
   end subroutine get_restraints_from_c

!--------------------------------------------------------------

   subroutine get_restraints_from_graphic(code,restr)
   USE rietv, only: resvr
   USE rrestr
         !corr USE variables, only: atm
         !corr USE General, only: cell
         !corr USE unit_cell
   implicit none
   integer, intent(in)              :: code
   type(restraint_type), intent(in) :: restr
   integer                          :: pos
!
!
!  prendo la posizione del restraint restr nella lista resvr
   pos = restraint_position(resvr,restr) 
   if (code == 1) then   ! check attivo
       if (pos == 0) then
!
!          aggiungi il restraint se non esiste
           call add_restraint_to_list(resvr,restr)
       else
!
!          se esiste aggiorna esd e target
           if (restr%code < 3) then
               resvr(pos)%sigma = restr%sigma
               resvr(pos)%targ = restr%targ
           else
               resvr(pos)%sigma = restr%sigma
           endif
       endif

   else                  ! check disattivato
!
!      rimuovi il restraint dalla lista resvr
       if (pos > 0) call delete_restraints(resvr,pos)
   endif
       !call stampa_restraints(0,resvr,atm,set_cell_type(cell))
!
   end subroutine get_restraints_from_graphic

!-----------------------------------------------------------------------
   subroutine finishnewrestraints(iExitC, cwqq) !  aggiunta da mr 240709
   USE RRESTR
   USE rietv
   implicit none
   integer, intent(in)              :: iExitC
   integer                          :: i
   real, intent(in)                 :: cwqq(3)
!
!   iexitC = 0 (Cancel); = 1 (OK)
!
   if (iExitC.eq.1) then
       do i=1,3
          wqq(i) = cwqq(i)
          call set_weight(resvr,i,wqq(i))
       enddo
   endif
!
   return
   end subroutine finishnewrestraints
! ----------------------------------------------------------------------
   subroutine LeggiRestraints(nomefile)         !  aggiunta da mr 240709
   USE rrestr
   USE rietv
   implicit none
   character(len=*), intent(in)     :: nomefile
!
   call read_restraints_from_file(nomefile,resvr)
!
   return
   end subroutine LeggiRestraints
! ----------------------------------------------------------------------
   subroutine ScriviRestraints(nomefile, cwqq)   !  aggiunta da mr 240709
   USE rietv
   USE variables, only: atm
   USE rrestr
   implicit none
   character(len=*), intent(in)     :: nomefile
!corr   integer                          :: i
   real, intent(in)                 :: cwqq(3)
!
!corr   do i=1,3
!corr      wqq(i) = cwqq(i)
!corr   enddo
   call write_restraints_on_file(nomefile,resvr,atm,cwqq)
!
   return
   end subroutine ScriviRestraints
! ----------------------------------------------------------------------

   subroutine get_newrestraint_from_c(active,code,val,targ,esd, nna, na)
!  aggiunta da mr 250709
   USE rrestr
   USE rietv
   USE variables, only: atm
   USE general, only: cell
   USE unit_cell
   implicit none
   integer, intent(in)              :: code
   integer, intent(in)              :: nna
   integer, intent(in)              :: na(*)
   integer, intent(in)              :: active
   type(restraint_type)             :: restr
   real, intent(in)                 :: val, targ, esd
   integer                          :: j
   integer                          :: pos
!
   restr%code  = code
   restr%val   = val
   restr%targ  = targ
   restr%sigma = esd
   allocate(restr%na(nna))
   restr%na(1:nna) = na(1:nna)
   write(0,*)'code, nna, na ',restr%code, nna, (restr%na(j),j=1, nna)
!
!  aggiungi il restraint solo se non esiste già
   pos = restraint_position(resvr,restr) 
   if (pos == 0) then
       call set_value_for_restraint(restr,atm,set_cell_type(cell))
       call add_restraint_to_list(resvr,restr)
   endif
!
   end subroutine get_newrestraint_from_c

!-----------------------------------------------------------------------
   subroutine get_resttodel_from_c(ind_l,active,code,val,targ,esd, nna, na)
!  aggiunta da mr 250709
   USE rrestr
   USE rietv
   implicit none
   integer, intent(in)              :: ind_l
   integer, intent(in)              :: code
   integer, intent(in)              :: nna
   integer, intent(in)              :: na(*)
   integer, intent(in)              :: active
   real, intent(in)                 :: val, targ, esd
!!!!!   integer                          :: j
   type(restraint_type)             :: restr
   integer                          :: pos
!
!!!!!   write(0,*)'indl, code, nna, na ',ind_l, code, nna, (na(j),j=1, nna)
   restr%code  = code
   restr%val   = val
   restr%targ  = targ
   restr%sigma = esd
   allocate(restr%na(nna))
   restr%na(1:nna) = na(1:nna)
   pos = restraint_position(resvr,restr) 
   if (pos > 0) call delete_restraints(resvr,pos)
!
   end subroutine get_resttodel_from_c
!-----------------------------------------------------------------------
   subroutine PassaWeight(w, icode)
   USE RRESTR
!
   implicit none
   integer, intent(in)              :: icode
   real, intent(in)                 :: w
!
!  errore- se entri ed esci da rietveld dopo aver settato restraint su piano
!  arriva icode = 0. Da capire perche
!   if (icode > 0) then
       wqq(icode) = w
!   endif
!
   return
   end subroutine PassaWeight
! ----------------------------------------------------------------------

   integer function num_restraints(code)
   USE rietv, only: resvr
   USE rrestr
   integer, intent(in) :: code
   num_restraints = nrestraints(resvr,code) 
   end function num_restraints

! ----------------------------------------------------------------------

!   subroutine test_esd()
!   USE tempp
!   implicit none
!   real    :: std
!   integer :: i
!   character(len=100) :: string
!!
!        std = 17.456
!      string = string_esdt(std,std)
!      write(0,*)'std=',std,trim(string)
!        return
!   do i=1,100
!      call random_number(std)
!      std = std*100
!      string = string_esdt(std,std)
!      write(0,*)'std=',std,trim(string)
!   enddo
!
!   end subroutine test_esd
